import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from './_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import copyIcon from '@jetbrains/icons/copy';
import Avatar from '../avatar/avatar.js';
import Link from '../link/link.js';
import clipboard from '../clipboard/clipboard.js';
import Tag from '../tag/tag.js';
import Icon from '../icon/icon.js';
import { I18nContext } from '../i18n/i18n-context.js';
import Tooltip from '../tooltip/tooltip.js';
import { Size } from '../avatar/avatar-size.js';
import { Size as Size$1 } from '../icon/icon__constants.js';

var styles = {"userCardSpaced":"userCardSpaced_rui_6e59","userInformationContainer":"userInformationContainer_rui_6e59","userAvatar":"userAvatar_rui_6e59","userInformation":"userInformation_rui_6e59","userInformationGeneral":"userInformationGeneral_rui_6e59","userNameLine":"userNameLine_rui_6e59","userName":"userName_rui_6e59","userLogin":"userLogin_rui_6e59","userEmail":"userEmail_rui_6e59","userCopyIcon":"userCopyIcon_rui_6e59","userEmailWrapper":"userEmailWrapper_rui_6e59","unverifiedLabel":"unverifiedLabel_rui_6e59","userNameInfo":"userNameInfo_rui_6e59","banLabel":"banLabel_rui_6e59","userActiveStatus":"userActiveStatus_rui_6e59","online":"online_rui_6e59"};

const _excluded = ["children", "info", "className", "user", "avatarInfo"];
class UserCard extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "copyEmail", () => {
      var _translations$copiedT, _translations$copingT;
      const {
        translate
      } = this.context;
      const translations = this.props.translations;
      clipboard.copyText(this.props.user.email || '', (_translations$copiedT = translations === null || translations === void 0 ? void 0 : translations.copiedToClipboard) !== null && _translations$copiedT !== void 0 ? _translations$copiedT : translate('copyToClipboard'), (_translations$copingT = translations === null || translations === void 0 ? void 0 : translations.copingToClipboardError) !== null && _translations$copingT !== void 0 ? _translations$copingT : translate('copingToClipboardError'));
    });
  }
  render() {
    var _translations$online, _translations$offline, _translations$banned, _translations$unverif, _translations$copyToC;
    const _this$props = this.props,
      {
        children,
        info,
        className,
        user,
        avatarInfo
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const {
      translate
    } = this.context;
    const translations = this.props.translations;
    const classes = classNames(className, {});
    const userActiveStatusClasses = classNames(styles.userActiveStatus, user.online ? styles.online : '');
    return jsx("div", _objectSpread2(_objectSpread2({}, restProps), {}, {
      className: classes,
      children: jsxs("div", {
        className: styles.userInformationContainer,
        children: [jsxs("div", {
          className: styles.userAvatar,
          children: [jsx(Avatar, {
            size: Size.Size56,
            url: user.avatarUrl,
            username: user.name,
            round: true
          }), !!avatarInfo && avatarInfo]
        }), jsxs("div", {
          className: styles.userInformation,
          children: [jsxs("div", {
            className: styles.userInformationGeneral,
            children: [jsxs("div", {
              className: styles.userNameLine,
              children: [user.href && jsx(Link, {
                href: user.href,
                className: styles.userName,
                "data-test": "user-card-link",
                children: user.name
              }), !user.href && jsx("span", {
                className: styles.userName,
                children: user.name
              }), typeof user.online === 'boolean' && jsx("span", {
                className: userActiveStatusClasses,
                title: user.online ? (_translations$online = translations === null || translations === void 0 ? void 0 : translations.online) !== null && _translations$online !== void 0 ? _translations$online : translate('online') : (_translations$offline = translations === null || translations === void 0 ? void 0 : translations.offline) !== null && _translations$offline !== void 0 ? _translations$offline : translate('offline')
              }), !!info && jsx("span", {
                className: styles.userNameInfo,
                children: info
              }), user.banned && jsx(Tooltip, {
                title: user.banReason,
                children: jsx(Tag, {
                  className: styles.banLabel,
                  children: (_translations$banned = translations === null || translations === void 0 ? void 0 : translations.banned) !== null && _translations$banned !== void 0 ? _translations$banned : translate('banned')
                })
              })]
            }), jsx("div", {
              className: styles.userLogin,
              children: user.login
            }), user.email && jsxs("span", {
              className: styles.userEmailWrapper,
              children: [jsx(Link, {
                href: "mailto:".concat(user.email),
                title: "mailto:".concat(user.email),
                target: "_blank",
                className: styles.userEmail,
                children: user.email
              }), user.unverifiedEmail && jsx("span", {
                className: styles.unverifiedLabel,
                children: (_translations$unverif = translations === null || translations === void 0 ? void 0 : translations.unverified) !== null && _translations$unverif !== void 0 ? _translations$unverif : translate('unverified')
              }), jsx(Icon, {
                title: (_translations$copyToC = translations === null || translations === void 0 ? void 0 : translations.copyToClipboard) !== null && _translations$copyToC !== void 0 ? _translations$copyToC : translate('copyToClipboard'),
                className: styles.userCopyIcon,
                onClick: this.copyEmail,
                glyph: copyIcon,
                size: Size$1.Size14,
                suppressSizeWarning: true
              })]
            })]
          }), children]
        })]
      })
    }));
  }
}
_defineProperty(UserCard, "contextType", I18nContext);

export { UserCard as U, styles as s };
