import { a as _objectWithoutProperties, b as _objectSpread2 } from './_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import { jsx } from 'react/jsx-runtime';
import { memo } from 'react';
import classNames from 'classnames';
import memoize from '../global/memoize.js';

var styles = {"icon":"icon_rui_2d39","glyph":"glyph_rui_2d39","compatibilityMode":"compatibilityMode_rui_2d39","gray":"gray_rui_2d39","hover":"hover_rui_2d39","green":"green_rui_2d39","magenta":"magenta_rui_2d39","red":"red_rui_2d39","blue":"blue_rui_2d39","white":"white_rui_2d39","loading":"loading_rui_2d39","icon-loading":"icon-loading_rui_2d39"};

const _excluded = ["src", "className"];
function convertReactSVGDOMProperty(str) {
  return str.replace(/[-|:]([a-z])/g, g => g[1].toUpperCase());
}
function serializeAttrs(map) {
  const res = {};
  for (let i = 0; i < map.length; i++) {
    const key = map[i].name;
    let prop = key;
    if (key === 'class') {
      prop = 'className';
    } else if (!key.startsWith('data-')) {
      prop = convertReactSVGDOMProperty(key);
    }
    res[prop] = map[i].value;
  }
  return res;
}
function extractSVGProps(svgNode) {
  const map = svgNode.attributes;
  return map.length > 0 ? serializeAttrs(map) : null;
}
const getSVGFromSource = memoize(src => {
  const svgContainer = document.createElement('div');
  svgContainer.innerHTML = src;
  const svg = svgContainer.firstElementChild;
  if (svg.remove) {
    svg.remove();
  } else {
    svgContainer.removeChild(svg);
  }
  return {
    props: extractSVGProps(svg),
    html: svg.innerHTML
  };
});
function isCompatibilityMode(iconSrc) {
  const hasWidth = /width=".+"/gi.test(iconSrc);
  const hasHeight = /height=".+"/gi.test(iconSrc);
  return !hasWidth || !hasHeight;
}
function IconSVG(_ref) {
  let {
      src,
      className
    } = _ref,
    rest = _objectWithoutProperties(_ref, _excluded);
  const glyphClasses = classNames(styles.glyph, {
    [styles.compatibilityMode]: isCompatibilityMode(src)
  }, className);
  const {
    props,
    html
  } = getSVGFromSource(src);
  return jsx("svg", _objectSpread2(_objectSpread2(_objectSpread2({}, props), rest), {}, {
    className: glyphClasses,
    dangerouslySetInnerHTML: {
      __html: html
    }
  }));
}
var IconSVG$1 = /*#__PURE__*/memo(IconSVG);

export { IconSVG$1 as I, styles as s };
