import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from './_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import closeIcon from '@jetbrains/icons/close-12px';
import { Button } from '../button/button.js';
import getUID from '../global/get-uid.js';
import Icon from '../icon/icon.js';
import { I18nContext } from '../i18n/i18n-context.js';
import { createComposedRef } from '../global/composeRefs.js';
import { ControlsHeightContext } from '../global/controls-height.js';
import { ControlLabel } from '../control-label/control-label.js';
import ControlHelp from '../control-help/control-help.js';

var inputStyles = {"outerContainer":"outerContainer_rui_e356","borderless":"borderless_rui_e356","container":"container_rui_e356","input":"input_rui_e356","error":"error_rui_e356","withIcon":"withIcon_rui_e356","clearable":"clearable_rui_e356","icon":"icon_rui_e356","clear":"clear_rui_e356","empty":"empty_rui_e356","errorText":"errorText_rui_e356","helpText":"helpText_rui_e356","sizeS":"sizeS_rui_e356","sizeM":"sizeM_rui_e356","sizeL":"sizeL_rui_e356","sizeFULL":"sizeFULL_rui_e356","heightS":"heightS_rui_e356","heightM":"heightM_rui_e356","heightL":"heightL_rui_e356"};

const _excluded = ["size", "multiline", "borderless", "label", "labelType", "error", "help", "className", "inputClassName", "children", "value", "onClear", "disabled", "inputRef", "onChange", "enableShortcuts", "id", "placeholder", "icon", "translations", "height", "beforeInput", "afterInput", "autogrow"];
function noop() {}
/**
 * @name Input
 */
var Size;
(function (Size) {
  Size["AUTO"] = "Auto";
  Size["S"] = "S";
  Size["M"] = "M";
  Size["L"] = "L";
  Size["FULL"] = "FULL";
})(Size || (Size = {}));
class Input extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      empty: true
    });
    _defineProperty(this, "frame", void 0);
    _defineProperty(this, "input", void 0);
    _defineProperty(this, "id", getUID('ring-input-'));
    _defineProperty(this, "inputRef", el => {
      this.input = el;
    });
    _defineProperty(this, "composedInputRef", createComposedRef());
    _defineProperty(this, "clear", e => {
      var _this$props$onClear, _this$props;
      (_this$props$onClear = (_this$props = this.props).onClear) === null || _this$props$onClear === void 0 || _this$props$onClear.call(_this$props, e);
    });
    _defineProperty(this, "handleInputChange", e => {
      if (!this.props.multiline) {
        var _this$props$onChange, _this$props2;
        (_this$props$onChange = (_this$props2 = this.props).onChange) === null || _this$props$onChange === void 0 || _this$props$onChange.call(_this$props2, e);
        this.checkValue();
      }
    });
    _defineProperty(this, "handleTextareaChange", e => {
      if (this.props.multiline) {
        var _this$props$onChange2, _this$props3;
        (_this$props$onChange2 = (_this$props3 = this.props).onChange) === null || _this$props$onChange2 === void 0 || _this$props$onChange2.call(_this$props3, e);
        this.checkValue();
      }
    });
  }
  componentDidMount() {
    this.frame = requestAnimationFrame(() => this.adapt());
  }
  componentDidUpdate() {
    this.adapt();
  }
  componentWillUnmount() {
    if (this.frame != null) {
      cancelAnimationFrame(this.frame);
    }
  }
  getId() {
    return this.props.id || this.id;
  }
  checkValue() {
    var _this$input;
    this.setState({
      empty: !((_this$input = this.input) !== null && _this$input !== void 0 && _this$input.value)
    });
    if (this.props.multiline && this.props.autogrow && this.input != null && this.input.scrollHeight >= this.input.clientHeight) {
      this.stretch(this.input);
    }
  }
  stretch(el) {
    if (!el || !el.style) {
      return;
    }
    el.style.height = '0'; // To know the real scrollHeight
    el.style.height = "".concat(el.scrollHeight + 2, "px");
  }
  adapt() {
    this.checkValue();
  }
  render() {
    const _this$props4 = this.props,
      {
        // Modifiers
        size,
        multiline,
        borderless,
        // Props
        label,
        labelType,
        error,
        help,
        className,
        inputClassName,
        children,
        value,
        onClear,
        disabled,
        inputRef,
        onChange,
        enableShortcuts,
        id,
        placeholder,
        icon,
        translations,
        height = this.context,
        beforeInput,
        afterInput,
        autogrow
      } = _this$props4,
      restProps = _objectWithoutProperties(_this$props4, _excluded);
    const {
      empty
    } = this.state;
    const clearable = !!onClear;
    const classes = classNames(className, inputStyles.outerContainer, [inputStyles["size".concat(size)]], [inputStyles["height".concat(height)]], {
      'ring-js-shortcuts': enableShortcuts === true,
      [inputStyles.error]: error != null,
      [inputStyles.empty]: empty,
      [inputStyles.withIcon]: icon != null,
      [inputStyles.clearable]: clearable,
      [inputStyles.borderless]: borderless
    });
    const inputClasses = classNames(inputStyles.input, inputClassName);
    const text = value != null ? value : children;
    const commonProps = {
      ref: this.composedInputRef(this.inputRef, inputRef),
      className: inputClasses,
      value: text,
      disabled,
      id: this.getId(),
      placeholder,
      'aria-label': typeof label === 'string' && label ? label : placeholder,
      'data-enabled-shortcuts': Array.isArray(enableShortcuts) ? enableShortcuts.join(',') : null
    };
    return jsx(I18nContext.Consumer, {
      children: _ref => {
        var _translations$clear;
        let {
          translate
        } = _ref;
        return jsxs("div", {
          className: classes,
          "data-test": "ring-input",
          children: [label && jsx(ControlLabel, {
            htmlFor: this.getId(),
            disabled: disabled,
            type: labelType,
            children: label
          }), jsxs("div", {
            className: inputStyles.container,
            children: [icon && jsx(Icon, {
              glyph: icon,
              className: inputStyles.icon
            }), beforeInput, multiline ? jsx("textarea", _objectSpread2(_objectSpread2({
              onChange: this.handleTextareaChange,
              rows: 1
            }, commonProps), restProps)) : jsx("input", _objectSpread2(_objectSpread2({
              onChange: this.handleInputChange
            }, commonProps), restProps)), clearable && !disabled && jsx(Button, {
              title: (_translations$clear = translations === null || translations === void 0 ? void 0 : translations.clear) !== null && _translations$clear !== void 0 ? _translations$clear : translate('clear'),
              "data-test": "ring-input-clear",
              className: inputStyles.clear,
              icon: closeIcon,
              onClick: this.clear
            }), afterInput]
          }), error ? jsx("div", {
            className: inputStyles.errorText,
            children: error
          }) : help && jsx(ControlHelp, {
            className: inputStyles.helpText,
            children: help
          })]
        });
      }
    });
  }
}
_defineProperty(Input, "defaultProps", {
  size: Size.M,
  onChange: noop,
  inputRef: noop,
  enableShortcuts: ['esc'],
  autogrow: true
});
_defineProperty(Input, "contextType", ControlsHeightContext);

export { Input as I, Size as S, inputStyles as i };
