import 'core-js/modules/es.symbol.description.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import List from '../list/list.js';

var styles = {"queryAssist":"queryAssist_rui_d22e","error":"error_rui_d22e","queryAssistDisabled":"queryAssistDisabled_rui_d22e","huge":"huge_rui_d22e","actions":"actions_rui_d22e","input":"input_rui_d22e","letter-text":"letter-text_rui_d22e","letterDefault":"letterDefault_rui_d22e","letter-field-name":"letter-field-name_rui_d22e","letter-field-value":"letter-field-value_rui_d22e","letter-operator":"letter-operator_rui_d22e","letter-error":"letter-error_rui_d22e","highlight":"highlight_rui_d22e","service":"service_rui_d22e","placeholder":"placeholder_rui_d22e resetButton_rui_8bff","hugePlaceholder":"hugePlaceholder_rui_d22e","letter":"letter_rui_d22e","icon":"icon_rui_d22e","rightSearchButton":"rightSearchButton_rui_d22e","clear":"clear_rui_d22e","withoutGlass":"withoutGlass_rui_d22e","loaderActive":"loaderActive_rui_d22e","loaderOnTheRight":"loaderOnTheRight_rui_d22e","inputRevertOrder":"inputRevertOrder_rui_d22e"};

const ICON_ID_LENGTH = 44;
class QueryAssistSuggestions {
  /*
   * Pay attention that this method produces not a 100% unique key.
   * Consider to use a unique identifier provided by a server.
   */
  static createKey(suggestion) {
    const {
      description,
      group,
      icon,
      option,
      prefix = '',
      suffix = ''
    } = suggestion;
    return prefix + option + suffix + group + description + (icon ? icon.substring(icon.length - ICON_ID_LENGTH) : '');
  }
  static renderLabel(suggestion) {
    const {
      className,
      matchingStart = 0,
      matchingEnd = 0,
      option,
      prefix = '',
      suffix = ''
    } = suggestion;
    let wrappedOption;
    let before = '';
    let after = '';
    if (matchingStart !== matchingEnd) {
      before = option.substring(0, matchingStart);
      wrappedOption = jsx("span", {
        className: styles.highlight,
        children: option.substring(matchingStart, matchingEnd)
      });
      after = option.substring(matchingEnd);
    } else {
      wrappedOption = option;
    }
    const wrappedPrefix = prefix && jsx("span", {
      className: styles.service,
      children: prefix
    });
    const wrappedSuffix = suffix && jsx("span", {
      className: styles.service,
      children: suffix
    });
    return jsxs("span", {
      className: className,
      children: [wrappedPrefix, before, wrappedOption, after, wrappedSuffix]
    });
  }
  static renderGroupSeparator(suggestion, prevSuggestion) {
    const {
      group,
      option
    } = suggestion;
    const {
      SEPARATOR
    } = List.ListProps.Type;
    if (prevSuggestion !== group) {
      return {
        key: option + group + SEPARATOR,
        description: group,
        rgItemType: SEPARATOR
      };
    }
    return null;
  }
  static renderList(suggestions, suggestionRenderer) {
    const renderedSuggestions = [];
    suggestions.forEach((suggestion, index, arr) => {
      const prevSuggestionGroup = arr[index - 1] && arr[index - 1].group;
      const groupSeparator = QueryAssistSuggestions.renderGroupSeparator(suggestion, prevSuggestionGroup);
      if (groupSeparator) {
        renderedSuggestions.push(groupSeparator);
      }
      renderedSuggestions.push(suggestionRenderer(suggestion));
    });
    return renderedSuggestions;
  }
}

export { QueryAssistSuggestions as Q, styles as s };
