import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from './_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import { Waypoint } from 'react-waypoint';

var styles = {"extra-small-screen-media":"(max-width: 639px)","small-screen-media":"(min-width: 640px) and (max-width: 959px)","contentLayout":"contentLayout_rui_8468","contentLayoutContent":"contentLayoutContent_rui_8468","sidebarContainer":"sidebarContainer_rui_8468","sidebarContainerRight":"sidebarContainerRight_rui_8468","sidebar":"sidebar_rui_8468","sidebarRight":"sidebarRight_rui_8468","sidebarFixedTop":"sidebarFixedTop_rui_8468","sidebarFixedBottom":"sidebarFixedBottom_rui_8468","bottomMarker":"bottomMarker_rui_8468","contentLayoutResponsive":"contentLayoutResponsive_rui_8468"};

const _excluded = ["right", "children", "className", "containerClassName", "fixedClassName", "contentNode"];
const ABOVE = 'above';
const INSIDE = 'inside';
class Sidebar extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      topIsOutside: true,
      bottomIsOutside: true,
      sidebarVisibleHeight: undefined
    });
    _defineProperty(this, "sidebarNode", void 0);
    _defineProperty(this, "handleTopWaypoint", _ref => {
      let {
        currentPosition
      } = _ref;
      this.setState({
        topIsOutside: currentPosition === ABOVE
      });
    });
    _defineProperty(this, "handleBottomWaypoint", _ref2 => {
      let {
        currentPosition,
        waypointTop
      } = _ref2;
      this.setState({
        sidebarVisibleHeight: waypointTop,
        bottomIsOutside: currentPosition !== INSIDE
      });
    });
    _defineProperty(this, "sidebarRef", node => {
      this.sidebarNode = node;
    });
  }
  shouldUseFixation() {
    const {
      contentNode
    } = this.props;
    const {
      sidebarNode
    } = this;
    if (!contentNode || !sidebarNode) {
      return false;
    }
    return contentNode.offsetHeight >= sidebarNode.offsetHeight;
  }
  shouldFixateBottom() {
    const {
      topIsOutside,
      bottomIsOutside
    } = this.state;
    return !bottomIsOutside && topIsOutside && this.shouldUseFixation();
  }
  render() {
    const _this$props = this.props,
      {
        right,
        children,
        className,
        containerClassName,
        fixedClassName,
        contentNode
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const {
      topIsOutside,
      bottomIsOutside,
      sidebarVisibleHeight
    } = this.state;
    const shouldFixateTop = bottomIsOutside && topIsOutside && this.shouldUseFixation();
    const shouldFixateBottom = this.shouldFixateBottom();
    const containerClasses = classNames(styles.sidebarContainer, containerClassName, {
      [styles.sidebarContainerRight]: right
    });
    const classes = classNames(styles.sidebar, className, {
      [styles.sidebarRight]: right,
      [styles.sidebarFixedTop]: shouldFixateTop,
      [styles.sidebarFixedBottom]: shouldFixateBottom,
      [fixedClassName !== null && fixedClassName !== void 0 ? fixedClassName : '']: shouldFixateTop || shouldFixateBottom
    });
    const style = {
      maxHeight: shouldFixateBottom && sidebarVisibleHeight ? "".concat(sidebarVisibleHeight, "px") : undefined
    };
    return jsxs("aside", {
      className: containerClasses,
      ref: this.sidebarRef,
      children: [jsx(Waypoint, {
        onEnter: this.handleTopWaypoint,
        onLeave: this.handleTopWaypoint
      }), jsx("div", _objectSpread2(_objectSpread2({}, restProps), {}, {
        style: style,
        className: classes,
        children: children
      })), jsx("div", {
        className: styles.bottomMarker,
        children: jsx(Waypoint, {
          onEnter: this.handleBottomWaypoint,
          onLeave: this.handleBottomWaypoint
        })
      })]
    });
  }
}

export { Sidebar as S, styles as s };
