import { a as _objectWithoutProperties, b as _objectSpread2 } from './_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { createContext, forwardRef, useMemo, useEffect, useContext, useState } from 'react';
import classNames from 'classnames';
import { createPortal } from 'react-dom';
import { PopupTargetContext, PopupTarget } from '../popup/popup.target.js';
import { getPopupContainer } from '../popup/popup.js';
import getUID from '../global/get-uid.js';

var defaultStyles = {"light":"light_rui_6155"};

var darkStyles = {"dark":"dark_rui_eb55"};

const _excluded = ["theme", "className", "passToPopups", "children", "WrapperComponent", "target"];
var Theme;
(function (Theme) {
  Theme["AUTO"] = "auto";
  Theme["LIGHT"] = "light";
  Theme["DARK"] = "dark";
})(Theme || (Theme = {}));
const ThemeContext = /*#__PURE__*/createContext({
  theme: Theme.LIGHT
});
const GLOBAL_DARK_CLASS_NAME = 'ring-ui-theme-dark';
const darkMatcher = window.matchMedia('(prefers-color-scheme: dark)');
function useTheme() {
  const [dark, setDark] = useState(darkMatcher.matches);
  useEffect(() => {
    const onChange = e => setDark(e.matches);
    darkMatcher.addEventListener('change', onChange);
    return () => darkMatcher.removeEventListener('change', onChange);
  }, []);
  return dark ? Theme.DARK : Theme.LIGHT;
}
function useThemeClasses(theme) {
  const systemTheme = useTheme();
  const resolvedTheme = theme === Theme.AUTO ? systemTheme : theme;
  return classNames({
    [darkStyles.dark]: resolvedTheme === Theme.DARK,
    [GLOBAL_DARK_CLASS_NAME]: resolvedTheme === Theme.DARK,
    [defaultStyles.light]: resolvedTheme === Theme.LIGHT
  });
}
function WithThemeClasses(_ref) {
  let {
    theme,
    children
  } = _ref;
  const themeClasses = useThemeClasses(theme);
  return children(themeClasses);
}
function applyTheme(theme, container) {
  if (theme === Theme.DARK) {
    container.classList.remove(defaultStyles.light);
    container.classList.add(darkStyles.dark);
    container.classList.add(GLOBAL_DARK_CLASS_NAME);
  } else {
    container.classList.remove(darkStyles.dark);
    container.classList.remove(GLOBAL_DARK_CLASS_NAME);
    container.classList.add(defaultStyles.light);
  }
}
const DefaultWrapper = /*#__PURE__*/forwardRef(function Wrapper(props, ref) {
  return jsx("div", _objectSpread2(_objectSpread2({}, props), {}, {
    ref: ref
  }));
});
const ThemeProvider = /*#__PURE__*/forwardRef(function ThemeProvider(_ref2, ref) {
  let {
      theme = Theme.AUTO,
      className,
      passToPopups,
      children: _children,
      WrapperComponent = DefaultWrapper,
      target
    } = _ref2,
    restProps = _objectWithoutProperties(_ref2, _excluded);
  const systemTheme = useTheme();
  const resolvedTheme = theme === Theme.AUTO ? systemTheme : theme;
  const id = useMemo(() => getUID('popups-with-theme-'), []);
  const themeValue = useMemo(() => ({
    theme: resolvedTheme
  }), [resolvedTheme]);
  useEffect(() => {
    if (target != null) {
      applyTheme(resolvedTheme, target);
    }
  }, [resolvedTheme, target]);
  const themeClasses = useThemeClasses(theme);
  const parentTarget = useContext(PopupTargetContext);
  return jsx(ThemeContext.Provider, {
    value: themeValue,
    children: jsx(WrapperComponent, _objectSpread2(_objectSpread2({
      ref: ref,
      className: target != null ? undefined : classNames(className, themeClasses)
    }, restProps), {}, {
      children: passToPopups ? jsx(PopupTarget, {
        id: id,
        children: popupTarget => jsxs(Fragment, {
          children: [_children, /*#__PURE__*/createPortal(jsx("div", {
            className: themeClasses,
            children: popupTarget
          }), parentTarget && getPopupContainer(parentTarget) || document.body)]
        })
      }) : _children
    }))
  });
});
var Theme$1 = Theme;

export { GLOBAL_DARK_CLASS_NAME as G, Theme$1 as T, WithThemeClasses as W, ThemeProvider as a, ThemeContext as b, useThemeClasses as c, darkStyles as d, applyTheme as e, useTheme as u };
