import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { createRoot } from 'react-dom/client';
import getUID from '../global/get-uid.js';
import Alert, { ANIMATION_TIME } from '../alert/alert.js';
import Alerts from '../alert/container.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'react';
import 'classnames';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/data-tests.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import 'react-dom';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';

const _excluded = ["message", "key"],
  _excluded2 = ["onCloseRequest", "onClose"];
/**
 * @name Alert Service
 */
class AlertService {
  constructor() {
    _defineProperty(this, "defaultTimeout", 0);
    // This alerts are stored in inverse order (last shown is first in array)
    _defineProperty(this, "showingAlerts", []);
    _defineProperty(this, "containerElement", document.createElement('div'));
    _defineProperty(this, "reactRoot", createRoot(this.containerElement));
  }
  _getShowingAlerts() {
    return [...this.showingAlerts];
  }
  renderAlertContainer(alerts) {
    if (alerts.length === 0) {
      return jsx("span", {});
    }
    return jsx(Alerts, {
      children: alerts.map(alert => {
        const {
            message,
            key
          } = alert,
          rest = _objectWithoutProperties(alert, _excluded);
        return jsx(Alert, _objectSpread2(_objectSpread2({}, rest), {}, {
          children: message
        }), key);
      })
    });
  }
  /**
   * Renders alert container into virtual node to skip maintaining container
   */
  renderAlerts() {
    this.reactRoot.render(this.renderAlertContainer(this.showingAlerts));
  }
  findSameAlert(message, type) {
    return this.showingAlerts.filter(it => it.type === type && it.message === message)[0];
  }
  startAlertClosing(alert) {
    alert.isClosing = true;
    this.renderAlerts();
  }
  remove(key) {
    const alertToClose = this.showingAlerts.filter(alert => alert.key === key)[0];
    if (!alertToClose) {
      return;
    }
    this.startAlertClosing(alertToClose);
  }
  removeWithoutAnimation(key) {
    this.showingAlerts = this.showingAlerts.filter(alert => alert.key !== key);
    this.renderAlerts();
  }
  stopShakingWhenAnimationDone(shakingAlert) {
    setTimeout(() => {
      shakingAlert.showWithAnimation = false;
      shakingAlert.isShaking = false;
      this.renderAlerts();
    }, ANIMATION_TIME);
  }
  addAlert(message, type) {
    let timeout = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.defaultTimeout;
    let options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    const {
        onCloseRequest: _onCloseRequest,
        onClose: _onClose
      } = options,
      restOptions = _objectWithoutProperties(options, _excluded2);
    const sameAlert = this.findSameAlert(message, type);
    if (sameAlert) {
      sameAlert.isShaking = true;
      this.renderAlerts();
      this.stopShakingWhenAnimationDone(sameAlert);
      return sameAlert.key;
    }
    const alert = _objectSpread2({
      key: getUID('alert-service-'),
      message,
      type,
      timeout,
      isClosing: false,
      onCloseRequest: () => {
        _onCloseRequest === null || _onCloseRequest === void 0 || _onCloseRequest();
        this.startAlertClosing(alert);
      },
      onClose: () => {
        _onClose === null || _onClose === void 0 || _onClose();
        this.removeWithoutAnimation(alert.key);
      }
    }, restOptions);
    this.showingAlerts = [alert, ...this.showingAlerts];
    this.renderAlerts();
    return alert.key;
  }
  setDefaultTimeout(timeout) {
    this.defaultTimeout = timeout;
  }
  error(message, timeout) {
    return this.addAlert(message, Alert.Type.ERROR, timeout);
  }
  message(message, timeout) {
    return this.addAlert(message, Alert.Type.MESSAGE, timeout);
  }
  warning(message, timeout) {
    return this.addAlert(message, Alert.Type.WARNING, timeout);
  }
  successMessage(message, timeout) {
    return this.addAlert(message, Alert.Type.SUCCESS, timeout);
  }
  loadingMessage(message, timeout) {
    return this.addAlert(message, Alert.Type.LOADING, timeout);
  }
}
const alertService = new AlertService();

export { alertService as default };
