import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';

const DEFAULT_FLUSH_INTERVAL = 10000;
const DEFAULT_FLUSH_MAX_PACK_SIZE = 100;
class AnalyticsCustomPlugin {
  constructor(send, isDevelopment, flushInterval) {
    _defineProperty(this, "_data", void 0);
    _defineProperty(this, "_flush", void 0);
    _defineProperty(this, "_isDevelopment", void 0);
    _defineProperty(this, "_flushInterval", void 0);
    _defineProperty(this, "_flushMaxPackSize", void 0);
    _defineProperty(this, "_hasSendSchedule", void 0);
    this._data = [];
    this.config({
      send,
      isDevelopment,
      flushInterval
    });
  }
  config(config) {
    this._flush = () => {
      if (this._data.length > 0) {
        config.send(this._data);
        this._data = [];
      }
    };
    this._isDevelopment = config.isDevelopment;
    this._flushInterval = config.flushInterval || DEFAULT_FLUSH_INTERVAL;
    this._flushMaxPackSize = config.flushMaxPackSize || DEFAULT_FLUSH_MAX_PACK_SIZE;
  }
  trackEvent(category, action, additionalData) {
    this._processEvent(category, action, additionalData);
  }
  trackPageView(path, data) {
    this._processEvent('page', 'view', data);
  }
  _initSendSchedule() {
    window.addEventListener('beforeunload', () => {
      var _this$_flush;
      return (_this$_flush = this._flush) === null || _this$_flush === void 0 ? void 0 : _this$_flush.call(this);
    });
    if (this._flush != null) {
      setInterval(this._flush, this._flushInterval);
    }
    this._hasSendSchedule = true;
  }
  _processEvent(category, action, data) {
    if (!this._hasSendSchedule && this._flush) {
      this._initSendSchedule();
    }
    if (this._isDevelopment) {
      console.log('TRACKING DATA = ', category, action, data); // eslint-disable-line no-console
    }
    const baseSendingData = {
      category,
      action,
      timestamp: Date.now()
    };
    this._addDataToFlushingPack(data ? _objectSpread2(_objectSpread2({}, baseSendingData), {}, {
      data
    }) : baseSendingData);
  }
  _addDataToFlushingPack(sendingData) {
    this._data.push(sendingData);
    if (this._flushMaxPackSize != null && this._data.length >= this._flushMaxPackSize) {
      var _this$_flush2;
      (_this$_flush2 = this._flush) === null || _this$_flush2 === void 0 || _this$_flush2.call(this);
    }
  }
}

export { AnalyticsCustomPlugin as default };
