import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import '../_helpers/island.js';
import '../island/header.js';
import ContentWrapper from '../island/content.js';
import Dialog from '../dialog/dialog.js';
import { Button } from '../button/button.js';
import { H2 } from '../heading/heading.js';
import '../global/linear-function.js';
import '../island/adaptive-island-hoc.js';
import 'element-resize-detector';
import '../global/schedule-raf.js';
import 'react-dom';
import '@jetbrains/icons/close';
import '../island/island.js';
import '../global/data-tests.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../_helpers/dialog__body-scroll-preventer.js';
import 'scrollbar-width';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/heading.js';

var styles = {"dialog":"dialog_rui_159b","content":"content_rui_159b font_rui_8bff","button":"button_rui_159b","firstButton":"firstButton_rui_159b button_rui_159b","title":"title_rui_159b","logo":"logo_rui_159b","error":"error_rui_159b"};

class AuthDialog extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      retrying: false
    });
    _defineProperty(this, "onEscPress", () => {
      if (this.props.cancelOnEsc) {
        this.props.onCancel();
      }
    });
    _defineProperty(this, "onRetryPress", async () => {
      if (!this.props.onTryAgain || this.state.retrying) {
        return;
      }
      this.setState({
        retrying: true
      });
      try {
        await this.props.onTryAgain();
      } catch (e) {
        // do nothing, error is handled in onTryAgain
      } finally {
        this.setState({
          retrying: false
        });
      }
    });
  }
  componentDidMount() {
    window.addEventListener('online', this.onRetryPress);
  }
  componentWillUnmount() {
    window.removeEventListener('online', this.onRetryPress);
  }
  render() {
    const {
      show,
      className,
      errorMessage,
      serviceImage,
      serviceName,
      loginCaption,
      loginToCaption,
      confirmLabel,
      cancelLabel,
      tryAgainLabel,
      onConfirm,
      onCancel,
      onTryAgain
    } = this.props;
    const {
      retrying
    } = this.state;
    const defaultTitle = serviceName ? loginToCaption : loginCaption;
    const title = (this.props.title || defaultTitle).replace('%serviceName%', serviceName !== null && serviceName !== void 0 ? serviceName : '').replace('{{serviceName}}', serviceName !== null && serviceName !== void 0 ? serviceName : '');
    return jsx(Dialog, {
      label: title,
      "data-test": "ring-auth-dialog",
      className: className,
      contentClassName: classNames(className, styles.dialog),
      onEscPress: this.onEscPress,
      show: show,
      trapFocus: true,
      children: jsx(ContentWrapper, {
        children: jsxs("div", {
          className: styles.content,
          children: [serviceImage && jsx("img", {
            alt: "".concat(serviceName, " logo"),
            className: styles.logo,
            src: serviceImage
          }), jsx(H2, {
            className: styles.title,
            children: title
          }), errorMessage && jsx("div", {
            className: styles.error,
            children: errorMessage
          }), jsx(Button, {
            primary: true,
            className: styles.firstButton,
            "data-test": "auth-dialog-confirm-button",
            onClick: onConfirm,
            children: confirmLabel
          }), onTryAgain && jsx(Button, {
            className: styles.button,
            "data-test": "auth-dialog-retry-button",
            onClick: () => this.onRetryPress(),
            loader: retrying,
            disabled: retrying,
            children: tryAgainLabel
          }), jsx(Button, {
            className: styles.button,
            "data-test": "auth-dialog-cancel-button",
            onClick: onCancel,
            children: cancelLabel
          })]
        })
      })
    });
  }
}
_defineProperty(AuthDialog, "defaultProps", {
  loginCaption: 'Log in',
  loginToCaption: 'Log in to %serviceName%',
  tryAgainLabel: 'Try again',
  show: false,
  cancelOnEsc: true,
  confirmLabel: 'Log in',
  cancelLabel: 'Remain a guest',
  onConfirm: () => {},
  onCancel: () => {}
});

export { AuthDialog as default };
