import 'core-js/modules/es.promise.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import alertService from '../alert-service/alert-service.js';
import Alert from '../alert/alert.js';
import Link from '../link/link.js';
import Group from '../group/group.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'react-dom/client';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../alert/container.js';
import 'react';
import 'react-dom';
import 'classnames';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/data-tests.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../_helpers/link.js';

var styles = {"title":"title_rui_b890","error":"error_rui_b890"};

let key = null;
function renderAlert(message) {
  let type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Alert.Type.WARNING;
  const existingAlert = alertService.showingAlerts.filter(alert => alert.key === key)[0];
  if (!existingAlert) {
    key = alertService.addAlert(message, type, 0, {
      closeable: false
    });
  } else {
    existingAlert.message = message;
    existingAlert.type = type;
    alertService.renderAlerts();
  }
}
function Message(_ref) {
  let {
    translations,
    onCheckAgain
  } = _ref;
  const {
    backendIsNotAvailable,
    checkAgain,
    errorMessage
  } = translations;
  return jsxs("div", {
    "data-test": "ring-backend-down-notification",
    children: [jsx(Group, {
      children: jsx("div", {
        className: styles.title,
        children: backendIsNotAvailable
      })
    }), jsxs("span", {
      className: styles.error,
      children: [errorMessage, " "]
    }), jsx(Link, {
      onClick: onCheckAgain,
      "data-test": "check-again",
      children: checkAgain
    })]
  });
}
function onBackendDown(_ref2) {
  let {
    onCheckAgain,
    translations
  } = _ref2;
  async function checkAgainWithoutClosing(e) {
    // Alert has weird behaviour of handling clicks by "a" tags
    e.stopPropagation();
    try {
      renderAlert('Connecting...', Alert.Type.LOADING);
      await onCheckAgain();
    } catch (err) {
      renderAlert(jsx(Message, {
        translations: translations,
        onCheckAgain: checkAgainWithoutClosing
      }));
    }
  }
  renderAlert(jsx(Message, {
    translations: translations,
    onCheckAgain: checkAgainWithoutClosing
  }));
  return () => alertService.remove(key);
}

export { onBackendDown as default };
