import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import { jsx } from 'react/jsx-runtime';
import showAuthDialog from '../login-dialog/service.js';
import Link from '../link/link.js';
import AuthResponseParser from './response-parser.js';
import 'react-dom/client';
import '../global/controls-height.js';
import 'react';
import '../login-dialog/login-dialog.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '../dialog/dialog.js';
import 'react-dom';
import 'classnames';
import '@jetbrains/icons/close';
import '../island/island.js';
import '../global/data-tests.js';
import '../island/adaptive-island-hoc.js';
import '../global/linear-function.js';
import '../_helpers/island.js';
import '../island/header.js';
import '../island/content.js';
import 'element-resize-detector';
import '../global/schedule-raf.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../_helpers/button__classes.js';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../_helpers/dialog__body-scroll-preventer.js';
import 'scrollbar-width';
import '../loader-screen/loader-screen.js';
import '../loader/loader.js';
import '../loader/loader__core.js';
import './background-flow.js';
import 'es6-error';
import '../global/url.js';
import '../_helpers/link.js';

class IFrameFlow {
  constructor(requestBuilder, storage, translations) {
    _defineProperty(this, "hideDialog", null);
    _defineProperty(this, "_requestBuilder", void 0);
    _defineProperty(this, "_storage", void 0);
    _defineProperty(this, "_translations", void 0);
    _defineProperty(this, "reject", void 0);
    _defineProperty(this, "_promise", void 0);
    _defineProperty(this, "_loginWindow", void 0);
    _defineProperty(this, "_reset", () => {
      this._promise = null;
      this.hideDialog = null;
    });
    this._requestBuilder = requestBuilder;
    this._storage = storage;
    this._translations = translations;
    this._reset();
  }
  /**
   * Initiates authorization in window
   */
  async _load() {
    const authRequest = await this._requestBuilder.prepareAuthRequest(
    // eslint-disable-next-line camelcase
    {
      request_credentials: 'required',
      auth_mode: 'bypass_to_login'
    }, {
      nonRedirect: false
    });
    const renderFallbackLink = () => jsx(Link, {
      href: authRequest.url,
      target: "_self",
      children: this._translations.nothingHappensLink
    });
    return new Promise((resolve, reject) => {
      this.hideDialog = showAuthDialog({
        url: authRequest.url,
        loader: true,
        onCancel: () => {
          // eslint-disable-next-line @typescript-eslint/no-use-before-define
          cleanUp();
          this.stop();
        },
        renderFallbackLink
      });
      this.reject = reject;
      const removeTokenListener = this._storage.onTokenChange(token => {
        if (token) {
          // eslint-disable-next-line @typescript-eslint/no-use-before-define
          cleanUp();
          resolve(token.accessToken);
        }
      });
      const removeStateListener = this._storage.onStateChange(authRequest.stateId, state => {
        if (state && state.error) {
          // eslint-disable-next-line @typescript-eslint/no-use-before-define
          cleanUp();
          reject(new AuthResponseParser.AuthError(state));
        }
      });
      const cleanUp = () => {
        var _this$hideDialog;
        (_this$hideDialog = this.hideDialog) === null || _this$hideDialog === void 0 || _this$hideDialog.call(this);
        removeStateListener();
        removeTokenListener();
      };
    });
  }
  stop() {
    if (this.hideDialog) {
      this.hideDialog();
    }
    if (this.reject) {
      this.reject('Login form closed');
    }
    this._reset();
  }
  authorize() {
    if (this._promise != null && this._loginWindow != null && !this._loginWindow.closed) {
      return this._promise;
    }
    this._promise = this._load();
    this._promise.then(this._reset, this._reset);
    return this._promise;
  }
}

export { IFrameFlow as default };
