import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.promise.js';
import uuid from 'simply-uuid';
import { encodeURL } from '../global/url.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';

class AuthRequestBuilder {
  /**
   * @param {{
   *   authorization: string,
   *   redirectUri: string?,
   *   requestCredentials: string?,
   *   clientId: string?,
   *   scopes: string[]
   * }} config
   * @param {AuthStorage} storage
   */
  constructor(config, storage) {
    _defineProperty(this, "config", void 0);
    _defineProperty(this, "storage", void 0);
    this.config = config;
    this.storage = storage;
  }
  /**
   * @return {string} random string used for state
   */

  /**
   * Save state and build an auth server redirect URL.
   *
   * @param {object=} extraParams additional query parameters for auth request
   * @param {object=} extraState additional state parameters to save
   * @return {Promise.<string>} promise that is resolved to authURL
   */
  async prepareAuthRequest(extraParams, extraState) {
    const stateId = AuthRequestBuilder._uuid();
    const scopes = this.config.scopes.map(scope => encodeURIComponent(scope));
    /* eslint-disable camelcase */
    const request = Object.assign({
      response_type: 'token',
      state: stateId,
      redirect_uri: this.config.redirectUri,
      request_credentials: this.config.requestCredentials,
      client_id: this.config.clientId,
      scope: scopes.join(' ')
    }, extraParams || {});
    /* eslint-enable camelcase */
    const authURL = encodeURL(this.config.authorization, request);
    const state = Object.assign({
      restoreLocation: window.location.href,
      scopes: this.config.scopes
    }, extraState || {});
    await this._saveState(stateId, state);
    return {
      url: authURL,
      stateId
    };
  }
  /**
   * @param {string} id
   * @param {StoredState} storedState
   * @return {Promise}
   * @private
   */
  _saveState(id, storedState) {
    var _this$storage;
    return (_this$storage = this.storage) === null || _this$storage === void 0 ? void 0 : _this$storage.saveState(id, storedState);
  }
}
_defineProperty(AuthRequestBuilder, "_uuid", uuid.generate);

export { AuthRequestBuilder as default };
