import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import AuthResponseParser from './response-parser.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'es6-error';
import '../global/url.js';
import 'core-js/modules/es.string.match.js';

const NAVBAR_HEIGHT = 50;
const CLOSED_CHECK_INTERVAL = 200;
class WindowFlow {
  constructor(requestBuilder, storage) {
    _defineProperty(this, "_timeoutId", undefined);
    _defineProperty(this, "_requestBuilder", void 0);
    _defineProperty(this, "_storage", void 0);
    _defineProperty(this, "reject", void 0);
    _defineProperty(this, "_loginWindow", void 0);
    _defineProperty(this, "_promise", void 0);
    _defineProperty(this, "checkIsClosed", () => {
      var _this$_loginWindow;
      if ((_this$_loginWindow = this._loginWindow) !== null && _this$_loginWindow !== void 0 && _this$_loginWindow.closed) {
        this.stop();
        return;
      }
      this._timeoutId = window.setTimeout(this.checkIsClosed, CLOSED_CHECK_INTERVAL);
    });
    _defineProperty(this, "_reset", () => {
      this._promise = null;
      this._loginWindow = null;
      clearTimeout(this._timeoutId);
    });
    this._requestBuilder = requestBuilder;
    this._storage = storage;
    this._reset();
  }
  /**
   * Opens window with the given URL
   * @param {string} url
   * @private
   */
  _openWindow(url) {
    const height = 700;
    const width = 750;
    const screenHalves = 2;
    const top = (window.screen.height - height - NAVBAR_HEIGHT) / screenHalves;
    const left = (window.screen.width - width) / screenHalves;
    return window.open(url, 'HubLoginWindow', "height=".concat(height, ", width=").concat(width, ", left=").concat(left, ", top=").concat(top));
  }
  /**
   * Initiates authorization in window
   */
  async _load() {
    const authRequest = await this._requestBuilder.prepareAuthRequest(
    // eslint-disable-next-line camelcase
    {
      request_credentials: 'required',
      auth_mode: 'bypass_to_login'
    }, {
      nonRedirect: true
    });
    return new Promise((resolve, reject) => {
      this.reject = reject;
      let cleanRun;
      const cleanUp = () => {
        var _this$_loginWindow2;
        if (cleanRun) {
          return;
        }
        cleanRun = true;
        /* eslint-disable @typescript-eslint/no-use-before-define */
        removeStateListener();
        removeTokenListener();
        /* eslint-enable @typescript-eslint/no-use-before-define */
        (_this$_loginWindow2 = this._loginWindow) === null || _this$_loginWindow2 === void 0 || _this$_loginWindow2.close();
        clearTimeout(this._timeoutId);
      };
      const removeTokenListener = this._storage.onTokenChange(token => {
        if (token) {
          cleanUp();
          resolve(token.accessToken);
        }
      });
      const removeStateListener = this._storage.onStateChange(authRequest.stateId, state => {
        if (state && state.error) {
          cleanUp();
          reject(new AuthResponseParser.AuthError(state));
        }
      });
      if (this._loginWindow == null || this._loginWindow.closed) {
        this._loginWindow = this._openWindow(authRequest.url);
      } else {
        this._loginWindow.location.href = authRequest.url;
      }
      this.checkIsClosed();
    });
  }
  stop() {
    if (this._loginWindow != null) {
      this._loginWindow.close();
    }
    if (this.reject) {
      this.reject('Authorization window closed');
    }
    this._reset();
  }
  authorize() {
    if (this._promise != null && this._loginWindow != null && !this._loginWindow.closed) {
      this._loginWindow.focus();
      return this._promise;
    }
    this._promise = this._load();
    this._promise.then(this._reset, this._reset);
    return this._promise;
  }
}

export { WindowFlow as default };
