import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import deprecate from 'util-deprecate';
import { isDataURI, parseQueryString, encodeURL } from '../global/url.js';
import { getPixelRatio } from '../global/dom.js';
import memoize from '../global/memoize.js';
import { s as styles, A as AvatarInfo } from '../_helpers/avatar-info.js';
import FallbackAvatar from './fallback-avatar.js';
import { Size } from './avatar-size.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import 'core-js/modules/es.weak-map.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';

const _excluded = ["size", "url", "dpr", "style", "round", "subavatar", "subavatarSize", "username", "info", "skipParams"];
const warnSize = memoize(size => deprecate(() => {}, "Avatar: Size".concat(size, " is deprecated and will be removed in 8.0. The supported sizes are: Size20, Size24, Size28, Size32, Size40.")));
class Avatar extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      errorUrl: ''
    });
    _defineProperty(this, "handleError", () => {
      this.setState({
        errorUrl: this.props.url
      });
    });
    _defineProperty(this, "handleSuccess", () => {
      this.setState({
        errorUrl: ''
      });
    });
  }
  render() {
    const _this$props = this.props,
      {
        size,
        url,
        dpr,
        style,
        round,
        subavatar,
        subavatarSize,
        username,
        info,
        skipParams
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    if ([Size.Size18, Size.Size48].includes(size)) {
      warnSize(size)();
    }
    const sizeString = "".concat(size, "px");
    const subavatarSizeString = "".concat(subavatarSize, "px");
    const styleObj = _objectSpread2({
      height: sizeString,
      width: sizeString
    }, style);
    const styleObjGroup = _objectSpread2({
      borderRadius: '2px',
      height: subavatarSizeString,
      width: subavatarSizeString
    }, style);
    const classes = classNames(styles.avatar, this.props.className, {
      [styles.round]: round
    });
    if (!url || this.state.errorUrl === url) {
      return jsxs("span", _objectSpread2(_objectSpread2({}, restProps), {}, {
        "data-test": "avatar",
        className: classNames(classes, {
          [styles.empty]: username == null && info == null
        }),
        style: styleObj,
        children: [username != null && jsx(FallbackAvatar, {
          size: size,
          round: round,
          username: username
        }), info != null && jsx(AvatarInfo, {
          size: size,
          children: info
        })]
      }));
    }
    let src = url;
    if (!skipParams && !isDataURI(url)) {
      const [urlStart, query] = url.split('?');
      const queryParams = _objectSpread2(_objectSpread2({}, parseQueryString(query)), {}, {
        dpr,
        size
      });
      src = encodeURL(urlStart, queryParams);
    }
    let subavatarSrc = null;
    if (subavatar && !isDataURI(subavatar)) {
      const [urlStart, query] = subavatar.split('?');
      const queryParams = _objectSpread2(_objectSpread2({}, parseQueryString(query)), {}, {
        dpr,
        subavatarSizeString
      });
      subavatarSrc = skipParams ? subavatar : encodeURL(urlStart, queryParams);
      return jsxs("div", {
        children: [jsx("img", _objectSpread2(_objectSpread2({}, restProps), {}, {
          onError: this.handleError,
          onLoad: this.handleSuccess,
          className: classNames(classes, styles.avatarShadow),
          style: styleObj,
          src: src,
          alt: "User avatar"
        })), jsx("img", _objectSpread2(_objectSpread2({}, restProps), {}, {
          "data-test": "avatar",
          onError: this.handleError,
          onLoad: this.handleSuccess,
          className: classNames(styles.subavatar),
          style: styleObjGroup,
          src: subavatarSrc,
          alt: "Subavatar"
        }))]
      });
    } else {
      return jsx("img", _objectSpread2(_objectSpread2({}, restProps), {}, {
        "data-test": "avatar",
        onError: this.handleError,
        onLoad: this.handleSuccess,
        className: classNames(classes, styles.avatarShadow),
        style: styleObj,
        src: src,
        alt: "User avatar"
      }));
    }
  }
}
_defineProperty(Avatar, "defaultProps", {
  dpr: getPixelRatio(),
  size: Size.Size20,
  subavatarSize: Size.Size20 / 2,
  style: {}
});

export { Size, Avatar as default };
