import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, Fragment, jsx } from 'react/jsx-runtime';
import { PureComponent, createRef } from 'react';
import classNames from 'classnames';
import chevronDownIcon from '@jetbrains/icons/chevron-down';
import chevron12pxDown from '@jetbrains/icons/chevron-12px-down';
import deprecate from 'util-deprecate';
import Icon from '../icon/icon.js';
import ClickableLink from '../link/clickableLink.js';
import { ControlsHeightContext } from '../global/controls-height.js';
import { g as getButtonClasses, s as styles } from '../_helpers/button__classes.js';
import { Size } from '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';

const _excluded = ["download", "href", "hrefLang", "media", "ping", "target", "type", "referrerPolicy", "onConditionalClick", "onPlainLeftClick", "activeClassName"],
  _excluded2 = ["active", "danger", "delayed", "loader", "primary", "success", "error", "secondary", "ghost", "short", "text", "dropdown", "height", "icon", "iconRight", "iconSize", "iconClassName", "iconRightClassName", "iconSuppressSizeWarning", "className", "children", "inline", "disabled"];
const warnText = deprecate(() => {}, 'Button: "text" prop is deprecated and will be removed in 8.0. Use inline instead.');
function removeLinkProps(props) {
  const {
      download,
      href,
      hrefLang,
      media,
      ping,
      target,
      type,
      referrerPolicy,
      onConditionalClick,
      onPlainLeftClick,
      activeClassName
    } = props,
    restProps = _objectWithoutProperties(props, _excluded);
  return restProps;
}
/**
 * @name Button
 */
/**
 * A component for displaying variously styled buttons.
 */
class Button extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "buttonRef", /*#__PURE__*/createRef());
  }
  render() {
    var _ref;
    const _this$props = this.props,
      {
        // Modifiers
        active,
        danger,
        delayed,
        loader,
        primary,
        success,
        error,
        secondary,
        ghost,
        short,
        text,
        dropdown,
        height,
        // Props
        icon,
        iconRight,
        iconSize,
        iconClassName,
        iconRightClassName,
        iconSuppressSizeWarning,
        className,
        children,
        inline,
        disabled
      } = _this$props,
      props = _objectWithoutProperties(_this$props, _excluded2);
    const isInline = (_ref = inline !== null && inline !== void 0 ? inline : text) !== null && _ref !== void 0 ? _ref : icon != null;
    if (text != null) {
      warnText();
    }
    const classes = getButtonClasses(_objectSpread2(_objectSpread2({}, this.props), {}, {
      inline: isInline,
      height: height !== null && height !== void 0 ? height : this.context
    }));
    const content = jsxs(Fragment, {
      children: [icon && jsx(Icon, {
        className: classNames(styles.icon, iconClassName),
        glyph: icon,
        size: iconSize,
        suppressSizeWarning: iconSuppressSizeWarning
      }), children, iconRight && jsx(Icon, {
        className: classNames(styles.iconRight, iconRightClassName),
        glyph: iconRight
      }), dropdown && jsx(Icon, {
        glyph: isInline ? chevron12pxDown : chevronDownIcon,
        className: styles.dropdownIcon
      })]
    });
    const isDisabled = disabled || loader || undefined;
    const commonProps = _objectSpread2(_objectSpread2({}, props.href != null && isDisabled ? removeLinkProps(props) : props), {}, {
      className: classes,
      children: jsxs(Fragment, {
        children: [loader && !isInline && jsx("div", {
          className: styles.loaderBackground
        }), content]
      })
    });
    return commonProps.href != null ? jsx(ClickableLink, _objectSpread2({}, commonProps)) : jsx("button", _objectSpread2({
      ref: this.buttonRef,
      type: "button",
      disabled: isDisabled
    }, commonProps));
  }
}
/**
 * @deprecated Use icons with appropriate intrinsic sizes instead
 */
_defineProperty(Button, "IconSize", Size);
_defineProperty(Button, "contextType", ControlsHeightContext);

export { Button, Size as IconSize, Button as default };
