import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import checkmarkIcon from '@jetbrains/icons/checkmark-12px';
import minusIcon from '@jetbrains/icons/remove-12px';
import Icon from '../icon/icon.js';
import { createComposedRef } from '../global/composeRefs.js';
import ControlHelp from '../control-help/control-help.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import 'memoize-one';

var styles = {"checkbox":"checkbox_rui_d990","cell":"cell_rui_d990","icon":"icon_rui_d990","check":"check_rui_d990 icon_rui_d990","minus":"minus_rui_d990 icon_rui_d990","input":"input_rui_d990","cellWrapper":"cellWrapper_rui_d990","focus":"focus_rui_d990","label":"label_rui_d990"};

const _excluded = ["children", "label", "className", "containerClassName", "containerStyle", "cellClassName", "labelClassName", "indeterminate", "inputRef", "help"];
/**
 * @name Checkbox
 */
/**
 * Displays a checkbox.
 */
class Checkbox extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "input", void 0);
    _defineProperty(this, "inputRef", el => {
      if (el != null) {
        el.indeterminate = this.props.indeterminate;
      }
      this.input = el;
    });
    _defineProperty(this, "composedInputRef", createComposedRef());
  }
  componentDidMount() {
    if (this.input != null) {
      this.input.indeterminate = this.props.indeterminate;
    }
  }
  componentDidUpdate(prevProps) {
    const {
      indeterminate
    } = this.props;
    if (this.input != null && indeterminate !== prevProps.indeterminate) {
      this.input.indeterminate = this.props.indeterminate;
    }
  }
  render() {
    const _this$props = this.props,
      {
        children,
        label,
        className,
        containerClassName,
        containerStyle,
        cellClassName,
        labelClassName,
        indeterminate,
        inputRef,
        help
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const classes = classNames(styles.input, className);
    const containerClasses = classNames(styles.checkbox, containerClassName);
    const cellClasses = classNames(styles.cell, cellClassName);
    const labelClasses = classNames(styles.label, labelClassName);
    return jsxs("label", {
      className: containerClasses,
      style: containerStyle,
      "data-test": "ring-checkbox",
      children: [jsx("input", _objectSpread2(_objectSpread2({}, restProps), {}, {
        "data-checked": restProps.checked,
        ref: this.composedInputRef(this.inputRef, inputRef),
        type: "checkbox",
        className: classes
      })), jsx("div", {
        className: styles.cellWrapper,
        children: jsxs("span", {
          className: cellClasses,
          children: [jsx(Icon, {
            glyph: checkmarkIcon,
            className: styles.check
          }), jsx(Icon, {
            glyph: minusIcon,
            className: styles.minus
          })]
        })
      }), jsxs("span", {
        className: labelClasses,
        children: [label || children, help && jsx(ControlHelp, {
          children: help
        })]
      })]
    });
  }
}
_defineProperty(Checkbox, "defaultProps", {
  indeterminate: false
});

export { Checkbox as default };
