import 'core-js/modules/es.promise.js';
import alertService from '../alert-service/alert-service.js';
import { copyHTMLToClipboard, copyTextToClipboard } from './clipboard-fallback.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'react/jsx-runtime';
import 'react-dom/client';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../alert/alert.js';
import 'react';
import 'classnames';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/data-tests.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import 'react-dom';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../alert/container.js';

const ALERT_DELAY = 1000;
function getClipboardImplementation() {
  if (navigator.clipboard && !window.isSecureContext) {
    return {
      copy: text => navigator.clipboard.writeText(text),
      copyHTML: html => navigator.clipboard.write([new ClipboardItem({
        ['text/html']: new Blob([html], {
          type: 'text/html'
        })
      })])
    };
  }
  return {
    copy: str => Promise.resolve(copyTextToClipboard(str)),
    copyHTML: html => Promise.resolve(copyHTMLToClipboard(html))
  };
}
async function copy(text, successMessage, errorMessage) {
  let delay = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : ALERT_DELAY;
  let isHtml = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  try {
    const clipboardImpl = getClipboardImplementation();
    const copyMethod = isHtml ? clipboardImpl.copyHTML : clipboardImpl.copy;
    await copyMethod(text);
    if (successMessage) {
      alertService.successMessage(successMessage, delay);
    }
  } catch (e) {
    if (errorMessage) {
      alertService.error(errorMessage, delay);
    }
    // eslint-disable-next-line no-console
    console.error(e);
  }
}
const clipboard = {
  copyText: async function copyText(text, successMessage, errorMessage) {
    let delay = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : ALERT_DELAY;
    return await copy(text, successMessage, errorMessage, delay);
  },
  copyHTML: async function copyHTML(html, successMessage, errorMessage) {
    let delay = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : ALERT_DELAY;
    return await copy(html, successMessage, errorMessage, delay, true);
  }
};

export { clipboard as default };
