import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { useContext, useRef, useState, useEffect, useMemo } from 'react';
import classNames from 'classnames';
import joinDataTestAttributes from '../global/data-tests.js';
import { getRect } from '../global/dom.js';
import { toPx } from './utils.js';
import { CollapseContext } from './collapse-context.js';
import { COLLAPSE_CONTENT_TEST_ID, COLLAPSE_CONTENT_CONTAINER_TEST_ID } from './consts.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.split.js';

var styles = {"container":"container_rui_88de","transition":"transition_rui_88de","summary":"summary_rui_88de","trigger":"trigger_rui_88de","fade":"fade_rui_88de"};

const DURATION_FACTOR = 0.5;
const DEFAULT_HEIGHT = 0;
const VISIBLE = 1;
const HIDDEN = 0;
/**
 * @name CollapseContent
 */
const CollapseContent = _ref => {
  let {
    children,
    minHeight = DEFAULT_HEIGHT,
    'data-test': dataTest
  } = _ref;
  const {
    collapsed,
    duration,
    id,
    disableAnimation
  } = useContext(CollapseContext);
  const containerRef = useRef(null);
  const contentRef = useRef(null);
  const initialContentHeight = useRef(minHeight);
  const contentHeight = useRef(DEFAULT_HEIGHT);
  const [dimensions, setDimensions] = useState({
    width: 0,
    height: 0
  });
  const [height, setHeight] = useState(toPx(minHeight));
  const [showFade, setShowFade] = useState(collapsed);
  const [shouldHideContent, setShouldHideContent] = useState(collapsed && minHeight <= DEFAULT_HEIGHT);
  useEffect(() => {
    function onTransitionEnd() {
      if (initialContentHeight.current <= DEFAULT_HEIGHT) {
        setShouldHideContent(collapsed);
      }
    }
    const container = containerRef.current;
    container === null || container === void 0 || container.addEventListener('transitionend', onTransitionEnd);
    return () => {
      container === null || container === void 0 || container.removeEventListener('transitionend', onTransitionEnd);
    };
  }, [collapsed]);
  useEffect(() => {
    setShowFade(collapsed);
    if (!collapsed) setShouldHideContent(false);
  }, [collapsed]);
  useEffect(() => {
    if (contentRef.current) {
      contentHeight.current = getRect(contentRef.current).height;
    }
  }, [minHeight, dimensions.height]);
  useEffect(() => {
    const nextHeight = collapsed ? initialContentHeight.current : contentHeight.current;
    setHeight(toPx(nextHeight));
  }, [collapsed, dimensions.height]);
  useEffect(() => {
    if (contentRef.current) {
      const observer = new ResizeObserver(_ref2 => {
        let [entry] = _ref2;
        if (entry && entry.borderBoxSize) {
          const {
            inlineSize,
            blockSize
          } = entry.borderBoxSize[0];
          setDimensions({
            width: inlineSize,
            height: blockSize
          });
        }
      });
      observer.observe(contentRef.current);
    }
  }, []);
  const style = useMemo(() => {
    const calculatedDuration = duration + contentHeight.current * DURATION_FACTOR;
    return {
      '--duration': "".concat(calculatedDuration, "ms"),
      height,
      opacity: collapsed && !minHeight ? HIDDEN : VISIBLE
    };
  }, [duration, height, collapsed, minHeight]);
  const fadeShouldBeVisible = useMemo(() => Boolean(minHeight && showFade), [minHeight, showFade]);
  const shouldRenderContent = disableAnimation ? !collapsed : !shouldHideContent;
  return jsxs("div", {
    ref: containerRef,
    id: "collapse-content-".concat(id),
    "data-test": joinDataTestAttributes(COLLAPSE_CONTENT_CONTAINER_TEST_ID),
    className: classNames(styles.container, {
      [styles.transition]: !disableAnimation
    }),
    style: style,
    children: [jsx("div", {
      ref: contentRef,
      "data-test": joinDataTestAttributes(COLLAPSE_CONTENT_TEST_ID, dataTest),
      children: shouldRenderContent ? children : null
    }), fadeShouldBeVisible && jsx("div", {
      className: styles.fade
    })]
  });
};

export { CollapseContent, CollapseContent as default };
