import { jsx } from 'react/jsx-runtime';
import { useContext, useMemo, cloneElement } from 'react';
import joinDataTestAttributes from '../global/data-tests.js';
import { CollapseContext } from './collapse-context.js';
import { COLLAPSE_CONTROL_TEST_ID } from './consts.js';
import 'core-js/modules/web.dom-collections.iterator.js';

/**
 * @name CollapseControl
 */
const CollapseControl = _ref => {
  let {
    children,
    'data-test': dataTest
  } = _ref;
  const {
    setCollapsed,
    collapsed,
    id
  } = useContext(CollapseContext);
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const child = useMemo(() => {
    if (typeof children === 'function') {
      return children(collapsed);
    }
    return children;
  }, [children, collapsed]);
  return jsx("p", {
    "data-test": joinDataTestAttributes(COLLAPSE_CONTROL_TEST_ID, dataTest),
    children: /*#__PURE__*/cloneElement(child, {
      onClick: setCollapsed,
      'aria-controls': "collapse-content-".concat(id),
      'aria-expanded': String(!collapsed)
    })
  });
};

export { CollapseControl, CollapseControl as default };
