import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { useState, useId, useMemo, useCallback } from 'react';
import { CollapseContext } from './collapse-context.js';
import { BASE_ANIMATION_DURATION } from './consts.js';

/**
 * @name Collapse
 */
const Collapse = _ref => {
  let {
    children,
    duration = BASE_ANIMATION_DURATION,
    disableAnimation = false,
    className = '',
    onChange = () => {},
    defaultCollapsed = true,
    collapsed = null
  } = _ref;
  const [innerCollapsed, setInnerCollapsed] = useState(defaultCollapsed);
  const id = useId();
  const finalCollapsedValue = useMemo(() => collapsed !== null && collapsed !== void 0 ? collapsed : innerCollapsed, [innerCollapsed, collapsed]);
  const setCollapsed = useCallback(() => {
    setInnerCollapsed(!finalCollapsedValue);
    onChange(!finalCollapsedValue);
  }, [setInnerCollapsed, onChange, finalCollapsedValue]);
  return jsx("div", {
    className: className,
    children: jsx(CollapseContext.Provider, {
      value: {
        collapsed: finalCollapsedValue,
        setCollapsed,
        duration,
        disableAnimation,
        id
      },
      children: children
    })
  });
};

export { Collapse, Collapse as default };
