import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent, Component } from 'react';
import classNames from 'classnames';
import focusSensorHOC from '../global/focus-sensor-hoc.js';
import selectionShortcutsHOC from '../table/selection-shortcuts-hoc.js';
import disableHoverHOC from '../table/disable-hover-hoc.js';
import getUID from '../global/get-uid.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import Loader from '../loader/loader.js';
import Item, { moreLessButtonStates } from './item.js';
import { s as styles } from '../_helpers/title.js';
import '../global/composeRefs.js';
import 'memoize-one';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/data-tests.js';
import '../loader/loader__core.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '@jetbrains/icons/chevron-right';
import '@jetbrains/icons/chevron-down';
import '../link/link.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '../text/text.js';
import '../loader-inline/loader-inline.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../control-help/control-help.js';

class DataList extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "shortcutsScope", getUID('ring-data-list-'));
    _defineProperty(this, "onItemFocus", item => {
      const {
        selection,
        onSelect
      } = this.props;
      onSelect(selection.focus(item));
    });
    _defineProperty(this, "onItemSelect", (item, selected) => {
      const {
        selection,
        onSelect
      } = this.props;
      if (selected) {
        onSelect(selection.select(item));
      } else {
        onSelect(selection.deselect(item));
      }
    });
    _defineProperty(this, "onEqualPress", () => {
      const {
        selection,
        itemFormatter
      } = this.props;
      const focused = selection.getFocused();
      if (focused == null) {
        throw new Error('No focused item');
      }
      const item = itemFormatter(focused);
      if (item.collapsed) {
        var _item$onExpand;
        (_item$onExpand = item.onExpand) === null || _item$onExpand === void 0 || _item$onExpand.call(item);
      } else {
        var _item$onCollapse;
        (_item$onCollapse = item.onCollapse) === null || _item$onCollapse === void 0 || _item$onCollapse.call(item);
      }
    });
    _defineProperty(this, "shortcutsMap", {
      '=': this.onEqualPress
    });
  }
  componentDidUpdate(prevProps) {
    const {
      data,
      selection,
      onSelect,
      selectable
    } = this.props;
    if (data !== prevProps.data && !prevProps.remoteSelection) {
      onSelect(selection.cloneWith({
        data
      }));
    }
    if (!selectable && prevProps.selectable) {
      onSelect(selection.resetSelection());
    }
  }
  render() {
    const {
      data,
      className,
      loading,
      selection,
      disabledHover,
      itemFormatter,
      focused,
      innerRef
    } = this.props;
    const shortcutsMap = _objectSpread2(_objectSpread2({}, this.shortcutsMap), this.props.shortcutsMap);
    const classes = classNames(className, {
      [styles.dataList]: true,
      [styles.disabledHover]: disabledHover,
      [styles.multiSelection]: selection.getSelected().size > 0
    });
    return jsxs("div", {
      className: styles.dataListWrapper,
      "data-test": "ring-data-list",
      ref: innerRef,
      children: [focused && jsx(Shortcuts, {
        map: shortcutsMap,
        scope: this.shortcutsScope
      }), jsx("ul", {
        className: classes,
        children: data.map(model => {
          var _this$props$itemMoreL, _this$props;
          const item = itemFormatter(model);
          const {
            id,
            key,
            title,
            items
          } = item;
          const showMoreLessButton = (_this$props$itemMoreL = (_this$props = this.props).itemMoreLessState) === null || _this$props$itemMoreL === void 0 ? void 0 : _this$props$itemMoreL.call(_this$props, item);
          return jsx(Item, {
            item: model,
            title: title,
            items: items,
            itemFormatter: itemFormatter,
            collapsible: item.collapsible,
            collapsed: item.collapsed,
            onCollapse: item.onCollapse,
            onExpand: item.onExpand,
            showFocus: selection.isFocused(model),
            onFocus: this.onItemFocus,
            selection: selection,
            selectable: item.selectable,
            selected: selection.isSelected(model),
            onSelect: this.onItemSelect,
            showMoreLessButton: showMoreLessButton,
            onItemMoreLess: this.props.onItemMoreLess
          }, key || id);
        })
      }), loading && jsx("div", {
        className: data.length > 0 ? styles.loadingOverlay : undefined,
        children: jsx(Loader, {})
      })]
    });
  }
}
_defineProperty(DataList, "defaultProps", {
  loading: false,
  onItemMoreLess: () => {},
  itemMoreLessState: () => moreLessButtonStates.UNUSED,
  remoteSelection: false
});
const getContainer = () => disableHoverHOC(selectionShortcutsHOC(focusSensorHOC(DataList)));
// eslint-disable-next-line react/no-multi-comp
class DataListContainer extends Component {
  constructor() {
    super(...arguments);
    // https://stackoverflow.com/a/53882322/6304152
    _defineProperty(this, "DataList", getContainer());
  }
  render() {
    return jsx(this.DataList, _objectSpread2({}, this.props));
  }
}

export { DataListContainer as default };
