import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import chevronRightIcon from '@jetbrains/icons/chevron-right';
import chevronDownIcon from '@jetbrains/icons/chevron-down';
import Link from '../link/link.js';
import Text from '../text/text.js';
import LoaderInline from '../loader-inline/loader-inline.js';
import { Button } from '../button/button.js';
import { s as styles, T as Title } from '../_helpers/title.js';
import 'classnames';
import '../global/data-tests.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../global/focus-sensor-hoc.js';
import '../global/composeRefs.js';
import 'memoize-one';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../control-help/control-help.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';

var moreLessButtonStates;
(function (moreLessButtonStates) {
  moreLessButtonStates[moreLessButtonStates["UNUSED"] = 0] = "UNUSED";
  moreLessButtonStates[moreLessButtonStates["MORE"] = 1] = "MORE";
  moreLessButtonStates[moreLessButtonStates["MORE_LOADING"] = 2] = "MORE_LOADING";
  moreLessButtonStates[moreLessButtonStates["LESS"] = 3] = "LESS";
})(moreLessButtonStates || (moreLessButtonStates = {}));
const ITEM_LEFT_OFFSET = 32;
const LIST_LEFT_OFFSET = 24;
class Item extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "onShowMore", () => {
      const {
        onItemMoreLess,
        item
      } = this.props;
      onItemMoreLess(item, true);
    });
    _defineProperty(this, "onShowLess", () => {
      const {
        onItemMoreLess,
        item
      } = this.props;
      onItemMoreLess(item, false);
    });
    _defineProperty(this, "onFocus", () => {
      const {
        onFocus,
        item
      } = this.props;
      onFocus(item);
    });
    _defineProperty(this, "onSelect", selected => {
      const {
        onSelect,
        item
      } = this.props;
      onSelect(item, selected);
    });
    _defineProperty(this, "renderItem", (model, parentShift) => {
      const {
        onFocus,
        onSelect,
        selection,
        level,
        itemFormatter
      } = this.props;
      const item = itemFormatter(model);
      return jsx(Item, {
        item: model,
        title: item.title,
        items: item.items,
        level: level + 1,
        parentShift: parentShift,
        itemFormatter: itemFormatter,
        collapsible: item.collapsible,
        collapsed: item.collapsed,
        onCollapse: item.onCollapse,
        onExpand: item.onExpand,
        showFocus: selection.isFocused(model),
        onFocus: onFocus,
        selection: selection,
        selectable: item.selectable,
        selected: selection.isSelected(model),
        onSelect: onSelect
      }, item.key || item.id);
    });
  }
  render() {
    const {
      title,
      items,
      showMoreLessButton,
      level,
      parentShift,
      showFocus,
      selectable,
      selected,
      collapsible,
      collapsed,
      onCollapse,
      onExpand
    } = this.props;
    let moreLessButton;
    if (showMoreLessButton === moreLessButtonStates.MORE || showMoreLessButton === moreLessButtonStates.MORE_LOADING) {
      moreLessButton = jsxs(Text, {
        info: true,
        children: [jsx(Link, {
          inherit: true,
          pseudo: true,
          onClick: this.onShowMore,
          children: 'Show more'
        }), showMoreLessButton === moreLessButtonStates.MORE_LOADING && jsx(LoaderInline, {
          className: styles.showMoreLoader
        })]
      });
    } else if (showMoreLessButton === moreLessButtonStates.LESS) {
      moreLessButton = jsx(Text, {
        info: true,
        children: jsx(Link, {
          inherit: true,
          pseudo: true,
          onClick: this.onShowLess,
          children: 'Show less'
        })
      });
    }
    let collapserExpander = null;
    if (collapsible) {
      if (collapsed) {
        collapserExpander = jsx(Button, {
          title: "Expand",
          onClick: onExpand,
          icon: chevronRightIcon,
          className: styles.collapseButton,
          iconClassName: styles.collapseIcon,
          "data-test": "ring-data-list-expand"
        });
      } else {
        collapserExpander = jsx(Button, {
          title: "Collapse",
          onClick: onCollapse,
          icon: chevronDownIcon,
          className: styles.collapseButton,
          iconClassName: styles.collapseIcon,
          "data-test": "ring-data-list-collapse"
        });
      }
    }
    const itemIsEmpty = !(items !== null && items !== void 0 && items.length) || collapsible && collapsed;
    const offset = level * LIST_LEFT_OFFSET + ITEM_LEFT_OFFSET + parentShift;
    return jsxs("li", {
      children: [jsx(Title, {
        title: title,
        focused: showFocus,
        showFocus: showFocus,
        selectable: selectable,
        selected: selected,
        collapserExpander: collapserExpander,
        onFocus: this.onFocus,
        onSelect: this.onSelect,
        offset: offset
      }), !itemIsEmpty ? jsxs("ul", {
        className: styles.itemContent,
        children: [items.map(model => this.renderItem(model, parentShift)), showMoreLessButton !== moreLessButtonStates.UNUSED ? jsx("li", {
          className: styles.showMore,
          children: moreLessButton
        }) : null]
      }) : null]
    });
  }
}
_defineProperty(Item, "defaultProps", {
  items: [],
  level: 0,
  parentShift: 0,
  showMoreLessButton: moreLessButtonStates.UNUSED,
  onItemMoreLess: () => {}
});

export { Item as default, moreLessButtonStates };
