import { PureComponent, ButtonHTMLAttributes } from 'react';
import * as React from 'react';
import type { Locale } from 'date-fns';
import Popup from '../popup/popup';
import { DropdownAttrs } from '../dropdown/dropdown';
import { Size } from '../input/input';
import { DatePopupProps } from './date-popup';
import { DateInputTranslations, DatePickerChange } from './consts';
export interface DatePickerTranslations extends Partial<DateInputTranslations> {
    setDate: string;
    setDateTime: string;
    setPeriod: string;
}
export type DatePickerProps = Omit<DatePopupProps, 'translations' | 'parseDateInput' | 'onComplete' | 'hidden'> & {
    className: string;
    clear: boolean;
    inline: boolean;
    popupClassName?: string | null | undefined;
    dropdownProps?: Partial<DropdownAttrs>;
    translations?: DatePickerTranslations | null | undefined;
    displayMonthFormat: (date: Date, locale: Locale | undefined) => string;
    displayDayFormat: (date: Date, locale: Locale | undefined) => string;
    displayTimeFormat: (date: Date, locale: Locale | undefined) => string;
    applyTimeInput: (date: Date, time: string | null | undefined) => Date;
    datePlaceholder?: string;
    dateTimePlaceholder?: string;
    rangePlaceholder?: string;
    disabled?: boolean | null | undefined;
    parseDateInput: (input: string | null | undefined) => Date | null;
    size?: Size;
    buttonAttributes?: Pick<ButtonHTMLAttributes<HTMLButtonElement>, 'aria-label'>;
};
/**
 * @name Date Picker
 */
export default class DatePicker extends PureComponent<DatePickerProps> {
    static defaultProps: DatePickerProps;
    static contextType: React.Context<import("../i18n/i18n-context").I18nContextProps>;
    context: React.ContextType<typeof DatePicker.contextType>;
    handleChange: (change: DatePickerChange | Date | null | undefined) => void;
    clear: () => void;
    popup?: Popup | null;
    popupRef: (el: Popup | null) => void;
    closePopup: () => void;
    parse: (arg: string | number | Date | null | undefined) => Date | null;
    formatTime(): string | null;
    getAnchorText: () => string;
    render(): import("react/jsx-runtime").JSX.Element;
}
export type DatePickerAttrs = React.JSX.LibraryManagedAttributes<typeof DatePicker, DatePickerProps>;
