import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import { format } from 'date-fns/format';
import { isSameDay } from 'date-fns/isSameDay';
import { isSameMonth } from 'date-fns/isSameMonth';
import { isSameYear } from 'date-fns/isSameYear';
import { isValid } from 'date-fns/isValid';
import { parse } from 'date-fns/parse';
import { set } from 'date-fns/set';
import calendarIcon from '@jetbrains/icons/calendar';
import chevronDownIcon from '@jetbrains/icons/chevron-down';
import memoize from '../global/memoize.js';
import Popup from '../popup/popup.js';
import Dropdown from '../dropdown/dropdown.js';
import Icon from '../icon/icon.js';
import { Button } from '../button/button.js';
import Link from '../link/link.js';
import { S as Size } from '../_helpers/input.js';
import { I18nContext } from '../i18n/i18n-context.js';
import DatePopup from './date-popup.js';
import { s as styles } from '../_helpers/date-picker.js';
import formats from './formats.js';
import 'core-js/modules/es.weak-map.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'react-dom';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/data-tests.js';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../global/typescript-utils.js';
import '../_helpers/anchor.js';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/link.js';
import '@jetbrains/icons/close-12px';
import '../global/composeRefs.js';
import 'memoize-one';
import '../control-label/control-label.js';
import '../control-help/control-help.js';
import '../i18n/i18n.js';
import 'date-fns/isAfter';
import 'date-fns/isBefore';
import 'date-fns/startOfDay';
import 'date-fns';
import './date-input.js';
import './months.js';
import 'date-fns/addMonths';
import 'date-fns/getDay';
import 'date-fns/getDaysInMonth';
import 'date-fns/startOfMonth';
import 'date-fns/subMonths';
import 'date-fns/endOfMonth';
import '../global/linear-function.js';
import '../global/use-event-callback.js';
import './month.js';
import 'date-fns/addDays';
import 'date-fns/setDay';
import './day.js';
import 'date-fns/getDate';
import 'date-fns/isToday';
import './consts.js';
import 'date-fns/add';
import './month-names.js';
import 'date-fns/isThisMonth';
import 'date-fns/startOfYear';
import './month-slider.js';
import 'date-fns/addYears';
import 'date-fns/subYears';
import './years.js';
import 'date-fns/getYear';
import 'date-fns/isThisYear';
import 'date-fns/setYear';
import './weekdays.js';

const _excluded = ["hidden", "className", "popupRef", "onClear", "datePopupProps", "onComplete"],
  _excluded2 = ["className", "popupClassName", "clear", "inline", "dropdownProps", "translations"];
const PopupComponent = _ref => {
  let {
      hidden = false,
      className,
      popupRef,
      onClear,
      datePopupProps,
      onComplete
    } = _ref,
    restProps = _objectWithoutProperties(_ref, _excluded);
  return jsx(Popup, _objectSpread2(_objectSpread2({
    hidden: hidden,
    className: className,
    ref: popupRef,
    directions: [Popup.PopupProps.Directions.BOTTOM_RIGHT, Popup.PopupProps.Directions.BOTTOM_LEFT, Popup.PopupProps.Directions.TOP_LEFT, Popup.PopupProps.Directions.TOP_RIGHT]
  }, restProps), {}, {
    trapFocus: true,
    children: jsx(DatePopup, _objectSpread2(_objectSpread2({
      onClear: onClear
    }, datePopupProps), {}, {
      onComplete: onComplete
    }))
  }));
};
/**
 * @name Date Picker
 */
class DatePicker extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "handleChange", change => {
      const {
        onChange,
        withTime,
        applyTimeInput
      } = this.props;
      const adjustedChange = withTime && !(change instanceof Date) && (change === null || change === void 0 ? void 0 : change.date) != null ? applyTimeInput(change.date, change.time) : change;
      onChange(adjustedChange);
    });
    _defineProperty(this, "clear", () => {
      let change = null;
      if (this.props.range) {
        change = {
          from: null,
          to: null
        };
      }
      this.handleChange(change);
    });
    _defineProperty(this, "popup", void 0);
    _defineProperty(this, "popupRef", el => {
      this.popup = el;
    });
    _defineProperty(this, "closePopup", () => {
      var _this$popup;
      (_this$popup = this.popup) === null || _this$popup === void 0 || _this$popup._onCloseAttempt();
    });
    _defineProperty(this, "parse", memoize(date => {
      const {
        parseDateInput
      } = this.props;
      if (date instanceof Date) {
        return date;
      }
      if (typeof date === 'number') {
        return new Date(date);
      }
      return parseDateInput(date);
    }));
    _defineProperty(this, "getAnchorText", () => {
      const {
        range,
        datePlaceholder,
        dateTimePlaceholder,
        rangePlaceholder,
        withTime,
        displayFormat,
        displayMonthFormat,
        displayDayFormat,
        translations,
        locale
      } = this.props;
      const {
        translate
      } = this.context;
      const date = this.parse(this.props.date);
      const from = this.parse(this.props.from);
      const to = this.parse(this.props.to);
      const time = this.formatTime();
      if (!range && !withTime) {
        var _ref2;
        return date ? displayFormat(date, locale) : (_ref2 = datePlaceholder !== null && datePlaceholder !== void 0 ? datePlaceholder : translations === null || translations === void 0 ? void 0 : translations.setDate) !== null && _ref2 !== void 0 ? _ref2 : translate('setDate');
      } else if (!range && withTime) {
        if (!date && !time) {
          var _ref3;
          return (_ref3 = dateTimePlaceholder !== null && dateTimePlaceholder !== void 0 ? dateTimePlaceholder : translations === null || translations === void 0 ? void 0 : translations.setDateTime) !== null && _ref3 !== void 0 ? _ref3 : translate('setDateTime');
        } else {
          return "".concat(date && displayFormat(date, locale) || '—', ", ").concat(time || '—');
        }
      } else if (from && to) {
        if (!isSameYear(from, to)) {
          return "".concat(displayFormat(from, locale), " \u2014 ").concat(displayFormat(to, locale));
        } else if (!isSameMonth(from, to)) {
          return "".concat(displayMonthFormat(from, locale), " \u2014 ").concat(displayFormat(to, locale));
        } else if (!isSameDay(from, to)) {
          return "".concat(displayDayFormat(from, locale), " \u2014 ").concat(displayFormat(to, locale));
        } else {
          return "".concat(displayFormat(to, locale));
        }
      } else if (from) {
        return "".concat(displayFormat(from, locale), " \u2014");
      } else if (to) {
        return "\u2014 ".concat(displayFormat(to, locale));
      } else {
        var _ref4;
        return (_ref4 = rangePlaceholder !== null && rangePlaceholder !== void 0 ? rangePlaceholder : translations === null || translations === void 0 ? void 0 : translations.setPeriod) !== null && _ref4 !== void 0 ? _ref4 : translate('setPeriod');
      }
    });
  }
  formatTime() {
    const {
      displayTimeFormat,
      locale
    } = this.props;
    const date = this.parse(this.props.date);
    if (date != null) {
      return displayTimeFormat(date, locale);
    }
    return null;
  }
  render() {
    var _this$props$disabled, _this$props$disabled2;
    const anchorContent = jsxs("div", {
      className: styles.anchorContent,
      children: [jsx(Icon, {
        glyph: calendarIcon,
        className: styles.calendarIcon
      }), this.getAnchorText(), jsx(Icon, {
        glyph: chevronDownIcon,
        className: styles.chevronDownIcon
      })]
    });
    const _this$props = this.props,
      {
        className,
        popupClassName,
        clear,
        inline,
        dropdownProps,
        translations
      } = _this$props,
      datePopupProps = _objectWithoutProperties(_this$props, _excluded2);
    const classes = classNames(styles.datePicker, className, styles["size".concat(this.props.size)], {
      [styles.inline]: inline
    });
    return jsx(Dropdown, _objectSpread2(_objectSpread2({
      className: classes,
      disabled: this.props.disabled,
      "data-test": "ring-date-picker",
      anchor: inline ? jsx(Link, {
        "data-test-ring-dropdown-anchor": true,
        className: styles.anchor,
        disabled: (_this$props$disabled = this.props.disabled) !== null && _this$props$disabled !== void 0 ? _this$props$disabled : false,
        pseudo: true,
        children: this.getAnchorText()
      }) : jsx(Button, _objectSpread2(_objectSpread2({
        "data-test-ring-dropdown-anchor": true,
        className: styles.anchor,
        inline: false,
        disabled: (_this$props$disabled2 = this.props.disabled) !== null && _this$props$disabled2 !== void 0 ? _this$props$disabled2 : false
      }, this.props.buttonAttributes), {}, {
        children: anchorContent
      }))
    }, dropdownProps), {}, {
      children: jsx(PopupComponent, {
        className: popupClassName,
        popupRef: this.popupRef,
        onClear: clear ? this.clear : null,
        datePopupProps: _objectSpread2(_objectSpread2({}, datePopupProps), {}, {
          translations,
          onChange: this.handleChange,
          parseDateInput: this.parse,
          time: this.formatTime()
        }),
        onComplete: this.closePopup
      })
    }));
  }
}
_defineProperty(DatePicker, "defaultProps", {
  className: '',
  date: null,
  withTime: false,
  range: false,
  from: null,
  to: null,
  clear: false,
  inline: false,
  size: Size.M,
  displayFormat: (date, locale) => date ? format(date, 'd MMM yyyy', {
    locale
  }) : '',
  displayMonthFormat: (date, locale) => date ? format(date, 'd MMM', {
    locale
  }) : '',
  displayDayFormat: (date, locale) => date ? format(date, 'd', {
    locale
  }) : '',
  displayTimeFormat: (date, locale) => date ? format(date, 'HH:mm', {
    locale
  }) : '',
  minDate: null,
  maxDate: null,
  onChange() {},
  applyTimeInput(date, timeString) {
    var _timeString$split$map;
    const [hours, minutes] = (_timeString$split$map = timeString === null || timeString === void 0 ? void 0 : timeString.split(':').map(Number)) !== null && _timeString$split$map !== void 0 ? _timeString$split$map : [];
    return minutes != null ? set(date, {
      hours,
      minutes
    }) : date;
  },
  parseDateInput(string) {
    if (!string) {
      return null;
    }
    const today = new Date();
    for (const format of formats) {
      const date = parse(string, format, today);
      if (isValid(date)) {
        return date;
      }
    }
    return null;
  }
});
_defineProperty(DatePicker, "contextType", I18nContext);

export { DatePicker as default };
