import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import { addDays } from 'date-fns/addDays';
import { format } from 'date-fns/format';
import { getDay } from 'date-fns/getDay';
import { getDate } from 'date-fns/getDate';
import { isAfter } from 'date-fns/isAfter';
import { isBefore } from 'date-fns/isBefore';
import { isSameDay } from 'date-fns/isSameDay';
import { isToday } from 'date-fns/isToday';
import { startOfDay } from 'date-fns/startOfDay';
import { getDayNumInWeek, weekdays } from './consts.js';
import { s as styles } from '../_helpers/date-picker.js';
import 'core-js/modules/es.regexp.exec.js';
import 'date-fns/add';

let hoverTO;
class Day extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "handleClick", () => this.props.onSelect(this.props.day));
    _defineProperty(this, "handleMouseOver", () => {
      if (hoverTO) {
        window.clearTimeout(hoverTO);
        hoverTO = null;
      }
      this.props.onHover(this.props.day);
    });
    _defineProperty(this, "handleMouseOut", () => {
      hoverTO = window.setTimeout(this.props.onHover, 0);
    });
    _defineProperty(this, "isDay", date => isSameDay(this.props.day, date));
    _defineProperty(this, "is", name => {
      const value = this.props[name];
      return value != null && this.isDay(value);
    });
    _defineProperty(this, "inRange", range => range && isAfter(startOfDay(this.props.day), startOfDay(range[0])) && isBefore(startOfDay(this.props.day), startOfDay(range[1])));
    _defineProperty(this, "isDisabled", date => {
      const min = this.parse(this.props.minDate);
      const max = this.parse(this.props.maxDate);
      return min != null && isBefore(startOfDay(date), startOfDay(min)) || max != null && isAfter(startOfDay(date), startOfDay(max));
    });
  }
  parse(text) {
    return this.props.parseDateInput(text);
  }
  render() {
    const {
      day,
      from,
      currentRange,
      activeRange,
      empty,
      locale
    } = this.props;
    const reverse = activeRange && activeRange[1] === from;
    const dayInWeek = getDayNumInWeek(locale, getDay(day)) + 1;
    function makeSpreadRange(range) {
      return range && [range[0], addDays(range[1], 1)];
    }
    const spreadRange = makeSpreadRange(currentRange);
    const disabled = this.isDisabled(day);
    const activeSpreadRange = makeSpreadRange(activeRange);
    return (
      // TODO make keyboard navigation actually work
      jsx("button", {
        type: "button",
        className: classNames(styles.day, styles["Day".concat(dayInWeek)], {
          [styles.current]: ['date', 'from', 'to'].some(this.is),
          [styles.active]: !disabled && this.is('activeDate'),
          [styles.weekend]: [weekdays.SA, weekdays.SU].includes(getDay(day)),
          [styles.empty]: empty,
          [styles.from]: currentRange && this.isDay(currentRange[0]) && !reverse || activeRange && this.isDay(activeRange[0]),
          [styles.to]: currentRange && this.isDay(currentRange[1]) || activeRange && this.isDay(activeRange[1]),
          [styles.between]: this.inRange(currentRange),
          [styles.activeBetween]: !disabled && this.inRange(activeRange),
          [styles.first]: getDate(day) === 1,
          [styles.spread]: this.inRange(spreadRange),
          [styles.activeSpread]: !disabled && this.inRange(activeSpreadRange),
          [styles.disabled]: disabled
        }),
        onClick: this.handleClick,
        onMouseOver: this.handleMouseOver,
        onFocus: this.handleMouseOver,
        onMouseOut: this.handleMouseOut,
        onBlur: this.handleMouseOut,
        disabled: disabled,
        children: empty || jsx("span", {
          className: classNames({
            [styles.today]: isToday(day)
          }),
          children: format(day, 'd')
        })
      })
    );
  }
}

export { Day as default };
