import { b as _objectSpread2, _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import { endOfMonth } from 'date-fns/endOfMonth';
import { format } from 'date-fns/format';
import { isThisMonth } from 'date-fns/isThisMonth';
import { set } from 'date-fns/set';
import { startOfDay } from 'date-fns/startOfDay';
import { startOfYear } from 'date-fns/startOfYear';
import linearFunction from '../global/linear-function.js';
import MonthSlider from './month-slider.js';
import { YEAR, MIDDLE_DAY, yearScrollSpeed } from './consts.js';
import { s as styles } from '../_helpers/date-picker.js';
import 'date-fns/addYears';
import 'date-fns/subYears';
import 'core-js/modules/es.regexp.exec.js';
import 'date-fns/add';

class MonthName extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "handleClick", () => {
      const end = endOfMonth(this.props.month);
      this.props.onScrollChange(end.getTime());
    });
  }
  render() {
    const {
      month,
      locale
    } = this.props;
    return jsx("button", {
      type: "button",
      className: classNames(styles.monthName, {
        [styles.today]: isThisMonth(month)
      }),
      onClick: this.handleClick,
      children: format(month, 'LLL', {
        locale
      })
    });
  }
}
function MonthNames(props) {
  const {
    scrollDate,
    locale
  } = props;
  const months = [];
  for (let i = 0; i < YEAR; i++) {
    const middleDay = set(scrollDate, {
      month: i,
      date: MIDDLE_DAY
    });
    months.push(startOfDay(middleDay));
  }
  const pxToDate = linearFunction(0, Number(startOfYear(scrollDate)), yearScrollSpeed);
  let top = 0;
  let bottom = 0;
  if (props.currentRange) {
    [top, bottom] = props.currentRange.map(date => Math.floor(pxToDate.x(Number(date))));
  }
  return jsxs("div", {
    className: styles.monthNames,
    children: [months.map(month => jsx(MonthName, {
      month: month,
      onScrollChange: props.onScrollChange,
      locale: locale
    }, +month)), props.currentRange && jsx("div", {
      className: styles.range,
      style: {
        top: top - 1,
        height: bottom + 1 - (top - 1)
      }
    }), jsx(MonthSlider, _objectSpread2(_objectSpread2({}, props), {}, {
      pxToDate: pxToDate
    }))]
  });
}

export { MonthNames as default };
