import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import { addYears } from 'date-fns/addYears';
import { startOfDay } from 'date-fns/startOfDay';
import { subYears } from 'date-fns/subYears';
import linearFunction from '../global/linear-function.js';
import units, { yearScrollSpeed } from './consts.js';
import { s as styles } from '../_helpers/date-picker.js';
import 'core-js/modules/es.regexp.exec.js';
import 'date-fns/add';

const COVERYEARS = 3;
class MonthSlider extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      dragging: false
    });
    _defineProperty(this, "onMouseDown", () => {
      this.setState({
        dragging: true
      });
    });
    _defineProperty(this, "onMouseUp", () => {
      this.setState({
        dragging: false
      });
    });
    _defineProperty(this, "onMouseMove", e => {
      this.props.onScroll(linearFunction(0, Number(this.props.scrollDate), yearScrollSpeed).y(e.movementY));
    });
  }
  componentDidUpdate(prevProps, prevState) {
    if (this.state.dragging && !prevState.dragging) {
      window.addEventListener('mousemove', this.onMouseMove);
      window.addEventListener('mouseup', this.onMouseUp);
    } else if (!this.state.dragging && prevState.dragging) {
      window.removeEventListener('mousemove', this.onMouseMove);
      window.removeEventListener('mouseup', this.onMouseUp);
    }
  }
  render() {
    let year = subYears(startOfDay(this.props.scrollDate), 1);
    const years = [year];
    for (let i = 0; i <= COVERYEARS; i++) {
      year = addYears(year, 1);
      years.push(year);
    }
    const classes = classNames(styles.monthSlider, {
      [styles.dragging]: this.state.dragging
    });
    return jsx("div", {
      children: years.map(date => jsx("button", {
        type: "button",
        className: classes,
        style: {
          top: Math.floor(this.props.pxToDate.x(Number(date)) - units.cellSize)
        },
        onMouseDown: this.onMouseDown
      }, +date))
    });
  }
}

export { MonthSlider as default };
