import { b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { createElement } from 'react';
import { jsxs, jsx } from 'react/jsx-runtime';
import { addDays } from 'date-fns/addDays';
import { endOfMonth } from 'date-fns/endOfMonth';
import { format } from 'date-fns/format';
import { getDay } from 'date-fns/getDay';
import { setDay } from 'date-fns/setDay';
import Day from './day.js';
import { shiftWeekArray, getWeekStartsOn, weekdays, FIFTH_DAY, WEEK } from './consts.js';
import { s as styles } from '../_helpers/date-picker.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'classnames';
import 'date-fns/getDate';
import 'date-fns/isAfter';
import 'date-fns/isBefore';
import 'date-fns/isSameDay';
import 'date-fns/isToday';
import 'date-fns/startOfDay';
import 'core-js/modules/es.regexp.exec.js';
import 'date-fns/add';

function Month(props) {
  const start = props.month;
  const end = endOfMonth(start);
  const {
    locale
  } = props;
  // pad with empty cells starting from last friday
  const weekday = getDay(start);
  const weekDays = shiftWeekArray(Object.values(weekdays), getWeekStartsOn(props.locale));
  const fifthDayOfWeek = weekDays[FIFTH_DAY];
  let day = setDay(start, weekday >= fifthDayOfWeek ? fifthDayOfWeek : fifthDayOfWeek - WEEK);
  const days = [];
  while (day < end) {
    days.push(day);
    day = addDays(day, 1);
  }
  return jsxs("div", {
    className: styles.month,
    children: [jsx("span", {
      className: styles.monthTitle,
      children: format(props.month, 'LLLL', {
        locale
      })
    }), days.map(date => (/*#__PURE__*/createElement(Day, _objectSpread2(_objectSpread2({}, props), {}, {
      day: date,
      empty: date < start,
      key: +date
    }))))]
  });
}

export { Month as default };
