import { b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { useMemo, useRef, useEffect, createElement } from 'react';
import { addMonths } from 'date-fns/addMonths';
import { getDay } from 'date-fns/getDay';
import { getDaysInMonth } from 'date-fns/getDaysInMonth';
import { startOfMonth } from 'date-fns/startOfMonth';
import { subMonths } from 'date-fns/subMonths';
import { endOfMonth } from 'date-fns/endOfMonth';
import scheduleRAF from '../global/schedule-raf.js';
import linearFunction from '../global/linear-function.js';
import useEventCallback from '../global/use-event-callback.js';
import Month from './month.js';
import MonthNames from './month-names.js';
import { s as styles } from '../_helpers/date-picker.js';
import units, { DOUBLE, HALF, WEEK, weekdays } from './consts.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'date-fns/addDays';
import 'date-fns/format';
import 'date-fns/setDay';
import './day.js';
import 'core-js/modules/es.string.includes.js';
import 'classnames';
import 'date-fns/getDate';
import 'date-fns/isAfter';
import 'date-fns/isBefore';
import 'date-fns/isSameDay';
import 'date-fns/isToday';
import 'date-fns/startOfDay';
import 'date-fns/isThisMonth';
import 'date-fns/set';
import 'date-fns/startOfYear';
import './month-slider.js';
import 'date-fns/addYears';
import 'date-fns/subYears';
import 'core-js/modules/es.regexp.exec.js';
import 'date-fns/add';

const {
  unit,
  cellSize,
  calHeight
} = units;
const FridayToSunday = WEEK + weekdays.SU - weekdays.FR;
const FIVELINES = 31;
const TALLMONTH = 6;
const SHORTMONTH = 5;
const PADDING = 2;
const MONTHSBACK = 2;
function monthHeight(date) {
  const monthStart = startOfMonth(date);
  const daysSinceLastFriday = (getDay(monthStart) + FridayToSunday) % WEEK;
  const monthLines = daysSinceLastFriday + getDaysInMonth(monthStart) > FIVELINES ? TALLMONTH : SHORTMONTH;
  return monthLines * cellSize + unit * PADDING;
}
// in milliseconds per pixel
function scrollSpeed(date) {
  const monthStart = startOfMonth(date);
  const monthEnd = endOfMonth(date);
  return (Number(monthEnd) - Number(monthStart)) / monthHeight(monthStart);
}
const scrollSchedule = scheduleRAF();
let dy = 0;
function Months(props) {
  const {
    scrollDate
  } = props;
  const monthDate = scrollDate instanceof Date ? scrollDate : new Date(scrollDate);
  const monthStart = startOfMonth(monthDate);
  const months = useMemo(() => {
    let month = subMonths(monthStart, MONTHSBACK);
    const result = [month];
    for (let i = 0; i < MONTHSBACK * DOUBLE; i++) {
      month = addMonths(month, 1);
      result.push(month);
    }
    return result;
  }, [monthStart]);
  const currentSpeed = scrollSpeed(scrollDate);
  const pxToDate = linearFunction(0, Number(scrollDate), currentSpeed);
  const offset = pxToDate.x(Number(monthStart)); // is a negative number
  const bottomOffset = monthHeight(scrollDate) + offset;
  const componentRef = useRef(null);
  const handleWheel = useEventCallback(e => {
    e.preventDefault();
    dy += e.deltaY;
    scrollSchedule(() => {
      let date;
      // adjust scroll speed to prevent glitches
      if (dy < offset) {
        date = pxToDate.y(offset) + (dy - offset) * scrollSpeed(months[1]);
      } else if (dy > bottomOffset) {
        date = pxToDate.y(bottomOffset) + (dy - bottomOffset) * scrollSpeed(months[MONTHSBACK + 1]);
      } else {
        date = pxToDate.y(dy);
      }
      props.onScroll(date);
      dy = 0;
    });
  });
  useEffect(() => {
    const current = componentRef.current;
    if (current !== null) {
      current.addEventListener('wheel', handleWheel, {
        passive: false
      });
    }
    return () => {
      if (current !== null) {
        current.removeEventListener('wheel', handleWheel);
      }
    };
  }, [handleWheel]);
  return jsxs("div", {
    className: styles.months,
    ref: componentRef,
    children: [jsx("div", {
      style: {
        top: Math.floor(calHeight * HALF - monthHeight(months[0]) - monthHeight(months[1]) + offset)
      },
      className: styles.days,
      children: months.map(date => (/*#__PURE__*/createElement(Month, _objectSpread2(_objectSpread2({}, props), {}, {
        month: date,
        key: +date
      }))))
    }), jsx(MonthNames, _objectSpread2({}, props))]
  });
}

export { Months as default };
