import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, Fragment, jsx } from 'react/jsx-runtime';
import { PureComponent, createRef } from 'react';
import { createPortal } from 'react-dom';
import classNames from 'classnames';
import closeIcon from '@jetbrains/icons/close';
import { AdaptiveIsland } from '../island/island.js';
import getUID from '../global/get-uid.js';
import joinDataTestAttributes from '../global/data-tests.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import TabTrap from '../tab-trap/tab-trap.js';
import { Button } from '../button/button.js';
import { PopupTarget, PopupTargetContext } from '../popup/popup.target.js';
import { getPopupContainer } from '../popup/popup.js';
import { p as preventerFactory, s as styles } from '../_helpers/dialog__body-scroll-preventer.js';
import '../island/adaptive-island-hoc.js';
import '../global/linear-function.js';
import '../_helpers/island.js';
import '../island/header.js';
import '../island/content.js';
import 'element-resize-detector';
import '../global/schedule-raf.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import 'scrollbar-width';

const _excluded = ["show", "showCloseButton", "onOverlayClick", "onCloseAttempt", "onEscPress", "onCloseClick", "children", "className", "contentClassName", "trapFocus", "data-test", "closeButtonInside", "portalTarget", "label", "closeButtonTitle", "dense", "shortcutOptions", "native", "modal", "preventBodyScroll"];
/**
 * @name Dialog
 */
function noop() {}
class Dialog extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      shortcutsScope: getUID('ring-dialog-')
    });
    _defineProperty(this, "scrollPreventer", preventerFactory(getUID('preventer-')));
    _defineProperty(this, "uid", getUID('dialog-'));
    _defineProperty(this, "handleClick", event => {
      this.props.onOverlayClick(event);
      this.props.onCloseAttempt(event);
    });
    _defineProperty(this, "onCloseClick", event => {
      this.props.onCloseClick(event);
      this.props.onCloseAttempt(event);
    });
    _defineProperty(this, "getShortcutsMap", () => {
      const onEscape = event => {
        if (this.props.show) {
          this.props.onEscPress(event);
          this.props.onCloseAttempt(event);
        }
      };
      return {
        esc: onEscape
      };
    });
    _defineProperty(this, "dialog", void 0);
    _defineProperty(this, "dialogRef", tabTrap => {
      this.dialog = tabTrap && tabTrap.node;
    });
    _defineProperty(this, "nativeDialog", /*#__PURE__*/createRef());
  }
  componentDidMount() {
    const {
      show,
      native
    } = this.props;
    if (native && show) {
      this.toggleNativeDialog();
    }
    this.toggleScrollPreventer();
  }
  componentDidUpdate(prevProps) {
    const {
      show,
      native
    } = this.props;
    if (native && show !== prevProps.show) {
      this.toggleNativeDialog();
    }
    if (prevProps.show !== this.props.show) {
      this.toggleScrollPreventer();
    }
  }
  componentWillUnmount() {
    this.scrollPreventer.reset();
  }
  toggleNativeDialog() {
    const {
      show,
      modal
    } = this.props;
    if (this.nativeDialog.current != null) {
      if (show) {
        this.nativeDialog.current.removeAttribute('open');
        if (modal) {
          this.nativeDialog.current.showModal();
        } else {
          this.nativeDialog.current.show();
        }
      } else {
        this.nativeDialog.current.close();
      }
    }
  }
  toggleScrollPreventer() {
    if (!this.props.preventBodyScroll) {
      return;
    }
    if (this.props.show) {
      this.scrollPreventer.prevent();
    } else {
      this.scrollPreventer.reset();
    }
  }
  render() {
    const _this$props = this.props,
      {
        show,
        showCloseButton,
        onOverlayClick,
        onCloseAttempt,
        onEscPress,
        onCloseClick,
        children,
        className,
        contentClassName,
        trapFocus,
        'data-test': dataTest,
        closeButtonInside,
        portalTarget,
        label,
        closeButtonTitle,
        dense,
        shortcutOptions,
        native,
        modal,
        preventBodyScroll
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const classes = classNames(styles.container, className);
    const shortcutsMap = this.getShortcutsMap();
    const content = jsxs(Fragment, {
      children: [jsx(Shortcuts, {
        map: shortcutsMap,
        scope: this.state.shortcutsScope,
        options: this.props.shortcutOptions
      }), (onOverlayClick !== noop || onCloseAttempt !== noop) && jsx("div", {
        // click handler is duplicated in close button
        role: "presentation",
        className: styles.clickableOverlay,
        onClick: this.handleClick,
        "data-test": "ring-dialog-overlay"
      }), jsx("div", {
        className: styles.innerContainer,
        children: jsxs(AdaptiveIsland, {
          className: classNames(styles.content, contentClassName, {
            [styles.dense]: dense
          }),
          "data-test": "ring-dialog",
          role: "dialog",
          "aria-label": label,
          children: [children, showCloseButton && jsx(Button, {
            icon: closeIcon,
            "data-test": "ring-dialog-close-button",
            className: classNames(styles.closeButton, {
              [styles.closeButtonOutside]: !closeButtonInside,
              [styles.closeButtonInside]: closeButtonInside
            }),
            iconClassName: classNames(styles.closeIcon, {
              [styles.closeIconOutside]: !closeButtonInside,
              [styles.closeIconInside]: closeButtonInside
            }),
            onClick: this.onCloseClick,
            title: closeButtonTitle,
            "aria-label": closeButtonTitle || 'close dialog'
          })]
        })
      })]
    });
    if (native) {
      return jsx("dialog", {
        className: classNames(styles.nativeDialog, className),
        ref: this.nativeDialog,
        "data-rg-modal-dialog-container": modal ? '' : undefined,
        children: jsx(PopupTarget, {
          id: this.uid,
          className: styles.popupTarget,
          children: target => jsxs(Fragment, {
            children: [content, target]
          })
        })
      });
    }
    return show && jsx(PopupTargetContext.Consumer, {
      children: contextTarget => {
        let targetElement = document.body;
        if (portalTarget instanceof HTMLElement) {
          targetElement = portalTarget;
        } else if (contextTarget != null) {
          const container = getPopupContainer(contextTarget);
          if (container != null) {
            targetElement = container;
          }
        }
        return /*#__PURE__*/createPortal(jsx(PopupTarget, {
          id: this.uid,
          className: styles.popupTarget,
          children: target => jsxs(TabTrap, _objectSpread2(_objectSpread2({
            trapDisabled: !trapFocus,
            "data-test": joinDataTestAttributes('ring-dialog-container', dataTest),
            "data-rg-modal-dialog-container": "",
            ref: this.dialogRef,
            className: classes,
            role: "presentation"
          }, restProps), {}, {
            children: [content, target]
          }))
        }), targetElement);
      }
    });
  }
}
_defineProperty(Dialog, "defaultProps", {
  label: 'Dialog',
  onOverlayClick: noop,
  onEscPress: noop,
  onCloseClick: noop,
  onCloseAttempt: noop,
  showCloseButton: false,
  closeButtonInside: false,
  shortcutOptions: {
    modal: false
  },
  trapFocus: false,
  autoFocusFirst: true,
  modal: true,
  preventBodyScroll: true
});

export { Dialog as default };
