import { a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import { jsx } from 'react/jsx-runtime';
import { forwardRef, useMemo, cloneElement } from 'react';
import List, { ActiveItemContext } from '../list/list.js';
import Dropdown from '../dropdown/dropdown.js';
import PopupMenu from '../popup-menu/popup-menu.js';
import getUID from '../global/get-uid.js';
import { A as Anchor } from '../_helpers/anchor.js';
import { isArray } from '../global/typescript-utils.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.trim.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'classnames';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'util-deprecate';
import 'memoize-one';
import '../global/data-tests.js';
import '../global/schedule-raf.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/dom.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../avatar/avatar.js';
import '../global/url.js';
import 'core-js/modules/es.string.replace.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../link/link.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import 'core-js/modules/es.parse-float.js';
import '../popup/popup.js';
import 'react-dom';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';

const _excluded = ["anchor", "pinned", "active", "activeListItemId", "listId"],
  _excluded2 = ["id", "anchor", "ariaLabel", "data", "onSelect", "menuProps", "children"],
  _excluded3 = ["pinned", "active"];
const defaultAriaLabel = 'Dropdown menu';
function DropdownAnchorWrapper(_ref) {
  let {
      anchor,
      pinned,
      active,
      activeListItemId,
      listId
    } = _ref,
    restProps = _objectWithoutProperties(_ref, _excluded);
  const anchorAriaProps = useMemo(() => _objectSpread2(_objectSpread2(_objectSpread2({}, listId ? {
    'aria-haspopup': true
  } : {}), activeListItemId ? {
    'aria-activedescendant': activeListItemId,
    'aria-owns': listId
  } : {}), active ? {
    'aria-expanded': true
  } : {}), [active, activeListItemId, listId]);
  const anchorProps = useMemo(() => _objectSpread2(_objectSpread2({
    active,
    pinned
  }, restProps), anchorAriaProps), [pinned, active, restProps, anchorAriaProps]);
  const anchorComponentProps = useMemo(() => _objectSpread2(_objectSpread2({}, anchorProps), {}, {
    pinned: "".concat(anchorProps.pinned)
  }), [anchorProps]);
  if (typeof anchor === 'string') {
    return jsx(Anchor, _objectSpread2(_objectSpread2({}, anchorComponentProps), {}, {
      children: anchor
    }));
  }
  if (typeof anchor === 'function') {
    return anchor(_objectSpread2({
      active,
      pinned
    }, restProps), anchorAriaProps);
  }
  if (!isArray(anchor)) {
    return /*#__PURE__*/cloneElement(anchor, typeof anchor.type === 'string' ? anchorAriaProps : anchorComponentProps);
  }
  return jsx("div", _objectSpread2(_objectSpread2({}, anchorAriaProps), {}, {
    children: anchor
  }));
}
function renderDropdownMenuChildren(_ref2) {
  let {
    children,
    popupMenuProps
  } = _ref2;
  if (!children) {
    return jsx(PopupMenu, _objectSpread2({}, popupMenuProps));
  }
  return popupProps => children(_objectSpread2(_objectSpread2({}, popupProps), popupMenuProps));
}
const DropdownMenu = /*#__PURE__*/forwardRef(function DropdownMenu(_ref3, forwardedRef) {
  let {
      id,
      anchor: _anchor,
      ariaLabel,
      data,
      onSelect,
      menuProps,
      children
    } = _ref3,
    restDropdownProps = _objectWithoutProperties(_ref3, _excluded2);
  const listId = useMemo(() => id || getUID('dropdown-menu-list'), [id]);
  const popupMenuProps = useMemo(() => _objectSpread2({
    ref: forwardedRef,
    id: listId,
    ariaLabel: ariaLabel || defaultAriaLabel,
    closeOnSelect: true,
    activateFirstItem: true,
    data,
    onSelect
  }, menuProps), [ariaLabel, data, forwardedRef, listId, menuProps, onSelect]);
  return jsx(ActiveItemContext.Provider, {
    children: jsx(Dropdown, _objectSpread2(_objectSpread2({
      anchor: _ref4 => {
        let {
            pinned,
            active
          } = _ref4,
          restAnchorProps = _objectWithoutProperties(_ref4, _excluded3);
        return jsx(ActiveItemContext.ValueContext.Consumer, {
          children: activeItemId => jsx(DropdownAnchorWrapper, _objectSpread2({
            anchor: _anchor,
            pinned: pinned,
            active: active,
            activeListItemId: activeItemId,
            listId: listId
          }, restAnchorProps))
        });
      }
    }, restDropdownProps), {}, {
      children: renderDropdownMenuChildren({
        children,
        popupMenuProps
      })
    }))
  });
});
var DropdownMenu$1 = Object.assign(DropdownMenu, {
  ListProps: List.ListProps
});

export { DropdownMenu$1 as default };
