import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { Component, cloneElement } from 'react';
import classNames from 'classnames';
import joinDataTestAttributes from '../global/data-tests.js';
import { isArray } from '../global/typescript-utils.js';
import { s as styles, A as Anchor } from '../_helpers/anchor.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';

const _excluded = ["initShown", "onShow", "onHide", "hoverShowTimeOut", "hoverHideTimeOut", "children", "anchor", "className", "activeClassName", "hoverMode", "clickMode", "data-test", "disabled"];
/**
 * @name Dropdown
 */
class Dropdown extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      show: this.props.initShown,
      pinned: false
    });
    _defineProperty(this, "onClick", () => {
      if (this.props.disabled) {
        return;
      }
      const {
        show,
        pinned
      } = this.state;
      let nextPinned = pinned;
      if (this.props.hoverMode) {
        if (!pinned) {
          nextPinned = true;
          if (show) {
            this.setState({
              pinned: true
            });
            return;
          }
        } else {
          nextPinned = false;
        }
      }
      this._toggle(!show, nextPinned);
    });
    _defineProperty(this, "onChildCloseAttempt", () => {
      let nextPinned = this.state.pinned;
      if (this.props.hoverMode) {
        nextPinned = false;
      }
      this._toggle(false, nextPinned);
    });
    _defineProperty(this, "hoverTimer", void 0);
    _defineProperty(this, "onMouseEnter", event => {
      var _this$props$onMouseEn, _this$props;
      if (this.props.disabled) {
        return;
      }
      this._clearTimer();
      (_this$props$onMouseEn = (_this$props = this.props).onMouseEnter) === null || _this$props$onMouseEn === void 0 || _this$props$onMouseEn.call(_this$props, event);
      this.hoverTimer = window.setTimeout(() => {
        if (!this.state.show) {
          this._toggle(true);
        }
      }, this.props.hoverShowTimeOut);
    });
    _defineProperty(this, "onMouseLeave", event => {
      var _this$props$onMouseLe, _this$props2;
      if (this.props.disabled) {
        return;
      }
      (_this$props$onMouseLe = (_this$props2 = this.props).onMouseLeave) === null || _this$props$onMouseLe === void 0 || _this$props$onMouseLe.call(_this$props2, event);
      if (this.state.pinned) {
        return;
      }
      this._clearTimer();
      this.hoverTimer = window.setTimeout(() => {
        if (this.state.show) {
          this._toggle(false);
        }
      }, this.props.hoverHideTimeOut);
    });
    _defineProperty(this, "handlePopupInteraction", () => {
      this.setState(_ref => {
        let {
          pinned
        } = _ref;
        return pinned ? null : {
          pinned: true
        };
      });
    });
  }
  toggle() {
    let show = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : !this.state.show;
    this._toggle(show);
  }
  _toggle(show) {
    let pinned = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.state.pinned;
    this.setState({
      show,
      pinned
    }, () => show ? this.props.onShow() : this.props.onHide());
  }
  _clearTimer() {
    if (this.hoverTimer) {
      clearTimeout(this.hoverTimer);
      this.hoverTimer = null;
    }
  }
  render() {
    const {
      show,
      pinned
    } = this.state;
    const _this$props3 = this.props,
      {
        initShown,
        onShow,
        onHide,
        hoverShowTimeOut,
        hoverHideTimeOut,
        children,
        anchor,
        className,
        activeClassName,
        hoverMode,
        clickMode,
        'data-test': dataTest,
        disabled
      } = _this$props3,
      restProps = _objectWithoutProperties(_this$props3, _excluded);
    const classes = classNames(styles.dropdown, className, {
      [activeClassName !== null && activeClassName !== void 0 ? activeClassName : '']: activeClassName != null && show
    });
    let anchorElement;
    const active = hoverMode ? pinned : show;
    switch (typeof anchor) {
      case 'string':
        anchorElement = jsx(Anchor, {
          active: active,
          children: anchor
        });
        break;
      case 'function':
        anchorElement = anchor({
          active: show,
          pinned
        });
        break;
      default:
        if (isArray(anchor) || typeof anchor.type === 'string') {
          anchorElement = anchor;
        } else {
          anchorElement = /*#__PURE__*/cloneElement(anchor, {
            active
          });
        }
    }
    const childProps = {
      hidden: !show,
      onCloseAttempt: this.onChildCloseAttempt,
      onMouseDown: hoverMode ? this.handlePopupInteraction : undefined,
      onContextMenu: hoverMode ? this.handlePopupInteraction : undefined,
      dontCloseOnAnchorClick: true
    };
    return jsxs("div", _objectSpread2(_objectSpread2({
      "data-test": joinDataTestAttributes('ring-dropdown', dataTest)
    }, restProps), {}, {
      onClick: clickMode ? this.onClick : undefined,
      // anchorElement should be a `button` or an `a`
      role: "presentation",
      onMouseEnter: hoverMode ? this.onMouseEnter : undefined,
      onMouseLeave: hoverMode ? this.onMouseLeave : undefined,
      className: classes,
      children: [anchorElement, typeof children === 'function' ? children(childProps) : /*#__PURE__*/cloneElement(children, childProps)]
    }));
  }
}
_defineProperty(Dropdown, "defaultProps", {
  initShown: false,
  clickMode: true,
  hoverMode: false,
  hoverShowTimeOut: 300,
  hoverHideTimeOut: 600,
  disabled: false,
  onShow: () => {},
  onHide: () => {},
  onMouseEnter: () => {},
  onMouseLeave: () => {}
});

export { Anchor, Dropdown as default };
