import { a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.parse-float.js';
import 'core-js/modules/es.string.trim.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, Fragment, jsx } from 'react/jsx-runtime';
import * as React from 'react';
import { useCallback, useEffect } from 'react';
import classNames from 'classnames';
import Heading, { Levels } from '../heading/heading.js';
import { Button } from '../button/button.js';
import { S as Size, i as inputStyles } from '../_helpers/input.js';
import getUID from '../global/get-uid.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import 'util-deprecate';
import '../_helpers/heading.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '@jetbrains/icons/close-12px';
import '../i18n/i18n-context.js';
import '../i18n/i18n.js';
import '../global/composeRefs.js';
import 'memoize-one';
import '../control-label/control-label.js';
import '../control-help/control-help.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';

var styles = {"editableHeading":"editableHeading_rui_0870","fullSize":"fullSize_rui_0870","isEditing":"isEditing_rui_0870","headingWrapperButton":"headingWrapperButton_rui_0870","disabled":"disabled_rui_0870","selectionMode":"selectionMode_rui_0870","heading":"heading_rui_0870","multiline":"multiline_rui_0870","input":"input_rui_0870","error":"error_rui_0870","textarea":"textarea_rui_0870","textareaNotOverflow":"textareaNotOverflow_rui_0870","textareaFade":"textareaFade_rui_0870","textareaWrapper":"textareaWrapper_rui_0870","button":"button_rui_0870","errorText":"errorText_rui_0870","level1":"level1_rui_0870","level2":"level2_rui_0870","level3":"level3_rui_0870","sizeS":"sizeS_rui_0870","sizeM":"sizeM_rui_0870","sizeL":"sizeL_rui_0870","sizeFULL":"sizeFULL_rui_0870"};

const _excluded = ["level", "className", "headingClassName", "inputClassName", "children", "isEditing", "isSavingPossible", "isSaving", "embedded", "size", "onEdit", "onSave", "onCancel", "autoFocus", "data-test", "error", "disabled", "multiline", "renderMenu", "onFocus", "onBlur", "onChange", "onScroll", "maxInputRows", "translations"];
function noop() {}
const EditableHeading = props => {
  const {
      level = Levels.H1,
      className,
      headingClassName,
      inputClassName,
      children,
      isEditing = false,
      isSavingPossible = false,
      isSaving = false,
      embedded = false,
      size = Size.L,
      onEdit = noop,
      onSave = noop,
      onCancel = noop,
      autoFocus = true,
      'data-test': dataTest,
      error,
      disabled,
      multiline = false,
      renderMenu = () => null,
      onFocus,
      onBlur,
      onChange,
      onScroll,
      maxInputRows,
      translations = {
        save: 'Save',
        cancel: 'Cancel'
      }
    } = props,
    restProps = _objectWithoutProperties(props, _excluded);
  const [shortcutsScope] = React.useState(getUID('ring-editable-heading-'));
  const [isInFocus, setIsInFocus] = React.useState(false);
  const [isMouseDown, setIsMouseDown] = React.useState(false);
  const [isInSelectionMode, setIsInSelectionMode] = React.useState(false);
  const textAreaRef = React.useRef(null);
  const [isScrolledToBottom, setIsScrolledToBottom] = React.useState(false);
  const [isOverflow, setIsOverflow] = React.useState(false);
  const hasError = error !== undefined;
  const isSaveDisabled = !isSavingPossible || !children || children.trim() === '' || hasError || isSaving;
  const isCancelDisabled = isSaving;
  const isShortcutsDisabled = !isInFocus || isSaving;
  const shortcutsMap = React.useMemo(() => {
    const map = {};
    if (!isSaveDisabled) {
      map.enter = onSave;
    }
    if (isCancelDisabled) {
      map.esc = onCancel;
    }
    return map;
  }, [isSaveDisabled, isCancelDisabled, onSave, onCancel]);
  const classes = classNames(styles.editableHeading, className, {
    [styles.fullSize]: isEditing && size === Size.FULL,
    [styles.isEditing]: isEditing,
    [styles.error]: hasError,
    [styles.disabled]: disabled,
    [styles.multiline]: multiline,
    [styles.selectionMode]: isInSelectionMode
  });
  const headingClasses = classNames(styles.heading, headingClassName, styles["size".concat(size)]);
  const inputClasses = classNames('ring-js-shortcuts', styles.input, styles.textarea, {
    [styles.textareaNotOverflow]: !isOverflow
  }, inputStyles["size".concat(size)], styles["level".concat(level)], inputClassName);
  const stretch = useCallback(el => {
    if (!el || !el.style) {
      return;
    }
    el.style.height = '0';
    const {
      paddingTop,
      paddingBottom
    } = window.getComputedStyle(el);
    el.style.height = "".concat(el.scrollHeight - parseFloat(paddingTop) - parseFloat(paddingBottom), "px");
  }, []);
  const checkValue = useCallback(el => {
    if (multiline && el != null && el.scrollHeight >= el.clientHeight) {
      stretch(el);
    }
  }, [stretch, multiline]);
  const checkOverflow = useCallback(el => {
    const scrollHeight = el.scrollHeight || 0;
    const clientHeight = el.clientHeight || 0;
    const scrollTop = el.scrollTop || 0;
    setIsScrolledToBottom(scrollHeight - clientHeight <= scrollTop);
    setIsOverflow(scrollHeight > clientHeight);
  }, [setIsScrolledToBottom]);
  const onHeadingMouseDown = React.useCallback(() => {
    setIsMouseDown(true);
  }, []);
  const onMouseMove = React.useCallback(() => {
    if (!isMouseDown) {
      return;
    }
    setIsInSelectionMode(true);
  }, [isMouseDown]);
  const onMouseUp = React.useCallback(() => {
    if (isMouseDown && !isInSelectionMode && !disabled) {
      onEdit();
    }
    setIsMouseDown(false);
    setIsInSelectionMode(false);
  }, [isMouseDown, isInSelectionMode, disabled, onEdit]);
  const onInputFocus = React.useCallback(e => {
    setIsInFocus(true);
    checkValue(e.target);
    checkOverflow(e.target);
    onFocus === null || onFocus === void 0 || onFocus(e);
  }, [onFocus, checkOverflow, checkValue]);
  const onInputChange = React.useCallback(e => {
    checkValue(e.target);
    checkOverflow(e.target);
    onChange === null || onChange === void 0 || onChange(e);
  }, [onChange, checkOverflow, checkValue]);
  const onInputScroll = React.useCallback(e => {
    checkOverflow(e.target);
    onScroll === null || onScroll === void 0 || onScroll(e);
  }, [onScroll, checkOverflow]);
  const onInputBlur = React.useCallback(e => {
    setIsInFocus(false);
    onBlur === null || onBlur === void 0 || onBlur(e);
  }, [onBlur]);
  useEffect(() => {
    window.addEventListener('mousemove', onMouseMove);
    window.addEventListener('mouseup', onMouseUp);
    return () => {
      window.removeEventListener('mousemove', onMouseMove);
      window.removeEventListener('mouseup', onMouseUp);
    };
  }, [onMouseMove, onMouseUp]);
  return jsxs(Fragment, {
    children: [jsxs("div", {
      className: classes,
      children: [!disabled && isEditing ? jsxs(Fragment, {
        children: [jsx(Shortcuts, {
          map: shortcutsMap,
          scope: shortcutsScope,
          disabled: isShortcutsDisabled
        }), !multiline ? jsx("input", _objectSpread2(_objectSpread2({
          className: inputClasses,
          value: children,
          autoFocus: autoFocus,
          "data-test": dataTest,
          disabled: isSaving,
          onChange: onChange
        }, restProps), {}, {
          onFocus: onInputFocus,
          onBlur: onInputBlur
        })) : jsxs("div", {
          className: classNames(styles.textareaWrapper, inputStyles["size".concat(size)]),
          children: [jsx("textarea", _objectSpread2(_objectSpread2({
            ref: textAreaRef,
            className: inputClasses,
            value: children,
            autoFocus: autoFocus,
            "data-test": dataTest,
            disabled: isSaving,
            onChange: onInputChange
          }, restProps), {}, {
            onFocus: onInputFocus,
            onBlur: onInputBlur,
            onScroll: onInputScroll,
            style: {
              maxHeight: maxInputRows ? "".concat(maxInputRows, "lh") : ''
            }
          })), !isScrolledToBottom && jsx("div", {
            className: styles.textareaFade
          })]
        })]
      }) : jsx("button", {
        type: "button",
        className: styles.headingWrapperButton,
        onMouseDown: onHeadingMouseDown,
        children: jsx(Heading, {
          className: headingClasses,
          level: level,
          "data-test": dataTest,
          children: children
        })
      }), !isEditing && renderMenu(), isEditing && !embedded && jsxs(Fragment, {
        children: [jsx(Button, {
          className: styles.button,
          primary: true,
          disabled: isSaveDisabled,
          loader: isSaving,
          onClick: onSave,
          children: translations.save
        }), jsx(Button, {
          className: styles.button,
          disabled: isCancelDisabled,
          onClick: onCancel,
          children: translations.cancel
        })]
      })]
    }), isEditing && error && jsx("div", {
      className: classNames(styles.errorText, inputStyles["size".concat(size)]),
      children: error
    })]
  });
};
var editableHeading = /*#__PURE__*/React.memo(EditableHeading);

export { EditableHeading, Levels, Size, editableHeading as default };
