import { jsx } from 'react/jsx-runtime';
import { memo, isValidElement } from 'react';
import classNames from 'classnames';
import Link from '../link/link.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import '../global/data-tests.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';

var styles = {"footer":"footer_rui_fade font_rui_8bff","footerFloating":"footerFloating_rui_fade footer_rui_fade font_rui_8bff","column":"column_rui_fade","columnItem":"columnItem_rui_fade","columnLeft":"columnLeft_rui_fade column_rui_fade","columnCenter":"columnCenter_rui_fade column_rui_fade","line":"line_rui_fade","columnRight":"columnRight_rui_fade column_rui_fade","lineCenter":"lineCenter_rui_fade"};

const FooterColumn = /*#__PURE__*/memo(function FooterColumn(_ref) {
  let {
    position,
    children
  } = _ref;
  const classes = classNames({
    [styles.columnLeft]: position === 'left',
    [styles.columnCenter]: position === 'center',
    [styles.columnRight]: position === 'right'
  });
  return jsx("div", {
    className: classes,
    children: jsx("ul", {
      className: styles.columnItem,
      children: children
    })
  });
});
/**
 * Return copyright string
 * @param year {int}
 * @returns {string}
 */
function copyright(year) {
  const currentYear = new Date().getUTCFullYear();
  const ndash = '–';
  let ret = 'Copyright © ';
  if (year >= currentYear) {
    ret += year;
  } else {
    ret += year + ndash + currentYear;
  }
  return ret;
}
/**
 * @constructor
 * @extends {ReactComponent}
 */
const FooterLine = /*#__PURE__*/memo(function FooterLine(props) {
  const items = Array.isArray(props.item) ? props.item : [props.item];
  function renderItem(item) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    if (/*#__PURE__*/isValidElement(item) || typeof item !== 'object') {
      return item;
    }
    const element = (item.copyright ? copyright(item.copyright) : '') + (item.label || item);
    if (item.url) {
      return jsx(Link, {
        href: item.url,
        target: item.target,
        title: item.title,
        children: element
      }, item.url + item.title);
    }
    return element;
  }
  return jsx("li", {
    className: styles.line,
    children: items.map(renderItem)
  });
});
const Footer = /*#__PURE__*/memo(function Footer(_ref2) {
  let {
    floating,
    className,
    left,
    center,
    right
  } = _ref2;
  function content(elements, position) {
    if (!elements) {
      return false;
    }
    return jsx(FooterColumn, {
      position: position,
      children: elements.map((item, idx) => jsx(FooterLine
      // eslint-disable-next-line react/no-array-index-key
      , {
        item: item
      }, idx))
    }, position);
  }
  const classes = classNames(styles.footer, className, {
    [styles.footerFloating]: floating
  });
  return jsx("footer", {
    className: classes,
    "data-test": "ring-footer",
    children: [content(left, 'left'), content(center, 'center'), content(right, 'right')]
  });
});

export { copyright, Footer as default };
