import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { memo, createContext, useState, useContext, useEffect } from 'react';

function createStatefulContext(initialValue) {
  let name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  const ValueContext = /*#__PURE__*/createContext(initialValue);
  const UpdateContext = /*#__PURE__*/createContext(() => {});
  function Provider(_ref) {
    let {
      children
    } = _ref;
    const [value, update] = useState(initialValue);
    return jsx(ValueContext.Provider, {
      value: value,
      children: jsx(UpdateContext.Provider, {
        value: update,
        children: children
      })
    });
  }
  Provider.displayName = "".concat(name, "Provider");
  function useUpdate(value, skipUpdate) {
    const update = useContext(UpdateContext);
    useEffect(() => {
      if (!skipUpdate) {
        update(value);
      }
    }, [update, value, skipUpdate]);
  }
  function Updater(_ref2) {
    let {
      value,
      skipUpdate
    } = _ref2;
    useUpdate(value, skipUpdate);
    return null;
  }
  Updater.displayName = "".concat(name, "Updater");
  return {
    ValueContext,
    UpdateContext,
    Provider,
    useUpdate,
    Updater: /*#__PURE__*/memo(Updater)
  };
}

export { createStatefulContext as default };
