import { b as _objectSpread2, _ as _defineProperty, a as _objectWithoutProperties } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import { createComposedRef } from './composeRefs.js';
import 'memoize-one';

const _excluded = ["autofocus", "focused", "onFocus", "onBlur", "innerRef", "scrollOnTableFocus"];
function focusSensorHOC(ComposedComponent) {
  class FocusSensor extends Component {
    constructor() {
      super(...arguments);
      _defineProperty(this, "state", {
        focused: this.props.focused
      });
      _defineProperty(this, "node", void 0);
      _defineProperty(this, "_skipNextCapture", false);
      _defineProperty(this, "onRefUpdate", node => {
        if (node) {
          this.node = node;
        }
      });
      _defineProperty(this, "composedRef", createComposedRef());
      _defineProperty(this, "onFocusCapture", _ref => {
        var _this$node;
        let {
          target
        } = _ref;
        if (this._skipNextCapture) {
          this._skipNextCapture = false;
          return;
        }
        const focused = target instanceof Node && ((_this$node = this.node) === null || _this$node === void 0 ? void 0 : _this$node.contains(target));
        if (focused && !this.state.focused) {
          var _this$props$onFocus, _this$props;
          this.setState({
            focused: true
          });
          (_this$props$onFocus = (_this$props = this.props).onFocus) === null || _this$props$onFocus === void 0 || _this$props$onFocus.call(_this$props);
        }
      });
      _defineProperty(this, "onBlurCapture", _ref2 => {
        let {
          target
        } = _ref2;
        const {
          state: {
            focused
          },
          node
        } = this;
        if (focused) {
          setTimeout(() => {
            const blurred = target instanceof Node && (node === null || node === void 0 ? void 0 : node.contains(target)) && !node.contains(document.activeElement);
            if (blurred) {
              var _this$props$onBlur, _this$props2;
              this.setState({
                focused: false
              });
              (_this$props$onBlur = (_this$props2 = this.props).onBlur) === null || _this$props$onBlur === void 0 || _this$props$onBlur.call(_this$props2);
            }
          }, 1);
        }
      });
      _defineProperty(this, "onFocusRestore", () => {
        var _this$node2;
        this._skipNextCapture = true;
        (_this$node2 = this.node) === null || _this$node2 === void 0 || _this$node2.focus({
          preventScroll: !this.props.scrollOnTableFocus
        });
      });
      _defineProperty(this, "onFocusReset", () => {
        var _this$node3;
        (_this$node3 = this.node) === null || _this$node3 === void 0 || _this$node3.blur();
      });
    }
    componentDidMount() {
      const {
        props: {
          autofocus,
          scrollOnTableFocus
        },
        node
      } = this;
      node === null || node === void 0 || node.setAttribute('tabindex', '0');
      if (node != null) {
        node.style.outline = 'none';
      }
      document.addEventListener('focus', this.onFocusCapture, true);
      document.addEventListener('blur', this.onBlurCapture, true);
      if (autofocus) {
        node === null || node === void 0 || node.focus({
          preventScroll: !scrollOnTableFocus
        });
      }
    }
    componentDidUpdate(prevProps) {
      var _this$node4;
      const {
        focused
      } = this.props;
      const isInFocus = (_this$node4 = this.node) === null || _this$node4 === void 0 ? void 0 : _this$node4.contains(document.activeElement);
      if (focused && (!isInFocus || !prevProps.focused)) {
        this.onFocusRestore();
      } else if (!focused && prevProps.focused) {
        this.onFocusReset();
      }
    }
    componentWillUnmount() {
      document.removeEventListener('focus', this.onFocusCapture, true);
      document.removeEventListener('blur', this.onBlurCapture, true);
    }
    render() {
      const _this$props3 = this.props,
        {
          autofocus,
          focused,
          onFocus,
          onBlur,
          innerRef,
          scrollOnTableFocus
        } = _this$props3,
        rest = _objectWithoutProperties(_this$props3, _excluded);
      return jsx(ComposedComponent, _objectSpread2(_objectSpread2({}, rest), {}, {
        innerRef: this.composedRef(innerRef, this.onRefUpdate),
        focused: this.state.focused,
        onFocusReset: this.onFocusReset,
        onFocusRestore: this.onFocusRestore
      }));
    }
  }
  FocusSensor.defaultProps = _objectSpread2(_objectSpread2({}, ComposedComponent.defaultProps), {}, {
    focused: false,
    autofocus: false,
    scrollOnTableFocus: true,
    onFocus: () => {},
    onBlur: () => {}
  });
  return FocusSensor;
}

export { focusSensorHOC as default };
