import { a as _objectWithoutProperties } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.array.reverse.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import { s as styles } from '../_helpers/grid.js';

const _excluded = ["children", "className", "reverse"];
const classMap = {
  xs: 'col-xs',
  sm: 'col-sm',
  md: 'col-md',
  lg: 'col-lg',
  xsOffset: 'col-xs-offset',
  smOffset: 'col-sm-offset',
  mdOffset: 'col-md-offset',
  lgOffset: 'col-lg-offset'
};
/**
 * Converts props like "xs=9 xsOffset={2}" to classes "col-xs-9 col-xs-offset-2"
 * @param {Object} props incoming props
 * @returns {Array} result classnames
 */
function getClassNames(props) {
  return Object.entries(props).filter(_ref => {
    let [key] = _ref;
    return classMap[key];
  }).map(_ref2 => {
    let [key, value] = _ref2;
    return styles[Number.isInteger(value) ? "".concat(classMap[key], "-").concat(value) : classMap[key]];
  });
}
class Col extends Component {
  render() {
    const _this$props = this.props,
      {
        children,
        className,
        reverse
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const classes = classNames(styles.col, className, getClassNames(restProps), {
      [styles.reverse]: reverse
    });
    return jsx("div", {
      className: classes,
      "data-test": "ring-grid-column",
      children: children
    });
  }
}

export { Col as default };
