import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import Avatar from '../avatar/avatar.js';
import { Button } from '../button/button.js';
import DropdownMenu from '../dropdown-menu/dropdown-menu.js';
import PopupMenu from '../popup-menu/popup-menu.js';
import { I18nContext } from '../i18n/i18n-context.js';
import { isTruthy } from '../global/typescript-utils.js';
import { s as styles } from '../_helpers/header.js';
import { Size } from '../avatar/avatar-size.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'util-deprecate';
import '../global/url.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../_helpers/avatar-info.js';
import '../avatar/fallback-avatar.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../list/list.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.trim.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'memoize-one';
import '../global/data-tests.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../link/link.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../dropdown/dropdown.js';
import '../_helpers/anchor.js';
import 'core-js/modules/es.parse-float.js';
import '../popup/popup.js';
import 'react-dom';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../i18n/i18n.js';

const _excluded = ["className", "activeClassName", "closeOnSelect", "hasUpdates", "onLogout", "user", "profileUrl", "LinkComponent", "onSwitchUser", "renderPopupItems", "onRevertPostponement", "showApplyChangedUser", "showName", "showLogIn", "showLogOut", "showSwitchUser", "renderGuest", "translations", "size", "round", "loading", "onLogin", "menuProps", "onShow", "onHide"];
const rgItemType = PopupMenu.ListProps.Type.LINK;
class Profile extends PureComponent {
  render() {
    var _translations$applyCh, _translations$login, _translations$profile, _translations$switchU, _translations$logout;
    const _this$props = this.props,
      {
        className,
        activeClassName,
        closeOnSelect,
        hasUpdates,
        onLogout,
        user,
        profileUrl,
        LinkComponent,
        onSwitchUser,
        renderPopupItems,
        onRevertPostponement,
        showApplyChangedUser,
        showName,
        showLogIn,
        showLogOut,
        showSwitchUser,
        renderGuest,
        translations,
        size,
        round,
        loading,
        onLogin,
        menuProps,
        onShow,
        onHide
      } = _this$props,
      props = _objectWithoutProperties(_this$props, _excluded);
    const {
      translate
    } = this.context;
    if (!user) {
      return jsx("div", _objectSpread2(_objectSpread2({}, props), {}, {
        className: classNames(styles.profileEmpty, className),
        children: jsx(Avatar, {
          size: size,
          round: round
        })
      }));
    }
    if (user.guest) {
      return renderGuest(this.props);
    }
    const avatarWrapper = classNames(styles.avatarWrapper, {
      [styles.hasUpdates]: hasUpdates
    });
    const anchor = jsxs("button", {
      type: "button",
      className: styles.anchorClassName,
      children: [jsx("span", {
        className: avatarWrapper,
        children: jsx(Avatar, {
          url: user.profile && user.profile.avatar && user.profile.avatar.url,
          size: size,
          round: round,
          username: user.name
        })
      }), showName && jsx("span", {
        children: user.name
      })]
    });
    const items = [showApplyChangedUser && {
      rgItemType,
      label: (_translations$applyCh = translations === null || translations === void 0 ? void 0 : translations.applyChangedUser) !== null && _translations$applyCh !== void 0 ? _translations$applyCh : translate('applyChangedUser'),
      className: styles.profileMenuItem,
      onClick: onRevertPostponement
    }, showLogIn && {
      rgItemType,
      label: (_translations$login = translations === null || translations === void 0 ? void 0 : translations.login) !== null && _translations$login !== void 0 ? _translations$login : translate('login'),
      className: styles.profileMenuItem,
      onClick: onRevertPostponement
    }, {
      rgItemType: PopupMenu.ListProps.Type.LINK,
      label: (_translations$profile = translations === null || translations === void 0 ? void 0 : translations.profile) !== null && _translations$profile !== void 0 ? _translations$profile : translate('profile'),
      target: '_self',
      // Full page reload in Angular
      href: profileUrl,
      LinkComponent
    }, showSwitchUser && {
      rgItemType,
      label: (_translations$switchU = translations === null || translations === void 0 ? void 0 : translations.switchUser) !== null && _translations$switchU !== void 0 ? _translations$switchU : translate('switchUser'),
      className: styles.profileMenuItem,
      onClick: onSwitchUser
    }, showLogOut && {
      rgItemType,
      label: (_translations$logout = translations === null || translations === void 0 ? void 0 : translations.logout) !== null && _translations$logout !== void 0 ? _translations$logout : translate('logout'),
      onClick: onLogout
    }].filter(isTruthy);
    return jsx(DropdownMenu, _objectSpread2(_objectSpread2({}, props), {}, {
      onShow: onShow,
      onHide: onHide,
      title: user.name,
      anchor: anchor,
      data: renderPopupItems(items),
      "data-test": "ring-profile",
      className: classNames(styles.profile, className),
      activeClassName: activeClassName,
      menuProps: _objectSpread2({
        closeOnSelect,
        left: -2,
        top: -8,
        sidePadding: 32
      }, menuProps)
    }));
  }
}
_defineProperty(Profile, "defaultProps", {
  closeOnSelect: true,
  renderPopupItems: items => items,
  size: Size.Size32,
  renderGuest: _ref => {
    let {
      loading,
      onLogin,
      className,
      translations
    } = _ref;
    return jsx(I18nContext.Consumer, {
      children: _ref2 => {
        var _translations$login2;
        let {
          translate
        } = _ref2;
        return jsx("div", {
          className: classNames(styles.profileEmpty, className),
          children: jsx(Button, {
            primary: true,
            "data-test": "ring-header-login-button",
            disabled: loading,
            loader: loading,
            onClick: onLogin,
            children: (_translations$login2 = translations === null || translations === void 0 ? void 0 : translations.login) !== null && _translations$login2 !== void 0 ? _translations$login2 : translate('login')
          })
        });
      }
    });
  }
});
_defineProperty(Profile, "contextType", I18nContext);
_defineProperty(Profile, "Size", Size);

export { Profile as default };
