import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.array.sort.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import servicesIcon from '@jetbrains/icons/services-20px';
import classNames from 'classnames';
import Dropdown from '../dropdown/dropdown.js';
import Popup from '../popup/popup.js';
import { T as Theme, d as darkStyles } from '../_helpers/theme.js';
import HeaderIcon from './header-icon.js';
import { S as ServicesLink, s as styles } from '../_helpers/services-link.js';
import '../global/data-tests.js';
import '../global/typescript-utils.js';
import '../_helpers/anchor.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'react-dom';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../_helpers/header.js';
import '../link/link.js';
import '../_helpers/link.js';

const _excluded = ["clientId", "loading", "services", "initShown", "theme"];
const makeAnchor = loading => {
  const Anchor = _ref => {
    let {
      active
    } = _ref;
    return jsx(HeaderIcon, {
      loader: loading,
      active: active,
      icon: servicesIcon,
      "aria-label": "Services"
    });
  };
  return Anchor;
};
class Services extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "serviceIsActive", service => service.id === this.props.clientId);
  }
  render() {
    const _this$props = this.props,
      {
        clientId,
        loading,
        services,
        initShown,
        theme
      } = _this$props,
      props = _objectWithoutProperties(_this$props, _excluded);
    if (!services) {
      return jsx(HeaderIcon, _objectSpread2(_objectSpread2({}, props), {}, {
        loader: loading,
        active: loading,
        icon: servicesIcon,
        "aria-label": "Services"
      }));
    }
    const sortedServices = [...services].sort(Services.sort);
    const servicesWithIcons = sortedServices.filter(service => service.iconUrl && service.homeUrl);
    const servicesWithOutIcons = sortedServices.filter(service => !service.iconUrl && service.homeUrl);
    const separatorIsRequired = servicesWithIcons.length !== 0 && servicesWithOutIcons.length !== 0;
    return jsx(Dropdown, _objectSpread2(_objectSpread2({}, props), {}, {
      anchor: makeAnchor(loading),
      initShown: initShown,
      children: jsxs(Popup, {
        className: classNames(styles.services, {
          [darkStyles.dark]: theme === Theme.DARK
        }),
        top: -3,
        children: [servicesWithIcons.map(service => {
          const isActive = this.serviceIsActive(service);
          return jsx(Services.Link, {
            active: isActive,
            className: isActive ? styles.activeItem : styles.item,
            service: service
          }, service.id);
        }), separatorIsRequired && jsx("div", {
          className: styles.line
        }, "separator"), servicesWithOutIcons.map(service => {
          const isActive = this.serviceIsActive(service);
          return jsx(Services.Link, {
            active: isActive,
            className: isActive ? styles.activeItemStacked : styles.itemStacked,
            service: service
          }, service.id);
        })]
      })
    }));
  }
}
_defineProperty(Services, "sort", (a, b) => {
  const aApplicationName = a.applicationName || '';
  const bApplicationName = b.applicationName || '';
  return aApplicationName.localeCompare(bApplicationName) || a.name.localeCompare(b.name);
});
_defineProperty(Services, "defaultProps", {
  theme: Theme.DARK
});
_defineProperty(Services, "Link", ServicesLink);

export { Services as default };
