import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import '../auth/auth.js';
import alertService from '../alert-service/alert-service.js';
import Profile from './profile.js';
import { USER_CHANGED_EVENT, LOGOUT_POSTPONED_EVENT, USER_CHANGE_POSTPONED_EVENT } from '../auth/auth__core.js';
import '../auth/window-flow.js';
import '../auth/response-parser.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'es6-error';
import '../global/url.js';
import 'core-js/modules/es.string.match.js';
import '../auth/down-notification.js';
import '../alert/alert.js';
import 'classnames';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/data-tests.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import 'react-dom';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../alert/container.js';
import '../link/link.js';
import '../_helpers/link.js';
import '../group/group.js';
import 'react-dom/client';
import '../avatar/avatar.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import '../dropdown-menu/dropdown-menu.js';
import '../list/list.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.trim.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'memoize-one';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../dropdown/dropdown.js';
import '../global/typescript-utils.js';
import '../_helpers/anchor.js';
import '../popup-menu/popup-menu.js';
import 'core-js/modules/es.parse-float.js';
import '../i18n/i18n-context.js';
import '../i18n/i18n.js';
import '../_helpers/header.js';
import 'core-js/modules/es.parse-int.js';
import 'core-js/modules/es.string.search.js';
import 'core-js/modules/web.url.js';
import 'core-js/modules/web.url.to-json.js';
import 'core-js/modules/web.url-search-params.js';
import '../global/listeners.js';
import '../http/http.js';
import '../global/promise-with-timeout.js';
import '../auth/storage.js';
import '../storage/storage__local.js';
import '../auth/request-builder.js';
import 'simply-uuid';
import '../auth/background-flow.js';
import '../auth/token-validator.js';

const _excluded = ["auth", "profileUrl"];
const CERTIFICATE_MISMATCH_HEADER = 'x-client-certificate-token-mismatch';
class SmartProfile extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      user: null,
      size: Profile.defaultProps.size,
      isLogoutPostponed: false,
      isUserChangePostponed: false
    });
    _defineProperty(this, "login", async () => {
      this.setState({
        loading: true
      });
      try {
        await this.props.auth.login();
      } catch (err) {
        // eslint-disable-next-line no-console
        console.debug('Profile login errored', err);
      } finally {
        this.setState({
          loading: false
        });
      }
    });
    _defineProperty(this, "logout", () => this.props.auth.logout());
    _defineProperty(this, "switchUser", () => this.props.auth.switchUser());
    _defineProperty(this, "onRevertPostponement", () => {
      if (this.state.isLogoutPostponed) {
        this.props.auth.login();
      }
      if (this.state.isUserChangePostponed) {
        this.props.auth.updateUser();
      }
    });
  }
  componentDidMount() {
    this.requestUser();
  }
  async requestUser() {
    try {
      const {
        auth
      } = this.props;
      const user = await auth.requestUser();
      this.checkUserCertificateMismatch(user);
      this.setState({
        user
      });
      auth.addListener(USER_CHANGED_EVENT, newUser => {
        this.setState({
          user: newUser,
          isLogoutPostponed: false,
          isUserChangePostponed: false
        });
      });
      auth.addListener(LOGOUT_POSTPONED_EVENT, () => {
        this.setState({
          isLogoutPostponed: true
        });
      });
      auth.addListener(USER_CHANGE_POSTPONED_EVENT, () => {
        this.setState({
          isUserChangePostponed: true
        });
      });
    } catch (e) {
      // noop
    }
  }
  checkUserCertificateMismatch(user) {
    var _userMeta$headers;
    const {
      auth,
      translations
    } = this.props;
    const userMeta = auth.http.getMetaForResponse(user);
    if (userMeta !== null && userMeta !== void 0 && (_userMeta$headers = userMeta.headers) !== null && _userMeta$headers !== void 0 && _userMeta$headers.has(CERTIFICATE_MISMATCH_HEADER)) {
      const message = (translations === null || translations === void 0 ? void 0 : translations.certificateMismatch) || "You are authenticated as ".concat(user.login || user.name, ". To authenticate with the client certificate for your account, log out, then click the \"Log in with certificate\" option on the login page.");
      alertService.warning(message, 0);
    }
  }
  render() {
    const {
      user,
      loading,
      isLogoutPostponed,
      isUserChangePostponed
    } = this.state;
    const _this$props = this.props,
      {
        auth,
        profileUrl
      } = _this$props,
      props = _objectWithoutProperties(_this$props, _excluded);
    const url = profileUrl || (user ? "".concat(auth.config.serverUri, "users/").concat(user.id) : '');
    return jsx(Profile, _objectSpread2({
      onLogin: this.login,
      onLogout: this.logout,
      onSwitchUser: this.switchUser,
      loading: loading,
      user: user,
      profileUrl: url,
      showApplyChangedUser: isUserChangePostponed,
      showLogIn: isLogoutPostponed,
      showLogOut: !isLogoutPostponed,
      showSwitchUser: auth._canShowDialogs() && !isLogoutPostponed && !isUserChangePostponed,
      onRevertPostponement: this.onRevertPostponement
    }, props));
  }
}
_defineProperty(SmartProfile, "Size", Profile.Size);

export { SmartProfile as default };
