import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.weak-map.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import ExtendableError from 'es6-error';
import { encodeURL, joinBaseURLAndPath } from '../global/url.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';

const _excluded = ["body", "query"],
  _excluded2 = ["headers"],
  _excluded3 = ["headers", "body", "query", "sendRawBody"];
/**
 * @name HTTP
 */
const TOKEN_TYPE = 'Bearer';
const defaultFetchConfig = {
  headers: {
    'Content-Type': 'application/json',
    Accept: 'application/json'
  },
  credentials: 'same-origin'
};
class HTTPError extends ExtendableError {
  constructor(response) {
    let data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    super("".concat(response.status, " ").concat(response.statusText || ''));
    _defineProperty(this, "data", void 0);
    _defineProperty(this, "status", void 0);
    this.data = data;
    this.status = response.status;
  }
}
const CODE = {
  UNAUTHORIZED: 401
};
function isRawBody(params) {
  return params.sendRawBody === true;
}
class HTTP {
  constructor(_auth, _baseUrl) {
    var _this = this;
    let _fetchConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    _defineProperty(this, "baseUrl", null);
    _defineProperty(this, "_requestsMeta", new WeakMap());
    _defineProperty(this, "fetchConfig", void 0);
    _defineProperty(this, "requestToken", void 0);
    _defineProperty(this, "shouldRefreshToken", void 0);
    _defineProperty(this, "forceTokenUpdate", void 0);
    _defineProperty(this, "setAuth", auth => {
      this.requestToken = () => auth.requestToken();
      this.shouldRefreshToken = auth.constructor.shouldRefreshToken;
      this.forceTokenUpdate = () => auth.forceTokenUpdate();
    });
    _defineProperty(this, "setBaseUrl", baseUrl => {
      this.baseUrl = baseUrl;
    });
    _defineProperty(this, "fetch", async function (url) {
      let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      const {
          body,
          query = {}
        } = params,
        fetchConfig = _objectWithoutProperties(params, _excluded);
      const response = await _this._fetch(_this._makeRequestUrl(url, query), _objectSpread2(_objectSpread2({}, fetchConfig), {}, {
        headers: fetchConfig.headers,
        body: body !== null && body !== undefined && body !== '' ? JSON.stringify(body) : body
      }));
      return _this._processResponse(response);
    });
    _defineProperty(this, "request", async (url, params) => {
      var _this$requestToken;
      let token = await ((_this$requestToken = this.requestToken) === null || _this$requestToken === void 0 ? void 0 : _this$requestToken.call(this));
      let response = await this._performRequest(url, token, params);
      try {
        // Wait for result to catch an HTTP error
        return await this._processResponse(response);
      } catch (error) {
        var _this$shouldRefreshTo;
        if (!(error instanceof HTTPError)) {
          throw error;
        }
        const shouldRefreshToken = typeof error.data.error === 'string' ? (_this$shouldRefreshTo = this.shouldRefreshToken) === null || _this$shouldRefreshTo === void 0 ? void 0 : _this$shouldRefreshTo.call(this, error.data.error) : false;
        if (shouldRefreshToken) {
          var _this$forceTokenUpdat;
          token = await ((_this$forceTokenUpdat = this.forceTokenUpdate) === null || _this$forceTokenUpdat === void 0 ? void 0 : _this$forceTokenUpdat.call(this));
          response = await this._performRequest(url, token, params);
          return this._processResponse(response);
        }
        throw error;
      }
    });
    _defineProperty(this, "getMetaForResponse", response => this._requestsMeta.get(response));
    _defineProperty(this, "get", (url, params) => this.request(url, _objectSpread2(_objectSpread2({}, params), {}, {
      method: 'GET'
    })));
    _defineProperty(this, "post", (url, params) => this.request(url, _objectSpread2(_objectSpread2({}, params), {}, {
      method: 'POST'
    })));
    _defineProperty(this, "delete", (url, params) => this.request(url, _objectSpread2(_objectSpread2({}, params), {}, {
      method: 'DELETE'
    })));
    _defineProperty(this, "put", (url, params) => this.request(url, _objectSpread2(_objectSpread2({}, params), {}, {
      method: 'PUT'
    })));
    /**
     * Usage: const {promise, abort} = http.abortify(http.get<{id: string}>)('http://test.com');
     * @param method
     */
    _defineProperty(this, "abortify", method => function () {
      for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
        _ref[_key] = arguments[_key];
      }
      let [url, params] = _ref;
      const ctrl = new AbortController();
      if (params && !('signal' in params)) {
        params.signal = ctrl.signal;
      }
      return {
        promise: method.call(_this, url, params),
        abort: () => ctrl.abort()
      };
    });
    if (_auth) {
      this.setAuth(_auth);
    }
    this.setBaseUrl(_baseUrl);
    const {
        headers
      } = defaultFetchConfig,
      defaultConfig = _objectWithoutProperties(defaultFetchConfig, _excluded2);
    this.fetchConfig = _objectSpread2(_objectSpread2(_objectSpread2({}, defaultConfig), _fetchConfig), {}, {
      headers: _objectSpread2(_objectSpread2({}, headers), _fetchConfig.headers)
    });
  }
  _fetch() {
    return fetch(...arguments);
  }
  _makeRequestUrl(url, queryObject) {
    const urlWithQuery = encodeURL(url, queryObject);
    return joinBaseURLAndPath(this.baseUrl, urlWithQuery);
  }
  _performRequest(url, token) {
    let params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    const {
        headers,
        body,
        query = {},
        sendRawBody
      } = params,
      fetchConfig = _objectWithoutProperties(params, _excluded3);
    const combinedHeaders = _objectSpread2(_objectSpread2(_objectSpread2({}, this.fetchConfig.headers), token ? {
      Authorization: "".concat(TOKEN_TYPE, " ").concat(token)
    } : {}), headers);
    Object.entries(combinedHeaders).forEach(_ref2 => {
      let [key, header] = _ref2;
      if (header === null || header === undefined) {
        Reflect.deleteProperty(combinedHeaders, key);
      }
    });
    let bodyToSend;
    if (isRawBody(params)) {
      bodyToSend = params.body;
    } else {
      bodyToSend = params.body === null || params.body === undefined || params.body === '' ? params.body : JSON.stringify(body);
    }
    return this._fetch(this._makeRequestUrl(url, query), _objectSpread2(_objectSpread2(_objectSpread2({}, this.fetchConfig), {}, {
      headers: combinedHeaders
    }, fetchConfig), {}, {
      body: bodyToSend
    }));
  }
  _storeRequestMeta(parsedResponse, rawResponse) {
    const {
      headers,
      ok,
      redirected,
      status,
      statusText,
      type,
      url
    } = rawResponse;
    this._requestsMeta.set(parsedResponse, {
      headers,
      ok,
      redirected,
      status,
      statusText,
      type,
      url
    });
  }
  async _processResponse(response) {
    const contentType = response.headers.get('content-type');
    const isJson = contentType && contentType.indexOf('application/json') !== -1;
    if (!response.ok) {
      let resJson;
      try {
        resJson = await (isJson ? response.json() : response.text());
      } catch (err) {
        // noop
      }
      throw new HTTPError(response, resJson);
    }
    try {
      const parsedResponse = await (isJson ? response.json() : {
        data: await response.text()
      });
      this._storeRequestMeta(parsedResponse, response);
      return parsedResponse;
    } catch (err) {
      return response;
    }
  }
  async authorizedFetch() {
    const response = await this._performRequest(...arguments);
    return this._processResponse(response);
  }
}

export { CODE, HTTPError, HTTP as default, defaultFetchConfig };
