import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.regexp.constructor.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import HTTP from './http.js';
import 'core-js/modules/es.weak-map.js';
import 'es6-error';
import '../global/url.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';

const authMock = {
  requestToken: () => 'mock token',
  shouldRefreshToken: () => false,
  forceTokenUpdate: () => Promise.resolve(null)
};
class HTTPMock extends HTTP {
  constructor() {
    super(authMock);
    _defineProperty(this, "defaultResponse", void 0);
    _defineProperty(this, "requests", void 0);
    _defineProperty(this, "responsesByUrlMap", void 0);
    this.defaultResponse = null;
    this.requests = [];
    this.responsesByUrlMap = new Map();
  }
  _fetch(url, params) {
    this.requests = [...this.requests, {
      url,
      params: _objectSpread2(_objectSpread2({}, params), {}, {
        body: typeof params.body === 'string' ? JSON.parse(params.body) : params.body
      })
    }];
    return Promise.resolve({
      status: 200,
      ok: true,
      headers: new Headers({
        'content-type': 'application/json'
      }),
      json: () => Promise.resolve(this._getResponseForUrl(url) || this.defaultResponse)
    });
  }
  respondDefault(response) {
    this.defaultResponse = response;
  }
  respondForUrl(url, response) {
    this.responsesByUrlMap.set(url, response);
  }
  _getResponseForUrl(urlToMatch) {
    const urls = this.responsesByUrlMap.keys();
    for (const url of urls) {
      if (url === urlToMatch) {
        return this.responsesByUrlMap.get(url);
      }
      if (url instanceof RegExp && url.test(urlToMatch)) {
        return this.responsesByUrlMap.get(url);
      }
    }
    return null;
  }
  getRequestsByUrlPart(url) {
    return this.requests.filter(it => it.url.indexOf(url) !== -1);
  }
}

export { HTTPMock as default };
