import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import HubSource from './hub-source.js';

const defaultOptions = {
  searchMax: 20,
  searchSideThreshold: 200
};
class HubSourceUsersGroups {
  constructor(auth, options) {
    _defineProperty(this, "auth", void 0);
    _defineProperty(this, "options", void 0);
    _defineProperty(this, "usersSource", void 0);
    _defineProperty(this, "groupsSource", void 0);
    this.auth = auth;
    this.options = Object.assign({}, defaultOptions, options);
    this.usersSource = new HubSource(auth, 'users', {
      searchMax: this.options.searchMax,
      searchSideThreshold: this.options.searchSideThreshold,
      queryFormatter: query => "nameStartsWith: ".concat(HubSourceUsersGroups.wrapMultiwordQuery(query), " or loginStartsWith: ").concat(HubSourceUsersGroups.wrapMultiwordQuery(query))
    });
    this.groupsSource = new HubSource(auth, 'usergroups', {
      searchMax: this.options.searchMax,
      searchSideThreshold: this.options.searchSideThreshold
    });
  }
  static wrapMultiwordQuery(query) {
    if (query && query.indexOf(' ') !== -1) {
      return "{".concat(query, "}");
    }
    return query;
  }
  createUsersFilterFn(query) {
    if (!query) {
      return () => true;
    }
    const normalizedQuery = query.toLowerCase();
    return it => it.name.toLowerCase().indexOf(normalizedQuery) !== -1 || it.login.toLowerCase().indexOf(normalizedQuery) !== -1;
  }
  getUsers() {
    let query = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    return this.usersSource.get(query, {
      fields: 'id,name,login,total,profile/avatar/url',
      orderBy: 'name'
    }, this.createUsersFilterFn(query));
  }
  getGroups() {
    let query = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    return this.groupsSource.get(query, {
      fields: 'id,name,total,userCount,iconUrl',
      orderBy: 'name'
    });
  }
  getUserAndGroups(query) {
    return Promise.all([this.getUsers(query), this.getGroups(query)]);
  }
}

export { HubSourceUsersGroups as default };
