import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import deprecate from 'util-deprecate';
import { Color, Size } from './icon__constants.js';
import { s as styles, I as IconSVG } from '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import 'core-js/modules/web.dom-collections.iterator.js';

const _excluded = ["className", "size", "color", "loading", "glyph", "width", "height", "suppressSizeWarning"];
const warnSize = deprecate(() => {}, "`size`, `width` and `height` props are not recommended to use in Ring UI `Icon` component. The intrinsic sizes of SVG icon (`width` and `height` SVG attributes) are used instead.\n\nWe strongly recommend to use icons handcrafted for particular sizes. If your icon doesn't exist in the desired size, please ask your designer to draw one. \"Responsive\" checkmark should be unchecked when exporting icon.'");
class Icon extends PureComponent {
  warnSize() {
    if (this.props.suppressSizeWarning) {
      return;
    }
    warnSize();
  }
  getStyle() {
    const {
      size,
      width,
      height
    } = this.props;
    if (width || height) {
      this.warnSize();
      return {
        width,
        height
      };
    }
    if (size) {
      this.warnSize();
      return {
        width: size,
        height: size
      };
    }
    return undefined;
  }
  render() {
    const _this$props = this.props,
      {
        className,
        size,
        color,
        loading,
        glyph: Glyph,
        width,
        height,
        suppressSizeWarning
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    if (!Glyph) {
      return null;
    }
    const classes = classNames(styles.icon, {
      [styles[color]]: !!color,
      [styles.loading]: loading
    }, className);
    return jsx("span", _objectSpread2(_objectSpread2({
      "data-test": "ring-icon"
    }, restProps), {}, {
      className: classes,
      children: typeof Glyph === 'string' ? jsx(IconSVG, {
        src: Glyph,
        style: this.getStyle()
      }) : jsx(Glyph, {
        className: styles.glyph,
        style: this.getStyle()
      })
    }));
  }
}
_defineProperty(Icon, "defaultProps", {
  className: '',
  color: Color.DEFAULT,
  glyph: ''
});
_defineProperty(Icon, "Color", Color);
/**
 * @deprecated Use icons with appropriate intrinsic sizes instead
 */
_defineProperty(Icon, "Size", Size);

export { Color, Size, Icon as default };
