import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { forwardRef, Component } from 'react';
import classNames from 'classnames';
import createResizeDetector from 'element-resize-detector';
import scheduleRAF from '../global/schedule-raf.js';
import { s as styles } from '../_helpers/island.js';
import { ScrollHandlerContext } from './adaptive-island-hoc.js';
import '../global/linear-function.js';

const _excluded = ["children", "className", "bottomBorder", "scrollableWrapperClassName", "onScroll", "onScrollToBottom", "fade", "tabIndex"];
const scheduleScrollAction = scheduleRAF();
const noop = () => {};
const END_DISTANCE = 16;
class Content extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      scrolledToTop: true,
      scrolledToBottom: false
    });
    _defineProperty(this, "resizeDetector", createResizeDetector({
      strategy: 'scroll'
    }));
    _defineProperty(this, "wrapperNode", void 0);
    _defineProperty(this, "setWrapper", node => {
      if (!node) {
        return;
      }
      this.wrapperNode = node;
      this.resizeDetector.listenTo(node, this.calculateScrollPosition);
    });
    _defineProperty(this, "calculateScrollPosition", () => scheduleScrollAction(() => {
      const {
        scrollableNode
      } = this;
      if (!scrollableNode) {
        return;
      }
      this.props.onScroll(scrollableNode);
      const {
        scrollTop,
        scrollHeight,
        offsetHeight
      } = scrollableNode;
      const scrolledToTop = scrollTop === 0;
      const scrolledToBottom = offsetHeight + scrollTop >= scrollHeight - END_DISTANCE;
      if (scrolledToBottom) {
        var _this$props$onScrollT, _this$props;
        (_this$props$onScrollT = (_this$props = this.props).onScrollToBottom) === null || _this$props$onScrollT === void 0 || _this$props$onScrollT.call(_this$props);
      }
      this.setState({
        scrolledToTop,
        scrolledToBottom
      });
    }));
    _defineProperty(this, "scrollableNode", void 0);
    _defineProperty(this, "setScrollableNodeAndCalculatePosition", node => {
      if (!node) {
        return;
      }
      this.scrollableNode = node;
      this.calculateScrollPosition();
    });
  }
  componentWillUnmount() {
    this.scrollableNode = null;
    if (!this.wrapperNode) {
      return;
    }
    this.resizeDetector.removeAllListeners(this.wrapperNode);
  }
  render() {
    const _this$props2 = this.props,
      {
        children,
        className,
        bottomBorder,
        scrollableWrapperClassName,
        onScroll,
        onScrollToBottom,
        fade,
        tabIndex
      } = _this$props2,
      restProps = _objectWithoutProperties(_this$props2, _excluded);
    const {
      scrolledToTop,
      scrolledToBottom
    } = this.state;
    const classes = classNames(styles.content, className, {
      [styles.contentWithTopFade]: fade && !scrolledToTop,
      [styles.contentWithBottomFade]: fade && !scrolledToBottom,
      [styles.withTransparentBottomBorder]: bottomBorder,
      [styles.withBottomBorder]: bottomBorder && !scrolledToBottom
    });
    const scrollableWrapperClasses = classNames(styles.scrollableWrapper, scrollableWrapperClassName);
    return jsx("div", _objectSpread2(_objectSpread2({}, restProps), {}, {
      "data-test": "ring-island-content",
      className: classes,
      children: jsxs("div", {
        tabIndex: tabIndex,
        className: scrollableWrapperClasses,
        ref: this.setScrollableNodeAndCalculatePosition,
        onScroll: fade ? this.calculateScrollPosition : noop,
        children: [fade && jsx("div", {
          ref: this.setWrapper,
          children: children
        }), !fade && children]
      })
    }));
  }
}
_defineProperty(Content, "defaultProps", {
  fade: true,
  bottomBorder: false,
  onScroll: noop,
  onScrollToBottom: noop
});
const ContentWrapper = /*#__PURE__*/forwardRef((props, ref) => jsx(ScrollHandlerContext.Consumer, {
  children: onScroll => {
    const addProps = onScroll != null ? {
      onScroll,
      bottomBorder: true
    } : {};
    return jsx(Content, _objectSpread2(_objectSpread2(_objectSpread2({}, props), addProps), {}, {
      ref: ref
    }));
  }
}));
ContentWrapper.displayName = 'ContentWrapper';

export { ContentWrapper as default };
