import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import { interpolateLinear } from '../global/linear-function.js';
import { s as styles } from '../_helpers/island.js';
import { PhaseContext } from './adaptive-island-hoc.js';
import 'core-js/modules/web.dom-collections.iterator.js';

const _excluded = ["children", "className", "wrapWithTitle", "border", "phase"];
const Start = {
  FONT_SIZE: 24,
  LINE_HEIGHT: 28,
  PADDING_TOP: 24,
  PADDING_BOTTOM: 0,
  X: 0,
  Y: 0,
  SPACING: 0
};
const End = {
  FONT_SIZE: 13,
  LINE_HEIGHT: 20,
  PADDING_TOP: 16,
  PADDING_BOTTOM: 8,
  // Compensation
  X: 0.4,
  Y: 0.1,
  SPACING: 1.09
};
const BORDER_APPEAR_PHASE = 0.5;
class Header extends Component {
  style(name) {
    var _this$props$phase;
    return interpolateLinear(Start[name], End[name], (_this$props$phase = this.props.phase) !== null && _this$props$phase !== void 0 ? _this$props$phase : 0);
  }
  render() {
    const _this$props = this.props,
      {
        children,
        className,
        wrapWithTitle,
        border,
        phase
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const classes = classNames(styles.header, className, {
      [styles.withBottomBorder]: border || phase != null && phase >= BORDER_APPEAR_PHASE
    });
    const headerStyle = phase != null ? {
      lineHeight: "".concat(this.style('LINE_HEIGHT'), "px"),
      // need to append px because number is a valid line-height value
      paddingTop: this.style('PADDING_TOP'),
      paddingBottom: this.style('PADDING_BOTTOM')
    } : undefined;
    const scaleFont = phase != null && this.style('FONT_SIZE') / Start.FONT_SIZE;
    const titleStyle = phase != null && phase < 1 ? {
      fontSize: Start.FONT_SIZE,
      transform: "translate(".concat(this.style('X'), "px, ").concat(this.style('Y'), "px) scale(").concat(scaleFont, ")"),
      letterSpacing: this.style('SPACING')
    } : undefined;
    return jsxs("div", _objectSpread2(_objectSpread2({}, restProps), {}, {
      "data-test": "ring-island-header",
      className: classes,
      style: headerStyle,
      children: [wrapWithTitle && jsx("h2", {
        className: styles.title,
        style: titleStyle,
        children: children
      }), !wrapWithTitle && children]
    }));
  }
}
_defineProperty(Header, "defaultProps", {
  wrapWithTitle: true
});
const HeaderWrapper = props => jsx(PhaseContext.Consumer, {
  children: phase => {
    const addProps = phase != null ? {
      phase
    } : {};
    return jsx(Header, _objectSpread2(_objectSpread2({}, props), addProps));
  }
});

export { HeaderWrapper as default };
