import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import joinDataTestAttributes from '../global/data-tests.js';
import Avatar from '../avatar/avatar.js';
import Checkbox from '../checkbox/checkbox.js';
import Icon from '../icon/icon.js';
import getUID from '../global/get-uid.js';
import Link, { linkHOC } from '../link/link.js';
import { s as styles } from '../_helpers/list.js';
import { Type } from './consts.js';
import { getListClasses } from './list__classes.js';
import { Size } from '../avatar/avatar-size.js';
import 'core-js/modules/es.string.includes.js';
import 'util-deprecate';
import '../global/url.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../_helpers/avatar-info.js';
import '../avatar/fallback-avatar.js';
import 'core-js/modules/es.regexp.to-string.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../global/composeRefs.js';
import 'memoize-one';
import '../control-help/control-help.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';

const _excluded = ["disabled", "checkbox", "avatar", "subavatar", "glyph", "icon", "rightGlyph", "description", "label", "title", "details", "hover", "level", "tabIndex", "onClick", "onCheckboxChange", "onMouseOver", "onMouseDown", "onMouseUp", "rightNodes", "leftNodes", "showGeneratedAvatar", "username", "labelWrapper", "rgItemType", "scrolling", "data-test", "className", "url", "LinkComponent", "compact", "hoverClassName", "children"];
/**
 * @constructor
 * @extends {ReactComponent}
 */
const RING_UNIT = 8;
const DEFAULT_PADDING = 8;
const CHECKBOX_WIDTH = 28;
class ListItem extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "id", getUID('list-item-'));
    _defineProperty(this, "stopBubbling", e => e.stopPropagation());
    _defineProperty(this, "_isString", val => typeof val === 'string' || val instanceof String);
  }
  render() {
    const _this$props = this.props,
      {
        disabled,
        checkbox,
        avatar,
        subavatar,
        glyph,
        icon,
        rightGlyph,
        description,
        label,
        title,
        details,
        hover,
        level,
        tabIndex,
        onClick,
        onCheckboxChange,
        onMouseOver,
        onMouseDown,
        onMouseUp,
        rightNodes,
        leftNodes,
        showGeneratedAvatar,
        username,
        labelWrapper,
        rgItemType,
        scrolling,
        'data-test': dataTest,
        className,
        url,
        LinkComponent,
        compact,
        hoverClassName,
        children // TODO use an allow list in 8.0
      } = _this$props,
      restLinkProps = _objectWithoutProperties(_this$props, _excluded);
    const checkable = checkbox !== undefined;
    const shouldShowGeneratedAvatar = showGeneratedAvatar && username != null;
    const hasLeftNodes = leftNodes || glyph || avatar || shouldShowGeneratedAvatar;
    const showCheckbox = checkable && (checkbox || !hasLeftNodes || hover && !disabled);
    const classes = getListClasses(this.props);
    const detailsClasses = classNames({
      [styles.details]: details,
      [styles.padded]: icon !== undefined || checkbox !== undefined || glyph !== undefined
    });
    const style = {
      paddingLeft: "".concat((Number(level) || 0) * RING_UNIT + DEFAULT_PADDING + (showCheckbox ? CHECKBOX_WIDTH : 0), "px")
    };
    let computedTitle = null;
    if (this._isString(title)) {
      // if title is specified and is a string then use it
      computedTitle = title;
    } else {
      // otherwise use label if it is a string;
      // label can also be an element, use empty string in this case
      computedTitle = this._isString(label) ? label : '';
    }
    const isLink = rgItemType === Type.LINK;
    const combinedDataTest = joinDataTestAttributes({
      'ring-list-item': (dataTest || '').indexOf('ring-list-item') === -1,
      'ring-list-item-action': !disabled,
      'ring-list-item-selected': checkbox,
      'ring-list-link': isLink
    }, dataTest);
    const labelElement = jsx("span", {
      className: styles.label,
      title: computedTitle,
      "data-test": "ring-list-item-label",
      children: label !== null && label !== void 0 ? label : children
    });
    const commonProps = {
      id: this.id,
      tabIndex,
      onClick,
      onMouseOver,
      onMouseDown,
      onFocus: onMouseOver,
      onMouseUp,
      className: classes,
      style,
      disabled,
      children: jsxs(Fragment, {
        children: [jsxs("div", {
          className: styles.top,
          onMouseOut: this.stopBubbling,
          onBlur: this.stopBubbling,
          children: [!showCheckbox && jsxs("div", {
            className: styles.left,
            children: [leftNodes, glyph && jsx(Icon, {
              className: styles.glyph,
              glyph: glyph,
              size: this.props.iconSize,
              suppressSizeWarning: this.props.suppressSizeWarning
            }), (avatar || shouldShowGeneratedAvatar) && jsx(Avatar, {
              className: styles.avatar,
              url: avatar,
              size: Size.Size20,
              subavatar: subavatar,
              username: username
            })]
          }), labelWrapper ? labelWrapper(labelElement) : labelElement, description && jsx("span", {
            className: styles.description,
            "data-test": "ring-list-item-description",
            children: description
          }), jsxs("div", {
            className: styles.right,
            children: [rightGlyph && jsx(Icon, {
              className: styles.rightGlyph,
              glyph: rightGlyph,
              suppressSizeWarning: this.props.suppressSizeWarning,
              size: this.props.iconSize
            }), icon && jsx("div", {
              className: styles.icon,
              style: {
                backgroundImage: "url(\"".concat(icon, "\")")
              }
            }), rightNodes]
          })]
        }), details && jsx("div", {
          className: detailsClasses,
          children: details
        })]
      })
    };
    const LinkComponentToUse = LinkComponent ? linkHOC(LinkComponent) : Link;
    return jsxs("div", {
      className: styles.itemContainer,
      "data-test": combinedDataTest,
      children: [showCheckbox && jsx("div", {
        className: styles.checkboxContainer,
        children: jsx(Checkbox, {
          "aria-labelledby": this.id,
          checked: checkbox,
          disabled: disabled,
          onChange: onCheckboxChange,
          onClick: this.stopBubbling
        })
      }), isLink ? jsx(LinkComponentToUse, _objectSpread2(_objectSpread2({
        pseudo: !restLinkProps.href
      }, commonProps), restLinkProps)) : jsx("button", _objectSpread2({
        type: "button"
      }, commonProps))]
    });
  }
}

export { ListItem as default };
