import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import GroupIcon from '@jetbrains/icons/group';
import HubSourceUsersGroups from '../hub-source/hub-source__users-groups.js';
import List from './list.js';
import '../hub-source/hub-source.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.trim.js';
import 'react';
import 'react/jsx-runtime';
import 'classnames';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'util-deprecate';
import 'memoize-one';
import '../global/data-tests.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../global/schedule-raf.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/dom.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/create-stateful-context.js';
import './list__item.js';
import '../avatar/avatar.js';
import '../global/url.js';
import 'core-js/modules/es.string.replace.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../link/link.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import './consts.js';
import './list__classes.js';
import './list__custom.js';
import '../global/get-event-key.js';
import './list__title.js';
import './list__separator.js';
import './list__hint.js';

const defaultOptions = {
  GroupsTitle: 'Groups',
  NoGroupsTitle: 'No groups',
  UsersTitle: 'Users',
  NoUsersTitle: 'No users',
  getPluralForUserCount: count => {
    // eslint-disable-next-line @typescript-eslint/no-magic-numbers
    const plural = count % 10 !== 1 || count % 100 === 11;
    return "".concat(count, " member").concat(plural ? 's' : '');
  }
};
var Filter;
(function (Filter) {
  Filter[Filter["ALL"] = 0] = "ALL";
  Filter[Filter["USERS"] = 1] = "USERS";
  Filter[Filter["GROUPS"] = 2] = "GROUPS";
})(Filter || (Filter = {}));
class ListUsersGroupsSource extends HubSourceUsersGroups {
  constructor(auth, options) {
    super(auth, options);
    _defineProperty(this, "listSourceOptions", void 0);
    this.listSourceOptions = Object.assign({}, defaultOptions, options);
  }
  getGroupsSectionTitle(groups) {
    return groups.length ? this.listSourceOptions.GroupsTitle : this.listSourceOptions.NoGroupsTitle;
  }
  getUsersSectionTitle(users) {
    return users.length ? this.listSourceOptions.UsersTitle : this.listSourceOptions.NoUsersTitle;
  }
  async getForList(query) {
    let filter = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Filter.ALL;
    const [users, groups] = await this.getUserAndGroups(query);
    const items = [];
    if (filter === Filter.ALL) {
      items.push({
        rgItemType: List.ListProps.Type.SEPARATOR,
        key: 2,
        description: this.getUsersSectionTitle(users)
      });
    }
    if (filter !== Filter.GROUPS) {
      users.forEach(user => {
        var _user$profile$avatar;
        return items.push(_objectSpread2(_objectSpread2({}, user), {}, {
          key: user.id,
          label: user.name,
          avatar: user.profile ? (_user$profile$avatar = user.profile.avatar) === null || _user$profile$avatar === void 0 ? void 0 : _user$profile$avatar.url : null,
          description: user.login
        }));
      });
    }
    if (filter === Filter.ALL) {
      items.push({
        rgItemType: List.ListProps.Type.SEPARATOR,
        key: 1,
        description: this.getGroupsSectionTitle(groups)
      });
    }
    if (filter !== Filter.USERS) {
      groups.forEach(group => items.push(_objectSpread2(_objectSpread2({}, group), {}, {
        key: group.id,
        label: group.name,
        avatar: group.iconUrl,
        glyph: group.iconUrl ? null : GroupIcon,
        description: this.listSourceOptions.getPluralForUserCount(group.userCount)
      })));
    }
    return items;
  }
}
_defineProperty(ListUsersGroupsSource, "Filter", Filter);

export { ListUsersGroupsSource as default };
