import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import Dialog from '../dialog/dialog.js';
import 'classnames';
import '../_helpers/island.js';
import '../island/header.js';
import ContentWrapper from '../island/content.js';
import LoaderScreen from '../loader-screen/loader-screen.js';
import { HUB_AUTH_PAGE_OPENED } from '../auth/background-flow.js';
import 'react-dom';
import '@jetbrains/icons/close';
import '../island/island.js';
import '../global/data-tests.js';
import '../island/adaptive-island-hoc.js';
import '../global/linear-function.js';
import 'element-resize-detector';
import '../global/schedule-raf.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../_helpers/dialog__body-scroll-preventer.js';
import 'scrollbar-width';
import '../loader/loader.js';
import '../loader/loader__core.js';
import 'core-js/modules/es.promise.js';
import '../auth/response-parser.js';
import 'es6-error';
import '../global/url.js';

var styles = {"iFrame":"iFrame_rui_ea70","dialogContent":"dialogContent_rui_ea70","nonOpaqueLoader":"nonOpaqueLoader_rui_ea70","fallbackLinkContainer":"fallbackLinkContainer_rui_ea70"};

const HUB_AUTH_PAGE_LOGIN_STARTED = 'HUB_AUTH_PAGE_LOGIN_STARTED';
const HUB_AUTH_PAGE_LOGIN_DIMENSIONS = 'HUB_AUTH_PAGE_LOGIN_DIMENSIONS';
const DEFAULT_HEIGHT = 517;
const DEFAULT_WIDTH = 333;
const DEFAULT_SHOW_FALLBACK_TIMEOUT = 5000;
/**
 * @name Login Dialog
 */
class LoginDialog extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      loading: true,
      loggingIn: false,
      showFallbackLink: false,
      height: DEFAULT_HEIGHT,
      width: DEFAULT_WIDTH
    });
    _defineProperty(this, "showFallbackTimout", void 0);
    _defineProperty(this, "onMessage", event => {
      const {
        data
      } = event;
      if (!data) {
        return;
      }
      if (data === HUB_AUTH_PAGE_OPENED) {
        clearTimeout(this.showFallbackTimout);
        this.setState({
          loading: false,
          loggingIn: false
        });
        return;
      }
      if (data === HUB_AUTH_PAGE_LOGIN_STARTED) {
        this.setState({
          loading: true,
          loggingIn: true
        });
        this.startFallbackCountdown();
        return;
      }
      if (data.message === HUB_AUTH_PAGE_LOGIN_DIMENSIONS) {
        this.setState({
          height: data.height,
          width: data.width
        });
      }
    });
  }
  componentDidMount() {
    window.addEventListener('message', this.onMessage);
    this.startFallbackCountdown();
  }
  componentWillUnmount() {
    window.removeEventListener('message', this.onMessage);
  }
  startFallbackCountdown() {
    this.showFallbackTimout = window.setTimeout(() => this.setState({
      showFallbackLink: true
    }), this.props.showFallbackTimeout);
  }
  render() {
    const {
      show,
      className,
      url,
      loadingMessage,
      renderFallbackLink,
      onCancel
    } = this.props;
    const {
      loading,
      height,
      width,
      loggingIn,
      showFallbackLink
    } = this.state;
    const iFrameStyle = {
      height,
      width
    };
    return jsxs(Dialog, {
      "data-test": "ring-login-dialog",
      className: className,
      contentClassName: styles.dialogContent,
      trapFocus: true,
      autoFocusFirst: false,
      show: show,
      showCloseButton: true,
      onCloseAttempt: onCancel,
      children: [jsx(ContentWrapper, {
        children: jsx("iframe", {
          title: "Login dialog",
          style: iFrameStyle,
          src: url,
          className: styles.iFrame,
          scrolling: "no"
        })
      }), loading && jsx(LoaderScreen, {
        message: loadingMessage,
        containerClassName: styles.nonOpaqueLoader
      }), showFallbackLink && jsx("div", {
        className: styles.fallbackLinkContainer,
        children: renderFallbackLink(loggingIn)
      })]
    });
  }
}
_defineProperty(LoginDialog, "defaultProps", {
  show: false,
  url: 'about:blank',
  renderFallbackLink: () => null,
  showFallbackTimeout: DEFAULT_SHOW_FALLBACK_TIMEOUT
});

export { LoginDialog as default };
