import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { Component } from 'react';
import classNames from 'classnames';
import gift from '@jetbrains/icons/gift';
import Popup from '../popup/popup.js';
import { Directions } from '../popup/popup.consts.js';
import Icon from '../icon/icon.js';
import { Button } from '../button/button.js';
import { I18nContext } from '../i18n/i18n-context.js';
import { T as Theme, d as darkStyles, W as WithThemeClasses, a as ThemeProvider } from '../_helpers/theme.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'react-dom';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/data-tests.js';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.target.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../i18n/i18n.js';

var styles = {"container":"container_rui_f3db","message":"message_rui_f3db","icon":"icon_rui_f3db","title":"title_rui_f3db font-lower_rui_8bff font_rui_8bff","description":"description_rui_f3db","gotIt":"gotIt_rui_f3db","tail":"tail_rui_f3db"};

/**
 * @name Message
 */
const UNIT = 8;
const TAIL_SIZE = 11;
const getTailOffsets = offset => ({
  [Directions.BOTTOM_RIGHT]: {
    top: 0,
    left: offset + UNIT,
    transform: 'rotate(135deg)'
  },
  [Directions.BOTTOM_LEFT]: {
    top: 0,
    right: offset - UNIT - TAIL_SIZE,
    transform: 'rotate(135deg)'
  },
  [Directions.BOTTOM_CENTER]: {
    top: 0,
    left: offset + UNIT,
    transform: 'rotate(135deg)'
  },
  [Directions.TOP_RIGHT]: {
    bottom: -11,
    left: offset - UNIT,
    transform: 'rotate(-45deg)'
  },
  [Directions.TOP_LEFT]: {
    bottom: -11,
    right: offset + UNIT - TAIL_SIZE,
    transform: 'rotate(-45deg)'
  },
  [Directions.TOP_CENTER]: {
    bottom: -11,
    left: offset - UNIT,
    transform: 'rotate(-45deg)'
  },
  [Directions.RIGHT_TOP]: {
    bottom: offset + UNIT - TAIL_SIZE,
    left: 0,
    transform: 'rotate(45deg)'
  },
  [Directions.RIGHT_BOTTOM]: {
    top: offset - UNIT,
    left: 0,
    transform: 'rotate(45deg)'
  },
  [Directions.RIGHT_CENTER]: {
    top: offset - UNIT,
    left: 0,
    transform: 'rotate(45deg)'
  },
  [Directions.LEFT_TOP]: {
    bottom: offset - UNIT - TAIL_SIZE,
    right: -11,
    transform: 'rotate(-135deg)'
  },
  [Directions.LEFT_BOTTOM]: {
    top: offset + UNIT,
    right: -11,
    transform: 'rotate(-135deg)'
  },
  [Directions.LEFT_CENTER]: {
    top: offset + UNIT,
    right: -11,
    transform: 'rotate(-135deg)'
  }
});
/**
 * Displays a popup containing a message.
 */
class Message extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {});
    _defineProperty(this, "_onDirectionChange", direction => this.setState({
      direction
    }));
    _defineProperty(this, "popup", void 0);
    _defineProperty(this, "node", void 0);
    _defineProperty(this, "popupRef", el => {
      var _this$popup;
      this.popup = el;
      this.node = (_this$popup = this.popup) === null || _this$popup === void 0 ? void 0 : _this$popup.node;
    });
  }
  getTailOffset() {
    var _this$popup2;
    const DEFAULT_OFFSET = 32;
    const {
      popupProps
    } = this.props;
    if (this.props.tailOffset != null) {
      return this.props.tailOffset;
    }
    const anchor = (popupProps === null || popupProps === void 0 ? void 0 : popupProps.anchorElement) || ((_this$popup2 = this.popup) === null || _this$popup2 === void 0 ? void 0 : _this$popup2.parent);
    if (!anchor) {
      return DEFAULT_OFFSET;
    }
    const offset = Math.floor(anchor.offsetWidth / 2);
    const isOpenedToRight = this.state.direction != null && [Directions.TOP_RIGHT, Directions.BOTTOM_RIGHT].includes(this.state.direction);
    if (popupProps !== null && popupProps !== void 0 && popupProps.left && isOpenedToRight) {
      return offset - (popupProps === null || popupProps === void 0 ? void 0 : popupProps.left);
    }
    return offset;
  }
  render() {
    const {
      children: _children,
      className,
      tailClassName,
      title,
      icon,
      popupProps,
      buttonProps,
      onClose,
      onDismiss,
      translations,
      theme
    } = this.props;
    const classes = classNames(styles.message, className, {
      [darkStyles.dark]: theme === Theme.DARK
    });
    const tailClasses = classNames(styles.tail, tailClassName);
    const popupDirections = this.props.direction ? [this.props.direction] : this.props.directions;
    const {
      direction
    } = this.state;
    return jsx(I18nContext.Consumer, {
      children: _ref => {
        let {
          translate
        } = _ref;
        return jsx(WithThemeClasses, {
          theme: theme,
          children: themeClasses => {
            var _translations$gotIt, _translations$dismiss;
            return jsx(Popup, _objectSpread2(_objectSpread2({
              ref: this.popupRef,
              hidden: false,
              directions: popupDirections,
              className: classNames(classes, themeClasses),
              offset: UNIT * 2,
              onDirectionChange: this._onDirectionChange
            }, popupProps), {}, {
              children: jsxs(ThemeProvider, {
                theme: theme,
                passToPopups: true,
                children: [direction && jsx("div", {
                  className: tailClasses,
                  style: getTailOffsets(this.getTailOffset())[direction]
                }), icon && jsx(Icon, {
                  className: styles.icon,
                  glyph: icon
                }), title && jsx("h1", {
                  "data-test": "rgMessageTitle",
                  className: styles.title,
                  children: title
                }), _children && jsx("div", {
                  className: styles.description,
                  children: _children
                }), (onClose || buttonProps) && jsx(Button, _objectSpread2(_objectSpread2({
                  className: styles.gotIt,
                  onClick: onClose,
                  primary: true
                }, buttonProps), {}, {
                  children: (_translations$gotIt = translations === null || translations === void 0 ? void 0 : translations.gotIt) !== null && _translations$gotIt !== void 0 ? _translations$gotIt : translate('gotIt')
                })), onDismiss && jsx(Button, {
                  onClick: onDismiss,
                  inline: true,
                  children: (_translations$dismiss = translations === null || translations === void 0 ? void 0 : translations.dismiss) !== null && _translations$dismiss !== void 0 ? _translations$dismiss : translate('dismiss')
                })]
              })
            }));
          }
        });
      }
    });
  }
}
_defineProperty(Message, "defaultProps", {
  icon: gift,
  directions: [Directions.TOP_RIGHT, Directions.TOP_LEFT, Directions.TOP_CENTER, Directions.BOTTOM_RIGHT, Directions.BOTTOM_LEFT, Directions.BOTTOM_CENTER, Directions.RIGHT_TOP, Directions.RIGHT_BOTTOM, Directions.RIGHT_CENTER, Directions.LEFT_TOP, Directions.LEFT_BOTTOM, Directions.LEFT_CENTER],
  theme: Theme.DARK
});
_defineProperty(Message, "Directions", Directions);
_defineProperty(Message, "PopupProps", Popup.PopupProps);

export { Message as default };
