import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import { createPortal } from 'react-dom';
import classNames from 'classnames';
import getUID from '../global/get-uid.js';
import scheduleRAF from '../global/schedule-raf.js';
import { Listeners, getStyles } from '../global/dom.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import joinDataTestAttributes from '../global/data-tests.js';
import TabTrap from '../tab-trap/tab-trap.js';
import position from './position.js';
import { DEFAULT_DIRECTIONS, MaxHeight, MinWidth, Dimension, Directions, Display } from './popup.consts.js';
import { PopupTargetContext, PopupTarget } from './popup.target.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import 'core-js/modules/es.array.sort.js';

var styles = {"popup":"popup_rui_7cd4 font_rui_8bff","largeBorderRadius":"largeBorderRadius_rui_7cd4","hidden":"hidden_rui_7cd4","showing":"showing_rui_7cd4","attached":"attached_rui_7cd4"};

const isPossibleClientSideNavigation = event => {
  const target = event.target;
  const link = target.closest('a');
  // Taken from https://github.com/nanostores/router/blob/80a333db4cf0789fda21a02715ebabca15192642/index.js#L58-L69
  return link && event.button === 0 &&
  // Left mouse button
  link.target !== '_blank' &&
  // Not for new tab
  link.origin === location.origin &&
  // Not external link
  link.rel !== 'external' &&
  // Not external link
  link.target !== '_self' &&
  // Now manually disabled
  !link.download &&
  // Not download link
  !event.altKey &&
  // Not download link by user
  !event.metaKey &&
  // Not open in new tab by user
  !event.ctrlKey &&
  // Not open in new tab by user
  !event.shiftKey &&
  // Not open in new window by user
  !event.defaultPrevented;
};
const stop = event => {
  if (!isPossibleClientSideNavigation(event)) {
    event.stopPropagation();
  }
};
const getPopupContainer = target => typeof target === 'string' ? document.querySelector("[data-portaltarget=".concat(target, "]")) : target;
/**
 * @constructor
 * @name Popup
 * @extends {ReactComponent}
 */
class Popup extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      display: Display.SHOWING
    });
    _defineProperty(this, "popup", void 0);
    _defineProperty(this, "node", void 0);
    _defineProperty(this, "parent", void 0);
    _defineProperty(this, "container", void 0);
    _defineProperty(this, "ringPopupTarget", void 0);
    _defineProperty(this, "listeners", new Listeners());
    _defineProperty(this, "redrawScheduler", scheduleRAF(true));
    _defineProperty(this, "uid", getUID('popup-'));
    _defineProperty(this, "calculateDisplay", prevState => _objectSpread2(_objectSpread2({}, prevState), {}, {
      display: this.props.hidden ? Display.SHOWING : Display.SHOWN
    }));
    _defineProperty(this, "portalRef", el => {
      this.node = el;
      this.parent = el && el.parentElement;
      if (el && this.getContainer()) {
        this._redraw();
      }
    });
    _defineProperty(this, "popupRef", el => {
      this.popup = el;
      this._redraw();
    });
    _defineProperty(this, "containerRef", el => {
      this.container = el;
    });
    _defineProperty(this, "_updateDirection", newDirection => {
      if (this.state.direction !== newDirection) {
        this.setState({
          direction: newDirection
        });
        if (this.props.onDirectionChange) {
          this.props.onDirectionChange(newDirection);
        }
      }
    });
    _defineProperty(this, "_updatePosition", () => {
      const popup = this.popup;
      if (popup) {
        popup.style.position = 'absolute';
        if (this.isVisible()) {
          const {
            styles: style,
            direction
          } = this.position();
          Object.entries(style).forEach(_ref => {
            let [key, value] = _ref;
            const propKey = key;
            if (typeof value === 'number') {
              popup.style[propKey] = "".concat(value, "px");
            } else {
              popup.style[propKey] = value.toString();
            }
          });
          if (direction != null) {
            this._updateDirection(direction);
          }
        }
        this.setState(this.calculateDisplay);
      }
    });
    _defineProperty(this, "_redraw", () => {
      if (this.isVisible()) {
        this.redrawScheduler(this._updatePosition);
      }
    });
    _defineProperty(this, "_listenersEnabled", void 0);
    _defineProperty(this, "_prevTimeout", void 0);
    _defineProperty(this, "_onEscPress", evt => {
      this.props.onEscPress(evt);
      this._onCloseAttempt(evt, true);
    });
    /**
     * @param {jQuery.Event} evt
     * @private
     */
    _defineProperty(this, "_onDocumentClick", evt => {
      var _this$_getAnchor;
      if (this.container && evt.target instanceof Node && this.container.contains(evt.target) || !this._listenersEnabled || this.props.dontCloseOnAnchorClick && evt.target instanceof Node && (_this$_getAnchor = this._getAnchor()) !== null && _this$_getAnchor !== void 0 && _this$_getAnchor.contains(evt.target) ||
      // ignore clicks in shadow DOM, e.g., Grammarly suggestions
      evt.target instanceof Element && evt.target.shadowRoot != null) {
        return;
      }
      this.props.onOutsideClick(evt);
      this._onCloseAttempt(evt, false);
    });
    _defineProperty(this, "shortcutsScope", this.uid);
    _defineProperty(this, "shortcutsMap", {
      esc: this._onEscPress
    });
  }
  componentDidMount() {
    if (!this.props.hidden) {
      this._setListenersEnabled(true);
    }
  }
  componentDidUpdate(prevProps, prevState) {
    const {
      hidden
    } = this.props;
    if (this.props !== prevProps) {
      if (prevProps.hidden !== hidden) {
        this._setListenersEnabled(!hidden);
      }
      this._redraw();
    }
    if (this.props.onShow && !hidden && this.state.display === Display.SHOWN && (prevProps.hidden || prevState.display !== Display.SHOWN)) {
      this.props.onShow();
    }
  }
  componentWillUnmount() {
    this._setListenersEnabled(false);
  }
  shouldUseShortcuts() {
    const {
      shortcuts,
      hidden
    } = this.props;
    return shortcuts && !hidden;
  }
  getContainer() {
    const target = this.props.target || this.ringPopupTarget;
    return target && getPopupContainer(target);
  }
  position() {
    const {
      directions,
      autoPositioning,
      autoCorrectTopOverflow,
      sidePadding,
      top,
      left,
      offset,
      maxHeight,
      minWidth
    } = this.props;
    const container = this.getContainer();
    return position({
      popup: this.popup,
      container: container && getStyles(container).position !== 'static' ? container : null,
      anchor: this._getAnchor(),
      directions,
      autoPositioning,
      autoCorrectTopOverflow,
      sidePadding,
      top,
      left,
      offset,
      maxHeight,
      minWidth
    });
  }
  _getAnchor() {
    return this.props.anchorElement || this.parent;
  }
  /**
   * @param {boolean} enable
   * @private
   */
  _setListenersEnabled(enable) {
    if (enable && !this._listenersEnabled) {
      clearTimeout(this._prevTimeout);
      this._prevTimeout = window.setTimeout(() => {
        this._listenersEnabled = true;
        this.listeners.add(window, 'resize', this._redraw);
        if (this.props.autoPositioningOnScroll) {
          this.listeners.add(window, 'scroll', this._redraw);
        }
        this.listeners.add(document, 'pointerdown', this._onDocumentClick, true);
        let el = this._getAnchor();
        while (el) {
          this.listeners.add(el, 'scroll', this._redraw);
          el = el.parentElement;
        }
      }, 0);
      return;
    }
    if (!enable && this._listenersEnabled) {
      this.listeners.removeAll();
      clearTimeout(this._prevTimeout);
      this._listenersEnabled = false;
    }
  }
  /**
   * Returns visibility state
   * @return {boolean}
   */
  isVisible() {
    return !this.props.hidden;
  }
  _onCloseAttempt(evt, isEsc) {
    this.props.onCloseAttempt(evt, isEsc);
  }
  getInternalContent() {
    const {
      trapFocus,
      autoFocusFirst,
      children
    } = this.props;
    return trapFocus ? jsx(TabTrap, {
      autoFocusFirst: autoFocusFirst,
      focusBackOnExit: true,
      children: children
    }) : children;
  }
  render() {
    const {
      className,
      style,
      hidden,
      attached,
      keepMounted,
      client,
      onMouseDown,
      onMouseUp,
      onMouseOver,
      onMouseOut,
      onContextMenu,
      'data-test': dataTest,
      largeBorderRadius
    } = this.props;
    const showing = this.state.display === Display.SHOWING;
    const classes = classNames(className, styles.popup, {
      [styles.attached]: attached,
      [styles.hidden]: hidden,
      [styles.showing]: showing,
      [styles.largeBorderRadius]: largeBorderRadius
    });
    const direction = (this.state.direction || '').toLowerCase().replace(/[_]/g, '-');
    return jsx(PopupTargetContext.Consumer, {
      children: value => {
        this.ringPopupTarget = value;
        return jsxs("span", {
          // prevent bubbling through portal
          onClick: stop,
          // This handler only blocks bubbling through React portal
          role: "presentation",
          ref: this.portalRef,
          children: [this.shouldUseShortcuts() && jsx(Shortcuts, {
            map: this.shortcutsMap,
            scope: this.shortcutsScope
          }), client !== false && (keepMounted || !hidden) && /*#__PURE__*/createPortal(jsx(PopupTarget, {
            id: this.uid,
            ref: this.containerRef,
            onMouseOver: onMouseOver,
            onFocus: onMouseOver,
            onMouseOut: onMouseOut,
            onBlur: onMouseOut,
            onContextMenu: onContextMenu,
            children: jsx("div", {
              "data-test": joinDataTestAttributes('ring-popup', dataTest),
              "data-test-shown": !hidden && !showing,
              "data-test-direction": direction,
              ref: this.popupRef,
              className: classes,
              style: style,
              onMouseDown: onMouseDown,
              onMouseUp: onMouseUp,
              // mouse handlers are used to track clicking on inner elements
              role: "presentation",
              children: this.getInternalContent()
            })
          }), this.getContainer() || document.body)]
        });
      }
    });
  }
}
_defineProperty(Popup, "defaultProps", {
  shortcuts: true,
  hidden: false,
  onOutsideClick() {},
  onEscPress() {},
  onCloseAttempt() {},
  dontCloseOnAnchorClick: false,
  keepMounted: false,
  directions: DEFAULT_DIRECTIONS,
  autoPositioning: true,
  autoPositioningOnScroll: true,
  autoCorrectTopOverflow: true,
  left: 0,
  top: 0,
  offset: 0,
  sidePadding: 8,
  attached: false,
  trapFocus: false,
  autoFocusFirst: false,
  legacy: false
});
_defineProperty(Popup, "PopupProps", {
  Directions,
  Dimension,
  MinWidth,
  MaxHeight
});

export { PopupTarget, PopupTargetContext, Popup as default, getPopupContainer };
