import { _ as _defineProperty, b as _objectSpread2, a as _objectWithoutProperties, d as _toPropertyKey } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.trim.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs, Fragment as Fragment$1 } from 'react/jsx-runtime';
import { Component, Fragment } from 'react';
import classNames from 'classnames';
import chevronDownIcon from '@jetbrains/icons/chevron-down';
import closeIcon from '@jetbrains/icons/close-12px';
import { dequal } from 'dequal';
import '../dropdown/dropdown.js';
import Avatar from '../avatar/avatar.js';
import Popup from '../popup/popup.js';
import List, { ActiveItemContext } from '../list/list.js';
import { S as Size, I as Input, i as inputStyles } from '../_helpers/input.js';
import { ControlLabel } from '../control-label/control-label.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import { Button } from '../button/button.js';
import joinDataTestAttributes from '../global/data-tests.js';
import getUID from '../global/get-uid.js';
import rerenderHOC from '../global/rerender-hoc.js';
import fuzzyHighlight from '../global/fuzzy-highlight.js';
import memoize from '../global/memoize.js';
import { I18nContext } from '../i18n/i18n-context.js';
import { createComposedRef } from '../global/composeRefs.js';
import { isArray } from '../global/typescript-utils.js';
import { ControlsHeightContext, ControlsHeight } from '../global/controls-height.js';
import SelectPopup from './select__popup.js';
import { Size as Size$1 } from '../avatar/avatar-size.js';
import { A as Anchor } from '../_helpers/anchor.js';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../_helpers/button__classes.js';
import 'core-js/modules/es.string.includes.js';
import '../global/url.js';
import 'core-js/modules/es.string.match.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../_helpers/avatar-info.js';
import '../avatar/fallback-avatar.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'react-dom';
import '../global/schedule-raf.js';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../shortcuts/core.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import 'core-js/modules/es.symbol.description.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'memoize-one';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../control-help/control-help.js';
import '../link/link.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../i18n/i18n.js';
import '@jetbrains/icons/search';
import '../loader-inline/loader-inline.js';
import '../shortcuts/shortcuts-hoc.js';
import '../tags-list/tags-list.js';
import '../tag/tag.js';
import '../caret/caret.js';
import '../text/text.js';
import '../_helpers/select__filter.js';

var styles = {"select":"select_rui_11de","value":"value_rui_11de ellipsis_rui_8bff font_rui_8bff","icons":"icons_rui_11de","toolbar":"toolbar_rui_11de","button":"button_rui_11de","buttonSpaced":"buttonSpaced_rui_11de","inputMode":"inputMode_rui_11de","heightS":"heightS_rui_11de","heightL":"heightL_rui_11de","selectedIcon":"selectedIcon_rui_11de resetButton_rui_8bff","clearIcon":"clearIcon_rui_11de","sizeS":"sizeS_rui_11de","sizeM":"sizeM_rui_11de","sizeL":"sizeL_rui_11de","sizeFULL":"sizeFULL_rui_11de","sizeAUTO":"sizeAUTO_rui_11de","buttonMode":"buttonMode_rui_11de","open":"open_rui_11de","buttonContainer":"buttonContainer_rui_11de","buttonValue":"buttonValue_rui_11de ellipsis_rui_8bff","buttonValueOpen":"buttonValueOpen_rui_11de","buttonValueEmpty":"buttonValueEmpty_rui_11de","label":"label_rui_11de","disabled":"disabled_rui_11de","avatar":"avatar_rui_11de","popup":"popup_rui_11de","chevron":"chevron_rui_11de","chevronIcon":"chevronIcon_rui_11de"};

/**
 * @name Select
 */
function noop() {}
/**
 * @enum {number}
 */
var Type;
(function (Type) {
  Type["BUTTON"] = "BUTTON";
  Type["INPUT"] = "INPUT";
  Type["CUSTOM"] = "CUSTOM";
  Type["INLINE"] = "INLINE";
  Type["INPUT_WITHOUT_CONTROLS"] = "INPUT_WITHOUT_CONTROLS";
})(Type || (Type = {}));
const ICONS_OFFSET = 5;
const ICON_WIDTH = 20;
const getStyle = memoize(iconsLength => ({
  paddingRight: ICONS_OFFSET + iconsLength * ICON_WIDTH
}));
const isInputMode = type => type === Type.INPUT || type === Type.INPUT_WITHOUT_CONTROLS;
function getLowerCaseLabel(item) {
  if (List.isItemType(List.ListProps.Type.SEPARATOR, item) || List.isItemType(List.ListProps.Type.HINT, item) || typeof item.label !== 'string') {
    return null;
  }
  return item.label.toLowerCase();
}
function doesLabelMatch(itemToCheck, fn) {
  const lowerCaseLabel = getLowerCaseLabel(itemToCheck);
  if (lowerCaseLabel == null) {
    return true;
  }
  return fn(lowerCaseLabel);
}
function getFilterFn(filter) {
  if (typeof filter === 'object') {
    if (filter.fn) {
      return filter.fn;
    }
    if (filter.fuzzy) {
      return (itemToCheck, checkString) => doesLabelMatch(itemToCheck, lowerCaseLabel => fuzzyHighlight(checkString, lowerCaseLabel).matched);
    }
  }
  return (itemToCheck, checkString) => doesLabelMatch(itemToCheck, lowerCaseLabel => lowerCaseLabel.indexOf(checkString) >= 0);
}
function buildMultipleMap(selected) {
  return selected.reduce((acc, item) => {
    acc[item.key] = true;
    return acc;
  }, {});
}
function getListItems(props, state, rawFilterString) {
  let data = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : props.data;
  let filterString = rawFilterString.trim();
  if (isInputMode(props.type) && !props.allowAny && state.selected && !Array.isArray(state.selected) && filterString === state.selected.label) {
    filterString = ''; // ignore multiple if it is exactly the selected item
  }
  const lowerCaseString = filterString.toLowerCase();
  const filteredData = [];
  let exactMatch = false;
  const check = getFilterFn(props.filter);
  for (let i = 0; i < data.length; i++) {
    const item = _objectSpread2({}, data[i]);
    if (check(item, lowerCaseString, data)) {
      var _state$multipleMap2;
      exactMatch = item.label === filterString;
      if (props.multiple && !(typeof props.multiple === 'object' && props.multiple.removeSelectedItems)) {
        var _state$multipleMap;
        item.checkbox = !!((_state$multipleMap = state.multipleMap) !== null && _state$multipleMap !== void 0 && _state$multipleMap[item.key]);
      }
      if (props.multiple && typeof props.multiple === 'object' && props.multiple.limit && Array.isArray(state.selected)) {
        item.disabled = props.multiple.limit === state.selected.length && !state.selected.find(selectedItem => selectedItem.key === item.key);
      }
      // Ignore item if it's multiple and is already selected
      if (!(props.multiple && typeof props.multiple === 'object' && props.multiple.removeSelectedItems && (_state$multipleMap2 = state.multipleMap) !== null && _state$multipleMap2 !== void 0 && _state$multipleMap2[item.key])) {
        filteredData.push(item);
      }
    }
  }
  let addButton = null;
  const {
    add
  } = props;
  if (add && filterString && !exactMatch || add && add.alwaysVisible) {
    if (!(add.regexp && !add.regexp.test(filterString)) && !(add.minlength && filterString.length < +add.minlength) || add.alwaysVisible) {
      var _add$delayed;
      let label;
      if (add.label) {
        label = typeof add.label === 'function' ? add.label(filterString) : add.label;
      } else {
        label = filterString;
      }
      addButton = {
        prefix: add.prefix,
        label,
        delayed: (_add$delayed = add.delayed) !== null && _add$delayed !== void 0 ? _add$delayed : true
      };
    }
  }
  return {
    filteredData,
    addButton
  };
}
function getSelectedIndex(selected, data) {
  const firstSelected = Array.isArray(selected) ? selected[0] : selected;
  if (firstSelected == null) {
    return null;
  }
  for (let i = 0; i < data.length; i++) {
    const item = data[i];
    if (item.key === undefined) {
      continue;
    }
    if (item.key === firstSelected.key) {
      return i;
    }
  }
  return null;
}
const getItemLabel = _ref => {
  let {
    selectedLabel,
    label
  } = _ref;
  if (selectedLabel != null) {
    return selectedLabel;
  }
  return typeof label === 'string' ? label : '';
};
const getValueForFilter = (selected, type, filterValue) => selected && !isArray(selected) && isInputMode(type) ? getItemLabel(selected) : filterValue;
function isSameSelected(prevSelected, selected) {
  if (!prevSelected || !selected || prevSelected.length !== selected.length) {
    return false;
  }
  const keysMap = selected.reduce((result, item) => {
    result[item.key] = true;
    return result;
  }, {});
  return prevSelected.every(it => keysMap[it.key]);
}
/**
 * @name Select
 * @constructor
 * @extends {Component}
 */
/**
 * Displays a select.
 */
class Select extends Component {
  constructor() {
    var _this;
    super(...arguments);
    _this = this;
    _defineProperty(this, "state", {
      data: [],
      shownData: [],
      selected: this.props.multiple ? [] : null,
      selectedIndex: null,
      filterValue: this.props.filter && typeof this.props.filter === 'object' && this.props.filter.value || '',
      shortcutsEnabled: false,
      popupShortcuts: false,
      showPopup: this.props.showPopup,
      prevData: [],
      prevSelected: null,
      prevMultiple: this.props.multiple,
      multipleMap: {},
      addButton: null
    });
    _defineProperty(this, "id", getUID('select-'));
    _defineProperty(this, "shortcutsScope", this.id);
    _defineProperty(this, "listId", "".concat(this.id, ":list"));
    _defineProperty(this, "_focusHandler", e => {
      this.props.onFocus(e);
      this.setState({
        shortcutsEnabled: true,
        focused: true
      });
    });
    _defineProperty(this, "isClickingSelect", false);
    _defineProperty(this, "mouseDownHandler", () => {
      this.isClickingSelect = true;
    });
    _defineProperty(this, "mouseUpHandler", () => {
      this.isClickingSelect = false;
    });
    _defineProperty(this, "_blurHandler", () => {
      var _this$_popup;
      this.props.onBlur();
      if (this._popup && this._popup.isVisible() && !this._popup.isClickingPopup && !this.isClickingSelect) {
        window.setTimeout(() => {
          this.setState({
            showPopup: false
          });
        });
      }
      if (!((_this$_popup = this._popup) !== null && _this$_popup !== void 0 && _this$_popup.isClickingPopup)) {
        this.setState({
          shortcutsEnabled: false,
          focused: false
        });
      }
    });
    _defineProperty(this, "node", void 0);
    _defineProperty(this, "nodeRef", el => {
      this.node = el;
    });
    _defineProperty(this, "_popup", null);
    _defineProperty(this, "onEmptyPopupEnter", () => {
      if (this.state.addButton) {
        this.addHandler();
      }
    });
    _defineProperty(this, "focus", () => {
      var _this$node;
      const focusableSelectExists = (_this$node = this.node) === null || _this$node === void 0 ? void 0 : _this$node.querySelector('[data-test~=ring-select__focus]');
      const restoreFocusNode = this.props.targetElement || focusableSelectExists;
      restoreFocusNode === null || restoreFocusNode === void 0 || restoreFocusNode.focus();
    });
    _defineProperty(this, "_onEnter", () => {
      var _this$_popup2;
      if (this.state.addButton && this.state.shownData.length === 0) {
        this.addHandler();
      }
      this.props.onDone();
      if (!((_this$_popup2 = this._popup) !== null && _this$_popup2 !== void 0 && _this$_popup2.isVisible()) && this.props.allowAny) {
        return true;
      }
      return undefined;
    });
    _defineProperty(this, "_onEsc", event => {
      var _this$_popup3;
      if (!((_this$_popup3 = this._popup) !== null && _this$_popup3 !== void 0 && _this$_popup3.isVisible())) {
        return true;
      } else if (this.props.multiple || !this.props.getInitial) {
        return false;
      }
      const selected = {
        key: Math.random(),
        label: this.props.getInitial()
      };
      this.setState({
        selected,
        filterValue: this.getValueForFilter(selected)
      }, () => {
        this.props.onChange(selected, event);
        this.props.onReset();
      });
      return undefined;
    });
    _defineProperty(this, "_inputShortcutHandler", () => {
      if (this.state.focused && this._popup && !this._popup.isVisible()) {
        this._clickHandler();
      }
    });
    _defineProperty(this, "popupRef", el => {
      this._popup = el;
    });
    _defineProperty(this, "_showPopup", () => {
      if (!this.node) {
        return;
      }
      const shownData = this.getListItems(this.filterValue());
      this.setState({
        showPopup: true,
        shownData
      });
    });
    _defineProperty(this, "_hidePopup", tryFocusAnchor => {
      if (this.node && this.state.showPopup) {
        this.setState(prevState => ({
          showPopup: false,
          filterValue: this.props.allowAny ? prevState.filterValue : ''
        }));
        if (tryFocusAnchor) {
          this.focus();
        }
      }
    });
    _defineProperty(this, "addHandler", () => {
      const value = this.filterValue();
      this._hidePopup();
      this.props.onAdd(value);
    });
    _defineProperty(this, "getLowerCaseLabel", getLowerCaseLabel);
    _defineProperty(this, "doesLabelMatch", doesLabelMatch);
    _defineProperty(this, "_clickHandler", () => {
      if (!this.props.disabled) {
        if (this.state.showPopup) {
          this._hidePopup();
        } else {
          this.props.onBeforeOpen();
          this._showPopup();
        }
      }
    });
    _defineProperty(this, "_openPopupIfClosed", () => {
      if (this.props.disabled || this.state.showPopup) {
        return;
      }
      this.props.onBeforeOpen();
      this._showPopup();
    });
    _defineProperty(this, "_filterChangeHandler", e => {
      this._setFilter(e.currentTarget.value, e);
    });
    _defineProperty(this, "_setFilter", (value, event) => {
      var _this$_popup4;
      if (this.isInputMode() && !this.state.focused) {
        return;
      }
      if (value === this.state.filterValue) {
        return;
      }
      const filterValue = value.replace(/^\s+/g, '');
      this.props.onFilter(filterValue);
      if (this.props.allowAny) {
        const fakeSelected = {
          key: Math.random(),
          label: filterValue
        };
        this.setState({
          selected: filterValue === '' ? null : fakeSelected,
          selectedIndex: null
        }, () => {
          this.props.onSelect(fakeSelected, event);
          this.props.onChange(fakeSelected, event);
        });
      }
      if (!((_this$_popup4 = this._popup) !== null && _this$_popup4 !== void 0 && _this$_popup4.isVisible())) {
        this.props.onBeforeOpen();
      }
      this.setState({
        filterValue
      }, () => {
        this._showPopup();
      });
    });
    _defineProperty(this, "_redrawPopup", () => {
      if (this.props.multiple) {
        setTimeout(() => {
          //setTimeout solves events order and bubbling issue
          if (this.isInputMode()) {
            this.clearFilter();
          }
          this._showPopup();
        }, 0);
      }
    });
    _defineProperty(this, "_listSelectHandler", function (selected, event) {
      var _this$props$tryKeepOp;
      let opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      const isItem = item => List.isItemType(List.ListProps.Type.ITEM, item);
      const isCustomItem = item => List.isItemType(List.ListProps.Type.CUSTOM, item);
      const isSelectItemEvent = event && (event.type === 'select' || event.type === 'keydown');
      if (isSelectItemEvent) {
        event.preventDefault();
      }
      if (!isItem(selected) && !isCustomItem(selected) || selected.disabled || selected.isResetItem) {
        return;
      }
      const tryKeepOpen = (_this$props$tryKeepOp = _this.props.tryKeepOpen) !== null && _this$props$tryKeepOp !== void 0 ? _this$props$tryKeepOp : opts.tryKeepOpen;
      if (!_this.props.multiple) {
        if (!tryKeepOpen) {
          _this._hidePopup(isSelectItemEvent);
        }
        _this.setState({
          selected,
          selectedIndex: _this._getSelectedIndex(selected, _this.props.data)
        }, () => {
          const newFilterValue = _this.isInputMode() && !_this.props.hideSelected ? getItemLabel(selected) : '';
          _this.filterValue(newFilterValue);
          _this.props.onFilter(newFilterValue);
          _this.props.onSelect(selected, event);
          _this.props.onChange(selected, event);
        });
      } else {
        if (!tryKeepOpen) {
          _this._hidePopup(isSelectItemEvent);
        }
        if (selected.key == null) {
          throw new Error('Multiple selection requires each item to have the "key" property');
        }
        _this.setState(prevState => {
          const currentSelection = prevState.selected;
          let nextSelection;
          if (!prevState.multipleMap[selected.key]) {
            nextSelection = currentSelection.concat(selected);
          } else {
            nextSelection = currentSelection.filter(item => item.key !== selected.key);
          }
          const nextState = {
            filterValue: '',
            selected: nextSelection,
            selectedIndex: _this._getSelectedIndex(selected, _this.props.data)
          };
          if (typeof _this.props.multiple === 'object' && _this.props.multiple.limit && nextSelection.length === _this.props.multiple.limit) {
            nextState.shownData = prevState.shownData.map(item => nextSelection.find(selectedItem => selectedItem.key === item.key) ? item : _objectSpread2(_objectSpread2({}, item), {}, {
              disabled: true
            }));
          }
          if (!prevState.multipleMap[selected.key]) {
            nextState.multipleMap = _objectSpread2(_objectSpread2({}, prevState.multipleMap), {}, {
              [selected.key]: true
            });
          } else {
            const _prevState$multipleMa = prevState.multipleMap,
              _selected$key = selected.key,
              {
                [_selected$key]: _
              } = _prevState$multipleMa,
              restMultipleMap = _objectWithoutProperties(_prevState$multipleMa, [_selected$key].map(_toPropertyKey));
            nextState.multipleMap = restMultipleMap;
          }
          return _objectSpread2(_objectSpread2({}, prevState), nextState);
        }, () => {
          if (_this.state.multipleMap[selected.key]) {
            var _this$props$onSelect, _this$props;
            (_this$props$onSelect = (_this$props = _this.props).onSelect) === null || _this$props$onSelect === void 0 || _this$props$onSelect.call(_this$props, selected, event);
          } else {
            var _this$props$onDeselec, _this$props2;
            (_this$props$onDeselec = (_this$props2 = _this.props).onDeselect) === null || _this$props$onDeselec === void 0 || _this$props$onDeselec.call(_this$props2, selected);
          }
          _this.props.onChange(_this.state.selected, event);
          if (tryKeepOpen) {
            _this._redrawPopup();
          }
        });
      }
    });
    _defineProperty(this, "_listSelectAllHandler", function () {
      let isSelectAll = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
      const isItem = item => List.isItemType(List.ListProps.Type.ITEM, item);
      const isCustomItem = item => List.isItemType(List.ListProps.Type.CUSTOM, item);
      _this.setState(prevState => {
        const currentSelection = prevState.selected;
        let nextSelection;
        if (isSelectAll) {
          nextSelection = _this.props.data.filter(item => (isItem(item) || isCustomItem(item)) && !item.disabled);
          nextSelection.filter(item => !_this.props.selected.find(selectedItem => item.key === selectedItem.key)).forEach(item => {
            var _this$props$onSelect2, _this$props3;
            (_this$props$onSelect2 = (_this$props3 = _this.props).onSelect) === null || _this$props$onSelect2 === void 0 || _this$props$onSelect2.call(_this$props3, item);
          });
        } else {
          nextSelection = [];
          currentSelection.forEach(item => {
            var _this$props$onDeselec2, _this$props4;
            (_this$props$onDeselec2 = (_this$props4 = _this.props).onDeselect) === null || _this$props$onDeselec2 === void 0 || _this$props$onDeselec2.call(_this$props4, item);
          });
        }
        _this.props.onChange(nextSelection, event);
        return {
          filterValue: '',
          selected: nextSelection,
          selectedIndex: isSelectAll ? _this._getSelectedIndex(nextSelection, _this.props.data) : null,
          shownData: prevState.shownData.map(item => _objectSpread2(_objectSpread2({}, item), {}, {
            checkbox: isSelectAll
          })),
          multipleMap: isSelectAll ? buildMultipleMap(_this.props.data.filter(item => !item.disabled)) : {}
        };
      }, _this._redrawPopup);
    });
    _defineProperty(this, "_onCloseAttempt", (event, isEsc) => {
      if (this.isInputMode()) {
        if (!this.props.allowAny) {
          if (this.props.hideSelected || !this.state.selected || this.props.multiple) {
            this.clearFilter();
          } else if (this.state.selected && !Array.isArray(this.state.selected)) {
            this.filterValue(getItemLabel(this.state.selected));
          }
        }
      }
      const isTagRemoved = this.props.tags && event && event.target && event.target instanceof Element && event.target.matches('[data-test="ring-tag-remove"]');
      if (!isTagRemoved) {
        this._hidePopup(isEsc);
      }
    });
    _defineProperty(this, "clearFilter", e => {
      this._setFilter('', e);
    });
    _defineProperty(this, "clear", event => {
      if (event) {
        event.stopPropagation();
      }
      const empty = this.props.multiple ? [] : null;
      this.setState({
        selected: empty,
        selectedIndex: null,
        filterValue: ''
      }, () => {
        if (this.props.onChange) {
          this.props.onChange(empty, event);
        }
      });
      return false;
    });
    _defineProperty(this, "filter", void 0);
    _defineProperty(this, "filterRef", el => {
      this.filter = el;
    });
    _defineProperty(this, "composedFilterRef", createComposedRef());
  }
  static getDerivedStateFromProps(nextProps, prevState) {
    const {
      multiple,
      data,
      type
    } = nextProps;
    const {
      prevSelected,
      prevData,
      prevMultiple,
      filterValue
    } = prevState;
    const nextState = {
      prevData: data,
      prevSelected: nextProps.selected,
      prevMultiple: multiple
    };
    if ('data' in nextProps && data !== prevData) {
      const {
        filteredData,
        addButton
      } = getListItems(nextProps, prevState, filterValue, data);
      Object.assign(nextState, {
        shownData: filteredData,
        addButton
      });
      if (prevState.selected) {
        Object.assign(nextState, {
          selectedIndex: getSelectedIndex(prevState.selected, data),
          filterValue: getValueForFilter(prevState.selected, type, filterValue)
        });
      }
    }
    if ('selected' in nextProps && nextProps.selected !== prevSelected) {
      const selected = nextProps.selected || (multiple ? [] : null);
      const selectedIndex = getSelectedIndex(selected, data || prevData);
      Object.assign(nextState, {
        selected,
        filterValue: getValueForFilter(selected, type, filterValue)
      });
      if (!Array.isArray(prevSelected) || !Array.isArray(selected) || !isSameSelected(prevSelected, selected)) {
        Object.assign(nextState, {
          selectedIndex
        });
      }
    }
    if (prevMultiple !== multiple && !dequal(prevMultiple, multiple)) {
      nextState.selected = multiple ? [] : null;
    }
    if (multiple && !nextState.selected) {
      nextState.selected = prevState.selected;
    }
    const {
      selected
    } = _objectSpread2(_objectSpread2({}, prevState), nextState);
    if (selected && Array.isArray(selected)) {
      nextState.multipleMap = buildMultipleMap(selected);
      const {
        filteredData,
        addButton
      } = getListItems(nextProps, nextState, filterValue, data);
      Object.assign(nextState, {
        shownData: filteredData,
        addButton
      });
    }
    return nextState;
  }
  componentDidUpdate(prevProps, prevState) {
    const {
      showPopup,
      selected
    } = this.state;
    const {
      onClose,
      onOpen,
      onChange,
      multiple
    } = this.props;
    if (prevState.showPopup && !showPopup) {
      onClose(selected);
    } else if (!prevState.showPopup && showPopup) {
      onOpen();
    }
    if (multiple !== prevProps.multiple && !dequal(multiple, prevProps.multiple)) {
      onChange(selected);
    }
  }
  getValueForFilter(selected) {
    return getValueForFilter(selected, this.props.type, this.state.filterValue);
  }
  _getSelectedIndex(selected, data) {
    return getSelectedIndex(selected, data);
  }
  _getResetOption() {
    const isOptionsSelected = Array.isArray(this.state.selected) && this.state.selected.length;
    const reset = this.props.tags && typeof this.props.tags === 'object' ? this.props.tags.reset : null;
    if (!isOptionsSelected || !reset) {
      return null;
    }
    const resetHandler = (item, event) => {
      this.clear(event);
      this.clearFilter();
      this.props.onFilter('');
      this.setState(prevState => ({
        shownData: prevState.shownData.slice(reset.separator ? 2 : 1),
        multipleMap: {}
      }));
      this._redrawPopup();
    };
    return {
      isResetItem: true,
      separator: reset.separator,
      key: reset.label,
      rgItemType: List.ListProps.Type.CUSTOM,
      template: jsx(Button, {
        inline: true,
        className: styles.button,
        "data-test": "ring-select-reset-tags-button",
        height: ControlsHeight.S,
        children: reset.label
      }),
      glyph: reset.glyph,
      onClick: resetHandler
    };
  }
  _prependResetOption(shownData) {
    const resetOption = this._getResetOption();
    if (resetOption) {
      const resetItems = [resetOption];
      if (resetOption.separator) {
        resetItems.push({
          rgItemType: List.ListProps.Type.SEPARATOR
        });
      }
      return resetItems.concat(shownData);
    }
    return shownData;
  }
  _renderPopup() {
    const anchorElement = this.props.targetElement || this.node;
    const {
      showPopup,
      shownData
    } = this.state;
    const _shownData = this._prependResetOption(shownData);
    return jsx(I18nContext.Consumer, {
      children: _ref2 => {
        let {
          translate
        } = _ref2;
        let message;
        if (this.props.loading) {
          var _this$props$loadingMe;
          message = (_this$props$loadingMe = this.props.loadingMessage) !== null && _this$props$loadingMe !== void 0 ? _this$props$loadingMe : translate('loading');
        } else if (!shownData.length) {
          var _this$props$notFoundM;
          message = (_this$props$notFoundM = this.props.notFoundMessage) !== null && _this$props$notFoundM !== void 0 ? _this$props$notFoundM : translate('noOptionsFound');
        }
        return jsx(SelectPopup, {
          data: _shownData,
          message: message,
          toolbar: showPopup && this.getToolbar(),
          topbar: this.getTopbar(),
          loading: this.props.loading,
          activeIndex: this.state.selectedIndex,
          hidden: !showPopup,
          ref: this.popupRef,
          maxHeight: this.props.maxHeight,
          minWidth: this.props.minWidth,
          directions: this.props.directions,
          className: this.props.popupClassName,
          style: this.props.popupStyle,
          top: this.props.top,
          left: this.props.left,
          offset: this.props.offset,
          filter: this.isInputMode() ? false : this.props.filter,
          filterIcon: this.props.filterIcon,
          filterRef: this.props.filterRef,
          multiple: this.props.multiple,
          filterValue: this.state.filterValue,
          anchorElement: anchorElement,
          onCloseAttempt: this._onCloseAttempt,
          onOutsideClick: this.props.onOutsideClick,
          onSelect: this._listSelectHandler,
          onSelectAll: this._listSelectAllHandler,
          onFilter: this._filterChangeHandler,
          onClear: this.clearFilter,
          onLoadMore: this.props.onLoadMore,
          isInputMode: this.isInputMode(),
          selected: this.state.selected,
          tags: this.props.tags,
          compact: this.props.compact,
          renderOptimization: this.props.renderOptimization,
          ringPopupTarget: this.props.ringPopupTarget,
          disableMoveOverflow: this.props.disableMoveOverflow,
          disableScrollToActive: this.props.disableScrollToActive,
          dir: this.props.dir,
          onEmptyPopupEnter: this.onEmptyPopupEnter,
          listId: this.listId,
          preventListOverscroll: this.props.preventListOverscroll
        });
      }
    });
  }
  getToolbar() {
    const {
      hint,
      renderBottomToolbar
    } = this.props;
    const {
      prefix,
      label,
      delayed
    } = this.state.addButton || {};
    const isToolbarHasElements = this.state.addButton || hint || renderBottomToolbar;
    if (!isToolbarHasElements) {
      return null;
    }
    return jsxs("div", {
      className: classNames({
        [styles.toolbar]: Boolean(this.state.addButton || renderBottomToolbar)
      }),
      "data-test": "ring-select-toolbar",
      children: [renderBottomToolbar && renderBottomToolbar(), this.state.addButton && jsx(Button, {
        inline: true,
        delayed: delayed,
        className: classNames(styles.button, styles.buttonSpaced),
        onClick: this.addHandler,
        "data-test": "ring-select-toolbar-button",
        children: prefix ? "".concat(prefix, " ").concat(label) : label
      }), hint && jsx(List.ListHint, {
        label: hint,
        "data-test": "ring-select-toolbar-hint"
      })]
    });
  }
  getTopbar() {
    var _this$props$renderTop, _this$props5;
    return (_this$props$renderTop = (_this$props5 = this.props).renderTopToolbar) === null || _this$props$renderTop === void 0 ? void 0 : _this$props$renderTop.call(_this$props5);
  }
  getFilterFn() {
    return getFilterFn(this.props.filter);
  }
  getListItems(rawFilterString, data) {
    const {
      filteredData,
      addButton
    } = getListItems(this.props, this.state, rawFilterString, data);
    this.setState({
      addButton
    });
    return filteredData;
  }
  filterValue(setValue) {
    if (typeof setValue === 'string' || typeof setValue === 'number') {
      this.setState({
        filterValue: setValue
      });
      return undefined;
    } else {
      return this.state.filterValue;
    }
  }
  isInputMode() {
    return isInputMode(this.props.type);
  }
  _rebuildMultipleMap(selected) {
    if (Array.isArray(selected)) {
      this.setState({
        multipleMap: buildMultipleMap(selected)
      });
    }
  }
  _selectionIsEmpty() {
    return Array.isArray(this.state.selected) && !this.state.selected.length || !this.state.selected;
  }
  _getLabel() {
    var _ref3, _this$props$label;
    return (_ref3 = (_this$props$label = this.props.label) !== null && _this$props$label !== void 0 ? _this$props$label : this.props.selectedLabel) !== null && _ref3 !== void 0 ? _ref3 : 'Select an option';
  }
  _getPlaceholder() {
    if (this._selectionIsEmpty()) {
      var _this$props$label2;
      return (_this$props$label2 = this.props.label) !== null && _this$props$label2 !== void 0 ? _this$props$label2 : 'Select an option';
    }
    return this._getSelectedString();
  }
  _getSelectedString() {
    if (Array.isArray(this.state.selected)) {
      const labels = [];
      for (let i = 0; i < this.state.selected.length; i++) {
        labels.push(getItemLabel(this.state.selected[i]));
      }
      return labels.filter(Boolean).join(', ');
    } else {
      return this.state.selected != null ? getItemLabel(this.state.selected) : null;
    }
  }
  _getIcons() {
    const {
      selected
    } = this.state;
    const {
      disabled,
      clear,
      hideArrow
    } = this.props;
    const icons = [];
    const height = this.props.height || this.context;
    if (!Array.isArray(selected) && selected !== null && selected !== void 0 && selected.icon) {
      icons.push(jsx("button", {
        title: "Toggle options popup",
        type: "button",
        className: styles.selectedIcon,
        disabled: this.props.disabled,
        onClick: this._clickHandler,
        style: {
          backgroundImage: "url(".concat(selected.icon, ")")
        }
      }, "selected"));
    }
    if (clear && !disabled && !this._selectionIsEmpty()) {
      icons.push(jsx(Button, {
        title: "Clear selection",
        "data-test": "ring-clear-select",
        className: styles.clearIcon,
        disabled: this.props.disabled,
        onClick: this.clear,
        height: height,
        icon: closeIcon
      }, "close"));
    }
    if (!hideArrow) {
      icons.push(jsx(Button, {
        title: "Toggle options popup",
        className: styles.chevron,
        iconClassName: styles.chevronIcon,
        icon: chevronDownIcon,
        disabled: this.props.disabled,
        height: height,
        onClick: this._clickHandler
      }, "hide"));
    }
    return icons;
  }
  _getAvatar() {
    var _this$state$selected, _this$state$selected2;
    return !Array.isArray(this.state.selected) && (((_this$state$selected = this.state.selected) === null || _this$state$selected === void 0 ? void 0 : _this$state$selected.avatar) || ((_this$state$selected2 = this.state.selected) === null || _this$state$selected2 === void 0 ? void 0 : _this$state$selected2.showGeneratedAvatar)) && jsx(Avatar, {
      className: styles.avatar,
      url: this.state.selected.avatar,
      username: this.state.selected.username,
      size: Size$1.Size20
    });
  }
  getShortcutsMap() {
    return {
      enter: this._onEnter,
      esc: this._onEsc,
      up: this._inputShortcutHandler,
      down: this._inputShortcutHandler,
      right: noop,
      left: noop,
      'shift+up': noop,
      'shift+down': noop,
      space: noop
    };
  }
  renderSelect(activeItemId) {
    var _ref4, _this$props$label3, _this$_popup5, _this$props$buttonCla;
    const dataTest = this.props['data-test'];
    const {
      selectedLabel
    } = this.props;
    const {
      shortcutsEnabled
    } = this.state;
    const classes = classNames(styles.select, 'ring-js-shortcuts', this.props.className, styles["height".concat(this.props.height || this.context)], {
      [styles["size".concat(this.props.size)]]: this.props.type !== Type.INLINE,
      [styles.disabled]: this.props.disabled
    });
    let style;
    let iconsNode;
    if (this.props.type === Type.INPUT || this.props.type === Type.BUTTON) {
      const icons = this._getIcons();
      style = getStyle(icons.length);
      iconsNode = jsx("div", {
        className: styles.icons,
        children: icons
      });
    }
    const ariaProps = {
      role: 'combobox',
      'aria-expanded': this.state.showPopup,
      'aria-controls': this.listId,
      'aria-activedescendant': this.state.showPopup ? activeItemId : undefined,
      'aria-label': (_ref4 = (_this$props$label3 = this.props.label) !== null && _this$props$label3 !== void 0 ? _this$props$label3 : this._getPlaceholder()) !== null && _ref4 !== void 0 ? _ref4 : undefined
    };
    switch (this.props.type) {
      case Type.INPUT_WITHOUT_CONTROLS:
      case Type.INPUT:
        return jsxs(Fragment$1, {
          children: [jsxs("div", {
            ref: this.nodeRef,
            className: classNames(classes, styles.inputMode),
            "data-test": joinDataTestAttributes('ring-select', dataTest),
            role: "presentation" // has interactive elements inside
            ,
            onMouseDown: this.mouseDownHandler,
            onMouseUp: this.mouseUpHandler,
            children: [shortcutsEnabled && jsx(Shortcuts, {
              map: this.getShortcutsMap(),
              scope: this.shortcutsScope
            }), jsx(Input, _objectSpread2(_objectSpread2({}, ariaProps), {}, {
              height: this.props.height,
              autoComplete: "off",
              id: this.props.id,
              onClick: this._clickHandler,
              inputRef: this.composedFilterRef(this.filterRef, this.props.filterRef),
              disabled: this.props.disabled,
              value: this.state.filterValue,
              borderless: this.props.type === Type.INPUT_WITHOUT_CONTROLS,
              style: style,
              size: Size.FULL,
              onChange: this._filterChangeHandler,
              onFocus: this._focusHandler,
              onBlur: this._blurHandler,
              // Input with error style without description
              error: this.props.error != null ? '' : null,
              label: this.props.type === Type.INPUT ? this._getLabel() : null,
              placeholder: this.props.inputPlaceholder,
              onKeyDown: this.props.onKeyDown,
              "data-test": "ring-select__focus",
              enableShortcuts: shortcutsEnabled ? Object.keys(_objectSpread2(_objectSpread2({}, this.getShortcutsMap()), (_this$_popup5 = this._popup) === null || _this$_popup5 === void 0 || (_this$_popup5 = _this$_popup5.list) === null || _this$_popup5 === void 0 ? void 0 : _this$_popup5.shortcutsMap)) : undefined,
              icon: this.props.filterIcon,
              afterInput: iconsNode
            })), this._renderPopup()]
          }), this.props.error && jsx("div", {
            className: classNames(inputStyles.errorText, inputStyles["size".concat(this.props.size)]),
            children: this.props.error
          })]
        });
      case Type.BUTTON:
        return jsxs("div", {
          ref: this.nodeRef,
          className: classNames(classes, styles.buttonMode),
          "data-test": joinDataTestAttributes('ring-select', dataTest),
          children: [selectedLabel && jsx(ControlLabel, {
            type: this.props.labelType,
            disabled: this.props.disabled,
            htmlFor: this.props.id,
            children: selectedLabel
          }), shortcutsEnabled && jsx(Shortcuts, {
            map: this.getShortcutsMap(),
            scope: this.shortcutsScope
          }), jsxs("div", {
            className: styles.buttonContainer,
            children: [jsxs(Button, _objectSpread2(_objectSpread2({}, ariaProps), {}, {
              height: this.props.height,
              id: this.props.id,
              onClick: this._clickHandler,
              className: classNames(this.props.buttonClassName, styles.buttonValue, {
                [styles.buttonValueOpen]: this.state.showPopup,
                [styles.buttonValueEmpty]: this._selectionIsEmpty()
              }),
              disabled: this.props.disabled,
              style: style,
              "data-test": "ring-select__button ring-select__focus",
              children: [this._getAvatar(), this._getPlaceholder()]
            })), iconsNode]
          }), this._renderPopup()]
        });
      case Type.INLINE:
        return jsxs("div", {
          className: classes,
          ref: this.nodeRef,
          "data-test": joinDataTestAttributes('ring-select', dataTest),
          children: [shortcutsEnabled && jsx(Shortcuts, {
            map: this.getShortcutsMap(),
            scope: this.shortcutsScope
          }), jsx(Anchor, _objectSpread2(_objectSpread2({}, ariaProps), {}, {
            className: (_this$props$buttonCla = this.props.buttonClassName) !== null && _this$props$buttonCla !== void 0 ? _this$props$buttonCla : undefined,
            id: this.props.id,
            onClick: this._clickHandler,
            "data-test": "ring-select__focus",
            disabled: this.props.disabled,
            active: this.state.showPopup,
            children: this._getPlaceholder()
          })), this._renderPopup()]
        });
      default:
        if (this.props.customAnchor) {
          return jsxs(Fragment, {
            children: [shortcutsEnabled && jsx(Shortcuts, {
              map: this.getShortcutsMap(),
              scope: this.shortcutsScope
            }), this.props.customAnchor({
              wrapperProps: {
                ref: this.nodeRef,
                'data-test': joinDataTestAttributes('ring-select', dataTest)
              },
              buttonProps: _objectSpread2(_objectSpread2({}, ariaProps), {}, {
                id: this.props.id,
                onClick: this._clickHandler,
                disabled: this.props.disabled,
                children: this._getPlaceholder(),
                'data-test': 'ring-select__focus'
              }),
              popup: this._renderPopup()
            })]
          });
        }
        return jsx("span", {
          id: this.props.id,
          ref: this.nodeRef,
          "data-test": "ring-select",
          children: this._renderPopup()
        });
    }
  }
  render() {
    return jsx(ActiveItemContext.Provider, {
      children: jsx(ActiveItemContext.ValueContext.Consumer, {
        children: activeItemId => this.renderSelect(activeItemId)
      })
    });
  }
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
_defineProperty(Select, "defaultProps", {
  data: [],
  filter: false,
  // enable filter (not in INPUT modes)
  filterIcon: null,
  filterRef: noop,
  multiple: false,
  // multiple can be an object - see demo for more information
  clear: false,
  // enable clear button that clears the "selected" state
  loading: false,
  // show a loading indicator while data is loading
  disabled: false,
  // disable select
  type: Type.BUTTON,
  size: Size.M,
  targetElement: null,
  // element to bind the popup to (select BUTTON or INPUT by default)
  hideSelected: false,
  // INPUT mode: clears the input after an option is selected (useful when the selection is displayed in some custom way elsewhere)
  allowAny: false,
  // INPUT mode: allows any value to be entered
  hideArrow: false,
  // hide dropdown arrow icon
  showPopup: false,
  maxHeight: 600,
  // height of the options list, including the filter and the 'Add' button
  directions: [Popup.PopupProps.Directions.BOTTOM_RIGHT, Popup.PopupProps.Directions.BOTTOM_LEFT, Popup.PopupProps.Directions.TOP_LEFT, Popup.PopupProps.Directions.TOP_RIGHT],
  selected: null,
  // current selection (item / array of items)
  label: null,
  // BUTTON or INPUT label (nothing selected)
  selectedLabel: null,
  // BUTTON or INPUT label (something selected)
  inputPlaceholder: '',
  // Placeholder for input modes
  hint: null,
  // hint text to display under the list
  shortcutsEnabled: false,
  onBeforeOpen: noop,
  onLoadMore: noop,
  onOpen: noop,
  onClose: noop,
  onFilter: noop,
  // search string as first argument
  onFocus: noop,
  onBlur: noop,
  onKeyDown: noop,
  onSelect: noop,
  // single + multi
  onDeselect: noop,
  // multi
  onOutsideClick: noop,
  // multi
  onChange: noop,
  // multi
  onAdd: noop,
  // search string as first argument
  onDone: noop,
  onReset: noop,
  tags: null,
  ringPopupTarget: null,
  dir: 'ltr',
  offset: 4
});
_defineProperty(Select, "contextType", ControlsHeightContext);
_defineProperty(Select, "Type", Type);
_defineProperty(Select, "Size", Size);
const RerenderableSelect = rerenderHOC(Select);

export { RerenderableSelect, Type, Select as default };
