import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.split.js';
import sniffr from '../global/sniffer.js';
import 'sniffr';

const macSymbolsMap = {
  enter: '⏎',
  shift: '⇧',
  meta: '⌘',
  mod: '⌘',
  alt: '⌥',
  ctrl: '⌃',
  backspace: '⌫',
  esc: '⎋',
  tab: '⇥',
  del: '⌦',
  home: '⇱',
  end: '⇲',
  space: '␣',
  ins: 'Insert',
  left: '←',
  up: '↑',
  right: '→',
  down: '↓'
};
const winSymbolsMap = {
  enter: 'Enter',
  shift: 'Shift',
  meta: 'Ctrl',
  mod: 'Ctrl',
  alt: 'Alt',
  ctrl: 'Ctrl',
  backspace: 'Backspace',
  esc: 'Esc',
  tab: 'Tab',
  del: 'Delete',
  home: 'Home',
  end: 'End',
  space: 'Space',
  ins: 'Insert',
  left: '←',
  up: '↑',
  right: '→',
  down: '↓'
};
function getShortcutTitle(shortcut) {
  const MAC_OS = sniffr.os.name === 'macos';
  const KEY_SEPARATOR = MAC_OS ? '' : '+';
  const symbolsMap = MAC_OS ? macSymbolsMap : winSymbolsMap;
  return shortcut.split(/\+/g).map(symbol => symbolsMap[symbol] || symbol.toUpperCase()).join(KEY_SEPARATOR);
}

export { getShortcutTitle };
