import { b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.array.reverse.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { useRef, useState, useMemo, useCallback, useEffect, Fragment } from 'react';
import classNames from 'classnames';
import { isArray } from '../global/typescript-utils.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import getUID from '../global/get-uid.js';
import { HUNDRED, toRange, validateValue, calculateMarks, toPercent, calculateValue, adjustValues } from './slider.utils.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.number.to-fixed.js';

var styles = {"slider":"slider_rui_d7e0","disabled":"disabled_rui_d7e0","marked":"marked_rui_d7e0","rail":"rail_rui_d7e0","rounded":"rounded_rui_d7e0","track":"track_rui_d7e0","thumb":"thumb_rui_d7e0 resetButton_rui_8bff","dragged":"dragged_rui_d7e0","tick":"tick_rui_d7e0","active":"active_rui_d7e0","markValue":"markValue_rui_d7e0","tag":"tag_rui_d7e0"};

const Slider = _ref => {
  let {
    defaultValue,
    value,
    min = 0,
    max = HUNDRED,
    step = 1,
    disabled,
    marks,
    showTicks,
    showTag,
    className,
    renderTag,
    onChange
  } = _ref;
  const ref = useRef(null);
  const previouslyDragged = useRef(false);
  const [values, setValues] = useState(defaultValue !== null && defaultValue !== void 0 ? defaultValue : min);
  const validValues = useMemo(() => toRange(value !== null && value !== void 0 ? value : values, min, max), [max, min, value, values]);
  const validStep = step < 0 ? 0 : step;
  const isRange = isArray(defaultValue !== null && defaultValue !== void 0 ? defaultValue : value);
  const [isDragging, setIsDragging] = useState(false);
  const [draggedIndex, setDraggedIndex] = useState(-1);
  const [shortcutsScope] = useState(getUID('ring-slider-'));
  const markValues = useMemo(() => {
    if (isArray(marks)) {
      return marks.map(mark => _objectSpread2(_objectSpread2({}, mark), {}, {
        value: validateValue(mark.value, min, max)
      }));
    } else if (marks) {
      return calculateMarks(min, max, validStep);
    }
    return [];
  }, [marks, max, min, validStep]);
  const tickMarks = useMemo(() => {
    if (showTicks) {
      return markValues.length ? markValues : calculateMarks(min, max, validStep);
    }
    return [];
  }, [max, min, markValues, showTicks, validStep]);
  const trackStart = useMemo(() => toPercent(isRange ? Math.min(...validValues) : min, min, max), [isRange, max, min, validValues]);
  const trackLength = useMemo(() => toPercent(Math.max(...validValues), min, max) - trackStart, [max, min, trackStart, validValues]);
  const handleValueChange = useCallback(nextValues => {
    setValues(nextValues);
    onChange === null || onChange === void 0 || onChange(isRange ? nextValues : nextValues[0]);
  }, [isRange, onChange]);
  const shortcutsMap = useMemo(() => {
    const setValueAndSwap = (nextValue, index) => {
      const nextValues = [...validValues];
      nextValues[index] = nextValue;
      if (nextValues[0] > nextValues[1]) {
        var _ref$current;
        const previousValue = nextValues[index];
        nextValues.reverse();
        const thumb = (_ref$current = ref.current) === null || _ref$current === void 0 ? void 0 : _ref$current.querySelector("[role=\"slider\"][data-index=\"".concat(nextValues.indexOf(previousValue), "\"]"));
        thumb === null || thumb === void 0 || thumb.focus();
      }
      handleValueChange(nextValues);
    };
    const getIndex = target => Number(target === null || target === void 0 ? void 0 : target.getAttribute('data-index'));
    const map = {};
    if (!disabled) {
      map.left = map.down = _ref2 => {
        let {
          target
        } = _ref2;
        const index = getIndex(target);
        setValueAndSwap(Math.max(min, validValues[index] - validStep), index);
      };
      map.right = map.up = _ref3 => {
        let {
          target
        } = _ref3;
        const index = getIndex(target);
        setValueAndSwap(Math.min(max, validValues[index] + validStep), index);
      };
      map.home = _ref4 => {
        let {
          target
        } = _ref4;
        const index = getIndex(target);
        setValueAndSwap(min, index);
      };
      map.end = _ref5 => {
        let {
          target
        } = _ref5;
        const index = getIndex(target);
        setValueAndSwap(max, index);
      };
    }
    return map;
  }, [disabled, handleValueChange, max, min, validStep, validValues]);
  const handleMouseDown = useCallback(e => {
    e.stopPropagation();
    if (disabled) {
      return;
    }
    const index = e.currentTarget.getAttribute('data-index');
    const nextValue = calculateValue(ref, e.pageX, min, max, validStep);
    if (nextValue !== null && !isNaN(nextValue) && !index) {
      const rangeIndex = Number(Math.abs(validValues[0] - nextValue) > Math.abs(validValues[1] - nextValue));
      setDraggedIndex(isRange ? rangeIndex : 0);
    } else {
      setDraggedIndex(Number(index));
    }
    setIsDragging(true);
    previouslyDragged.current = false;
  }, [disabled, isRange, max, min, validStep, validValues]);
  const handleMouseUp = useCallback(_ref6 => {
    let {
      pageX
    } = _ref6;
    const nextValues = adjustValues(validValues, ref, draggedIndex, pageX, max, min, validStep);
    if (nextValues[0] > nextValues[1]) {
      nextValues.reverse();
    }
    handleValueChange(nextValues);
    setDraggedIndex(-1);
    setIsDragging(false);
    previouslyDragged.current = true;
  }, [validValues, draggedIndex, handleValueChange, max, min, validStep]);
  const handleMouseMove = useCallback(_ref7 => {
    let {
      pageX
    } = _ref7;
    const nextValues = adjustValues(validValues, ref, draggedIndex, pageX, max, min, validStep);
    if (nextValues[0] > nextValues[1]) {
      nextValues.reverse();
      setDraggedIndex(prevState => prevState === 0 ? 1 : 0);
    }
    handleValueChange(nextValues);
  }, [validValues, draggedIndex, max, min, validStep, handleValueChange]);
  useEffect(() => {
    if (disabled) {
      return undefined;
    }
    if (isDragging && !previouslyDragged.current) {
      window.addEventListener('mousemove', handleMouseMove);
      window.addEventListener('mouseup', handleMouseUp);
    } else if (!isDragging && previouslyDragged.current) {
      window.removeEventListener('mousemove', handleMouseMove);
      window.removeEventListener('mouseup', handleMouseUp);
    }
    return () => {
      window.removeEventListener('mousemove', handleMouseMove);
      window.removeEventListener('mouseup', handleMouseUp);
    };
  }, [isDragging, handleMouseMove, handleMouseUp, disabled]);
  return jsxs("div", {
    ref: ref,
    role: "presentation" // contains interactive elements
    ,
    className: classNames(styles.slider, className, {
      [styles.disabled]: disabled,
      [styles.marked]: !!marks || showTag
    }),
    tabIndex: -1,
    onMouseDown: handleMouseDown,
    children: [jsx(Shortcuts, {
      map: shortcutsMap,
      scope: shortcutsScope
    }), jsx("div", {
      className: classNames(styles.rail, {
        [styles.rounded]: !showTicks,
        [styles.disabled]: disabled
      })
    }), jsx("div", {
      style: {
        left: "".concat(trackStart, "%"),
        width: "".concat(trackLength, "%")
      },
      className: classNames(styles.track, {
        [styles.rounded]: !showTicks,
        [styles.disabled]: disabled
      })
    }), validValues.map((numValue, index) => {
      const percent = toPercent(numValue, min, max);
      return (
        // eslint-disable-next-line react/no-array-index-key
        jsxs(Fragment, {
          children: [jsx("div", {
            tabIndex: 0,
            "aria-label": "Pick value",
            role: "slider",
            "aria-valuemin": min,
            "aria-valuemax": max,
            "aria-valuenow": numValue,
            "data-index": index,
            style: {
              left: "".concat(percent, "%")
            },
            className: classNames(styles.thumb, {
              [styles.disabled]: disabled,
              [styles.dragged]: isDragging && draggedIndex === index
            }),
            onMouseDown: handleMouseDown
          }), showTag && jsx("div", {
            style: {
              left: "".concat(percent, "%")
            },
            className: classNames(styles.tag, {
              [styles.disabled]: disabled
            }),
            role: "tooltip",
            children: renderTag ? renderTag(numValue) : numValue
          })]
        }, index)
      );
    }), markValues.map((_ref8, index) => {
      let {
        value: markValue,
        label
      } = _ref8;
      const percent = toPercent(markValue, min, max);
      return jsx("div", {
        style: {
          left: "".concat(percent, "%")
        },
        className: classNames(styles.markValue, {
          [styles.disabled]: disabled
        }),
        children: label !== null && label !== void 0 ? label : markValue
      }, index);
    }), tickMarks.map((_ref9, index) => {
      let {
        value: tickValue
      } = _ref9;
      const percent = toPercent(tickValue, min, max);
      const isActive = isRange ? tickValue >= validValues[0] && tickValue <= validValues[validValues.length - 1] : tickValue <= validValues[0];
      return jsx("div", {
        className: classNames(styles.tick, {
          [styles.active]: isActive,
          [styles.disabled]: disabled
        }),
        style: {
          left: "".concat(percent, "%")
        }
      }, index);
    })]
  });
};

export { Slider };
