import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import alertService from '../alert-service/alert-service.js';
import 'react/jsx-runtime';
import 'react-dom/client';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../alert/alert.js';
import 'react';
import 'classnames';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/data-tests.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import 'react-dom';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../alert/container.js';

/**
 * @return {LocalStorage}
 * @param {{type: string}} config Set to "session" to use sessionStorage
 * @constructor
 */
class LocalStorage {
  static async safePromise(resolver) {
    try {
      return await new Promise(resolver);
    } catch (e) {
      if (e instanceof Error && e.name === 'NS_ERROR_FILE_CORRUPTED') {
        alertService.error('Sorry, it looks like your browser storage is corrupted. ' + 'Please clear your storage by going to Tools -> Clear Recent History -> Cookies' + ' and setting time range to "Everything". This will remove the corrupted browser storage across all sites.');
      }
      throw e;
    }
  }
  constructor() {
    let config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    _defineProperty(this, "storageType", void 0);
    this.storageType = config.type === 'session' ? 'sessionStorage' : 'localStorage';
  }
  /**
   * @param {string} name
   * @return {Promise}
   */
  get(name) {
    return LocalStorage.safePromise(resolve => {
      const value = window[this.storageType].getItem(name);
      if (value != null) {
        try {
          resolve(JSON.parse(value));
        } catch (e) {
          resolve(value);
        }
      } else {
        resolve(value);
      }
    });
  }
  /**
   * @param {string} name
   * @param {object} value
   * @return {Promise}
   */
  set(name, value) {
    return LocalStorage.safePromise(resolve => {
      window[this.storageType].setItem(name, JSON.stringify(value));
      resolve(value);
    });
  }
  /**
   * @param {string} name
   * @return {Promise}
   */
  remove(name) {
    const storageType = this.storageType;
    return LocalStorage.safePromise(resolve => {
      if (window[storageType].hasOwnProperty(name)) {
        window[storageType].removeItem(name);
      }
      resolve();
    });
  }
  /**
   * @param callback
   * @return {Promise}s
   */
  each(callback) {
    const storageType = this.storageType;
    return LocalStorage.safePromise(resolve => {
      const promises = [];
      for (const item in window[storageType]) {
        if (window[storageType].hasOwnProperty(item)) {
          const value = window[storageType].getItem(item);
          let resolvedValue = null;
          if (value != null) {
            try {
              resolvedValue = JSON.parse(value);
            } catch (e) {
              resolvedValue = value;
            }
          }
          promises.push(Promise.resolve(callback(item, resolvedValue)));
        }
      }
      resolve(Promise.all(promises));
    });
  }
  /**
   * @param {string} name
   * @param {Function} callback
   * @return {Function}
   */
  on(name, callback) {
    function handleStorage(e) {
      if (e.key === name) {
        if (e.newValue != null) {
          try {
            callback(JSON.parse(e.newValue));
          } catch (err) {
            callback(e.newValue);
          }
        } else {
          callback(e.newValue);
        }
      }
    }
    window.addEventListener('storage', handleStorage, false);
    return () => window.removeEventListener('storage', handleStorage, false);
  }
}

export { LocalStorage as default };
