import { a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { forwardRef, useRef, useImperativeHandle, useCallback, useEffect } from 'react';
import { isNodeInVisiblePartOfPage } from '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.split.js';

var styles = {"trapButton":"trapButton_rui_3588"};

const _excluded = ["children", "trapDisabled", "autoFocusFirst", "focusBackOnClose", "focusBackOnExit"];
const FOCUSABLE_ELEMENTS = 'input, button, select, textarea, a[href], *[tabindex]:not([data-trap-button]):not([data-scrollable-container])';
// eslint-disable-next-line @typescript-eslint/no-shadow
const TabTrap = /*#__PURE__*/forwardRef(function TabTrap(_ref, ref) {
  let {
      children,
      trapDisabled = false,
      autoFocusFirst = true,
      focusBackOnClose = true,
      focusBackOnExit = false
    } = _ref,
    restProps = _objectWithoutProperties(_ref, _excluded);
  const nodeRef = useRef(null);
  const trapButtonNodeRef = useRef(null);
  const previousFocusedNodeRef = useRef(null);
  const trapWithoutFocusRef = useRef(false);
  const mountedRef = useRef(false);
  // It's the same approach as in focus-trap-react:
  // https://github.com/focus-trap/focus-trap-react/commit/3b22fca9eebeb883edc89548850fe5a5b9d6d50e
  // We can't do it in useEffect because it's too late, some children might have already
  // focused itself.
  if (previousFocusedNodeRef.current === null) {
    previousFocusedNodeRef.current = document.activeElement;
  }
  useImperativeHandle(ref, () => ({
    node: nodeRef.current
  }), []);
  const focusFirst = useCallback(() => focusElement(true), []);
  const focusLast = () => focusElement(false);
  useEffect(() => {
    mountedRef.current = true;
    return () => {
      mountedRef.current = false;
    };
  }, []);
  useEffect(() => {
    if (autoFocusFirst) {
      focusFirst();
    } else if (!trapDisabled) {
      var _nodeRef$current, _nodeRef$current2;
      const previousFocusedElementIsInContainer = previousFocusedNodeRef.current && ((_nodeRef$current = nodeRef.current) === null || _nodeRef$current === void 0 ? void 0 : _nodeRef$current.contains(previousFocusedNodeRef.current));
      // The component wrapped in TabTrap can already have a focused element (e.g. Date Picker),
      // so we need to check if it does. If so, we don't need to focus anything.
      const currentlyFocusedElementIsInContainer = (_nodeRef$current2 = nodeRef.current) === null || _nodeRef$current2 === void 0 ? void 0 : _nodeRef$current2.contains(document.activeElement);
      if (!nodeRef.current || !previousFocusedElementIsInContainer && !currentlyFocusedElementIsInContainer) {
        var _trapButtonNodeRef$cu;
        trapWithoutFocusRef.current = true;
        (_trapButtonNodeRef$cu = trapButtonNodeRef.current) === null || _trapButtonNodeRef$cu === void 0 || _trapButtonNodeRef$cu.focus();
      }
    }
    return () => {
      if (focusBackOnClose) {
        restoreFocus();
      }
    };
  }, [autoFocusFirst, trapDisabled, focusBackOnClose, focusFirst]);
  function restoreFocus() {
    const previousFocusedNode = previousFocusedNodeRef.current;
    if (previousFocusedNode instanceof HTMLElement && previousFocusedNode.focus && isNodeInVisiblePartOfPage(previousFocusedNode)) {
      // This is to prevent the focus from being restored the first time
      // componentWillUnmount is called in StrictMode.
      if (!mountedRef.current) {
        previousFocusedNode.focus({
          preventScroll: true
        });
      }
    }
  }
  function focusElement() {
    let first = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
    const node = nodeRef.current;
    if (!node) {
      return;
    }
    const tabables = [...node.querySelectorAll(FOCUSABLE_ELEMENTS)].filter(item => item.tabIndex >= 0);
    const toBeFocused = first ? tabables[0] : tabables[tabables.length - 1];
    if (toBeFocused) {
      toBeFocused.focus();
    }
  }
  function focusLastIfEnabled(event) {
    if (trapWithoutFocusRef.current) {
      return;
    }
    if (focusBackOnExit) {
      const prevFocused = event.nativeEvent.relatedTarget;
      if (prevFocused != null && nodeRef.current != null && prevFocused instanceof Element && nodeRef.current.contains(prevFocused)) {
        restoreFocus();
      }
    } else {
      focusLast();
    }
  }
  function handleBlurIfWithoutFocus(event) {
    var _nodeRef$current3;
    if (!trapWithoutFocusRef.current) {
      return;
    }
    trapWithoutFocusRef.current = false;
    const newFocused = event.nativeEvent.relatedTarget;
    if (!newFocused) {
      return;
    }
    if (newFocused instanceof Element && (_nodeRef$current3 = nodeRef.current) !== null && _nodeRef$current3 !== void 0 && _nodeRef$current3.contains(newFocused)) {
      return;
    }
    focusLast();
  }
  if (trapDisabled) {
    return jsx("div", _objectSpread2(_objectSpread2({
      ref: nodeRef
    }, restProps), {}, {
      children: children
    }));
  }
  return jsxs("div", _objectSpread2(_objectSpread2({
    ref: nodeRef
  }, restProps), {}, {
    children: [jsx("div", {
      // It never actually stays focused
      // eslint-disable-next-line jsx-a11y/no-noninteractive-tabindex
      tabIndex: 0,
      ref: trapButtonNodeRef,
      className: styles.trapButton,
      onFocus: focusLastIfEnabled,
      onBlur: handleBlurIfWithoutFocus,
      "data-trap-button": true
    }), children, jsx("div", {
      // It never actually stays focused
      // eslint-disable-next-line jsx-a11y/no-noninteractive-tabindex
      tabIndex: 0,
      onFocus: focusBackOnExit ? restoreFocus : focusFirst,
      "data-trap-button": true
    })]
  }));
});

export { FOCUSABLE_ELEMENTS, TabTrap as default };
