import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import sortableIcon from '@jetbrains/icons/unsorted-12px';
import chevron12pxDown from '@jetbrains/icons/chevron-12px-down';
import Icon from '../icon/icon.js';
import joinDataTestAttributes from '../global/data-tests.js';
import { s as style } from '../_helpers/table.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';

const _excluded = ["className", "column", "onSort", "sortKey", "sortOrder", "data-test"];
class HeaderCell extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "sortable", void 0);
    _defineProperty(this, "sorted", void 0);
    _defineProperty(this, "onClick", () => {
      if (this.sortable) {
        const {
          column,
          onSort,
          sortOrder
        } = this.props;
        onSort({
          column,
          order: !(this.sorted && sortOrder)
        });
      }
    });
  }
  onChildrenClick(e) {
    e.stopPropagation();
  }
  render() {
    const _this$props = this.props,
      {
        className,
        column,
        onSort,
        sortKey,
        sortOrder,
        'data-test': dataTest
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    this.sortable = column.sortable === true;
    this.sorted = sortKey === column.id;
    const glyph = this.sorted ? chevron12pxDown : sortableIcon;
    const classes = classNames(className, column.headerClassName, {
      [style.headerCell]: true,
      [style.headerCellSortable]: this.sortable,
      [style.headerCellSorted]: this.sorted,
      [style.sortedUp]: sortOrder && this.sorted,
      [style.cellRight]: column.rightAlign
    });
    return jsxs("th", _objectSpread2(_objectSpread2({}, restProps), {}, {
      className: classes,
      onClick: this.onClick,
      "data-test": joinDataTestAttributes('ring-table-header-cell', dataTest),
      children: [jsx("span", {
        onClick: this.onChildrenClick,
        role: "presentation",
        children: this.props.children
      }), column.getHeaderValue ? column.getHeaderValue() : column.title, this.sortable && jsx("span", {
        className: style.sorter,
        children: jsx(Icon, {
          glyph: glyph,
          className: style.icon
        })
      })]
    }));
  }
}
_defineProperty(HeaderCell, "defaultProps", {
  onSort: () => {}
});

export { HeaderCell as default };
