import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import Checkbox from '../checkbox/checkbox.js';
import getUID from '../global/get-uid.js';
import { s as style } from '../_helpers/table.js';
import HeaderCell from './header-cell.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/composeRefs.js';
import 'memoize-one';
import '../control-help/control-help.js';
import 'core-js/modules/es.regexp.to-string.js';
import '@jetbrains/icons/unsorted-12px';
import '@jetbrains/icons/chevron-12px-down';
import '../global/data-tests.js';

class Header extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "id", getUID('table-header-'));
    _defineProperty(this, "onCheckboxFocus", event => {
      event.currentTarget.blur();
    });
  }
  createCells() {
    let widths = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    const {
      selectable,
      draggable,
      columns,
      checked,
      checkboxDisabled,
      onCheckboxChange,
      onSort,
      sortKey,
      sortOrder
    } = this.props;
    const metaColumnClasses = classNames(style.metaColumn, style.headerMetaColumn);
    const metaColumn = jsx("div", {
      className: metaColumnClasses,
      children: selectable && jsx(Checkbox, {
        "aria-labelledby": this.id,
        disabled: checkboxDisabled,
        checked: checked,
        onChange: onCheckboxChange,
        onFocus: this.onCheckboxFocus
      })
    });
    return columns.map((column, index) => {
      const columnStyle = widths[index] ? {
        width: widths[index]
      } : undefined;
      const props = {
        column,
        onSort,
        sortKey,
        sortOrder,
        style: columnStyle
      };
      return jsx(HeaderCell, _objectSpread2(_objectSpread2({
        "data-test": column.id
      }, props), {}, {
        children: index === 0 && (draggable || selectable) && metaColumn
      }), column.id);
    });
  }
  render() {
    const {
      caption,
      sticky,
      topStickOffset
    } = this.props;
    const regularCells = this.createCells();
    return jsxs("thead", {
      id: this.id,
      "data-test": "ring-table-header",
      style: {
        top: topStickOffset
      },
      className: classNames({
        [style.tableHead]: true,
        [style.subHeaderSticky]: sticky
      }),
      children: [caption && jsx("tr", {
        "data-test": "ring-table-header-row",
        children: jsx("th", {
          className: classNames(style.headerCell, style.caption),
          colSpan: regularCells.length + 1,
          "data-test": "ring-table-header-cell",
          children: caption
        })
      }), jsx("tr", {
        className: style.subHeader,
        "data-test": "ring-table-header-row",
        children: regularCells
      })]
    });
  }
}
_defineProperty(Header, "defaultProps", {
  selectable: true,
  draggable: false,
  checked: true,
  sticky: true,
  topStickOffset: '0px',
  onSort: () => {},
  onCheckboxChange: () => {},
  sortKey: 'id',
  sortOrder: true
});

export { Header as default };
