import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import chevronRightIcon from '@jetbrains/icons/chevron-right';
import chevronDownIcon from '@jetbrains/icons/chevron-down';
import dragIcon from '@jetbrains/icons/drag';
import Checkbox from '../checkbox/checkbox.js';
import { Button } from '../button/button.js';
import Tooltip from '../tooltip/tooltip.js';
import joinDataTestAttributes from '../global/data-tests.js';
import getUID from '../global/get-uid.js';
import { createComposedRef } from '../global/composeRefs.js';
import Cell from './cell.js';
import { s as style } from '../_helpers/table.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../control-help/control-help.js';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../popup/popup.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'react-dom';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../_helpers/theme.js';
import 'memoize-one';

const _excluded = ["item", "columns", "selectable", "selected", "showFocus", "draggable", "alwaysShowDragHandle", "dragHandleTitle", "level", "collapsible", "parentCollapsible", "collapsed", "onCollapse", "onExpand", "showDisabledSelection", "onSelect", "checkboxTooltip", "innerRef", "focused", "autofocus", "onFocusReset", "onFocusRestore", "onHover", "className", "metaColumnClassName", "data-test"];
const DragHandle = _ref => {
  let {
    alwaysShowDragHandle,
    dragHandleTitle = 'Drag to reorder'
  } = _ref;
  const classes = classNames(style.dragHandle, {
    [style.visibleDragHandle]: alwaysShowDragHandle
  });
  return jsx(Button, {
    "data-movable-handle": true,
    title: dragHandleTitle,
    className: classes,
    icon: dragIcon
  });
};
class Row extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "id", getUID('table-row-'));
    _defineProperty(this, "onMouseEnter", e => {
      const {
        item,
        onHover
      } = this.props;
      onHover(item, e);
    });
    _defineProperty(this, "onClick", e => {
      const {
        item
      } = this.props;
      this.props.onClick(item, e);
      if (e.shiftKey) {
        this.toggleSelection();
      }
    });
    _defineProperty(this, "onCheckboxFocus", () => {
      this.props.onFocusRestore();
    });
    _defineProperty(this, "onCheckboxChange", () => {
      this.toggleSelection();
    });
    _defineProperty(this, "onDoubleClick", () => {
      const {
        item
      } = this.props;
      this.props.onDoubleClick(item);
    });
    _defineProperty(this, "row", void 0);
    _defineProperty(this, "rowRef", el => {
      this.row = el;
    });
    _defineProperty(this, "composedRowRef", createComposedRef());
  }
  toggleSelection() {
    const {
      selectable,
      selected,
      onSelect,
      item
    } = this.props;
    if (selectable) {
      onSelect(item, !selected);
    }
  }
  render() {
    const _this$props = this.props,
      {
        item,
        columns: columnProps,
        selectable,
        selected,
        showFocus,
        draggable,
        alwaysShowDragHandle,
        dragHandleTitle,
        level,
        collapsible,
        parentCollapsible,
        collapsed,
        onCollapse,
        onExpand,
        showDisabledSelection,
        onSelect,
        checkboxTooltip,
        innerRef,
        focused,
        autofocus,
        onFocusReset,
        onFocusRestore,
        onHover,
        className,
        metaColumnClassName,
        'data-test': dataTest
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const classes = classNames(className, {
      [style.row]: true,
      [style.rowFocused]: showFocus,
      [style.rowSelected]: selected
    });
    const testAttrs = {
      'data-test-focused': showFocus || undefined,
      'data-test-selected': selected || undefined
    };
    const metaColumnClasses = classNames(metaColumnClassName, style.metaColumn);
    const SUBITEM_OFFSET = 30;
    const COLLAPSIBLE_PARENT_OFFSET = 20;
    const gap = level * SUBITEM_OFFSET + (parentCollapsible ? COLLAPSIBLE_PARENT_OFFSET : 0);
    const metaColumnStyle = {
      paddingLeft: "".concat(gap, "px")
    };
    const metaColumn = jsxs("div", {
      className: metaColumnClasses,
      style: metaColumnStyle,
      children: [draggable && jsx(DragHandle, {
        alwaysShowDragHandle: alwaysShowDragHandle,
        dragHandleTitle: dragHandleTitle
      }), selectable && jsx(Tooltip, {
        title: checkboxTooltip,
        children: jsx(Checkbox, {
          "aria-labelledby": this.id,
          className: showFocus ? 'ring-checkbox_focus' : '',
          checked: selected,
          onFocus: this.onCheckboxFocus,
          onChange: this.onCheckboxChange,
          tabIndex: -1
        })
      }), !selectable && showDisabledSelection && jsx(Tooltip, {
        title: checkboxTooltip,
        children: jsx(Checkbox, {
          "aria-labelledby": this.id,
          checked: selected,
          disabled: true
        })
      }), collapsible && collapsed && jsx(Button, {
        className: style.rowCollapseExpandButton,
        icon: chevronRightIcon,
        onClick: () => onExpand(item)
      }), collapsible && !collapsed && jsx(Button, {
        className: style.rowCollapseExpandButton,
        icon: chevronDownIcon,
        onClick: () => onCollapse(item)
      })]
    });
    const columns = typeof columnProps === 'function' ? columnProps(item) : columnProps;
    const cells = columns.map((column, index) => {
      const getValue = column.getValue || (() => item[column.id]);
      const getDataTest = column.getDataTest || (() => column.id);
      const value = getValue(item, column);
      const cellClasses = classNames({
        [style.cellRight]: column.rightAlign
      }, column.className);
      const showMetaColumn = draggable || selectable || collapsible || showDisabledSelection || !!level;
      return jsxs(Cell, {
        colSpan: column.colSpan,
        className: cellClasses,
        "data-test": getDataTest(item, column),
        children: [index === 0 && showMetaColumn && metaColumn, value]
      }, column.id);
    });
    return jsx("tr", _objectSpread2(_objectSpread2(_objectSpread2({
      id: this.id,
      ref: this.composedRowRef(this.rowRef, innerRef),
      className: classes,
      tabIndex: 0,
      "data-test": joinDataTestAttributes('ring-table-row', dataTest)
    }, testAttrs), restProps), {}, {
      onMouseMove: this.onMouseEnter,
      onClick: this.onClick,
      onDoubleClick: this.onDoubleClick,
      children: cells
    }));
  }
}
_defineProperty(Row, "defaultProps", {
  selectable: true,
  showFocus: false,
  draggable: false,
  alwaysShowDragHandle: false,
  selected: false,
  onHover: () => {},
  onSelect: () => {},
  onDoubleClick: () => {},
  onClick: () => {},
  level: 0,
  collapsible: false,
  parentCollapsible: false,
  collapsed: false,
  onCollapse: () => {},
  onExpand: () => {}
});

export { Row as default };
