import { b as _objectSpread2, _ as _defineProperty, a as _objectWithoutProperties } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';

const _excluded = ["selection", "selectable", "onSelect", "shortcuts"];
function selectionShortcutsHOC(ComposedComponent) {
  class SelectionShortcuts extends PureComponent {
    constructor() {
      super(...arguments);
      _defineProperty(this, "onUpPress", () => {
        const {
          selection,
          onSelect
        } = this.props;
        const newSelection = selection.moveUp();
        if (newSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
        return false;
      });
      _defineProperty(this, "onDownPress", () => {
        const {
          selection,
          onSelect
        } = this.props;
        const newSelection = selection.moveDown();
        if (newSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
        return false;
      });
      _defineProperty(this, "shiftSelectionMode", void 0);
      _defineProperty(this, "onShiftKeyDown", () => {
        const {
          selection
        } = this.props;
        if (selection.isSelected(selection.getFocused())) {
          this.shiftSelectionMode = 'deletion';
        } else {
          this.shiftSelectionMode = 'addition';
        }
      });
      _defineProperty(this, "shiftSelect", selection => {
        if (this.shiftSelectionMode === 'addition') {
          return selection.select();
        } else {
          return selection.deselect();
        }
      });
      _defineProperty(this, "onShiftUpPress", e => {
        e.preventDefault();
        const {
          selectable,
          selection,
          onSelect
        } = this.props;
        if (!selectable) {
          return;
        }
        const newSelection = this.shiftSelect(selection);
        const newMovedSelection = newSelection.moveUp();
        if (newMovedSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newMovedSelection);
        } else {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
      });
      _defineProperty(this, "onShiftDownPress", e => {
        e.preventDefault();
        const {
          selectable,
          selection,
          onSelect
        } = this.props;
        if (!selectable) {
          return;
        }
        const newSelection = this.shiftSelect(selection);
        const newMovedSelection = newSelection.moveDown();
        if (newMovedSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newMovedSelection);
        } else {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
      });
      _defineProperty(this, "onHomePress", () => {
        const {
          selection,
          onSelect
        } = this.props;
        const newSelection = selection.moveStart();
        if (newSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
        return false;
      });
      _defineProperty(this, "onEndPress", () => {
        const {
          selection,
          onSelect
        } = this.props;
        const newSelection = selection.moveEnd();
        if (newSelection) {
          onSelect === null || onSelect === void 0 || onSelect(newSelection);
        }
        return false;
      });
      _defineProperty(this, "onSpacePress", () => {
        const {
          selectable,
          selection,
          onSelect
        } = this.props;
        if (!selectable) {
          return true;
        }
        onSelect === null || onSelect === void 0 || onSelect(selection.toggleSelection());
        return false;
      });
      _defineProperty(this, "onEscPress", () => {
        const {
          selection,
          onSelect
        } = this.props;
        onSelect === null || onSelect === void 0 || onSelect(selection.reset());
        //this.restoreFocusWithoutScroll();
      });
      _defineProperty(this, "onCmdAPress", () => {
        const {
          selectable,
          selection,
          onSelect
        } = this.props;
        if (!selectable) {
          return true;
        }
        onSelect === null || onSelect === void 0 || onSelect(selection.selectAll());
        return false;
      });
      _defineProperty(this, "shortcutsMap", {
        up: this.onUpPress,
        down: this.onDownPress,
        shift: this.onShiftKeyDown,
        'shift+up': this.onShiftUpPress,
        'shift+down': this.onShiftDownPress,
        home: this.onHomePress,
        end: this.onEndPress,
        space: this.onSpacePress,
        esc: this.onEscPress,
        'command+a': this.onCmdAPress,
        'ctrl+a': this.onCmdAPress
      });
    }
    render() {
      const _this$props = this.props,
        {
          selection,
          selectable,
          onSelect,
          shortcuts
        } = _this$props,
        restProps = _objectWithoutProperties(_this$props, _excluded);
      return jsx(ComposedComponent, _objectSpread2(_objectSpread2({}, restProps), {}, {
        selection: selection,
        selectable: selectable,
        onSelect: onSelect,
        shortcutsMap: _objectSpread2(_objectSpread2({}, this.shortcutsMap), this.props.shortcuts)
      }));
    }
  }
  SelectionShortcuts.defaultProps = _objectSpread2(_objectSpread2({}, ComposedComponent.defaultProps), {}, {
    selectable: true,
    onSelect: () => {},
    shortcuts: {}
  });
  return SelectionShortcuts;
}

export { selectionShortcutsHOC as default };
