import { _ as _defineProperty, b as _objectSpread2, a as _objectWithoutProperties } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { PureComponent, createElement, Component } from 'react';
import { jsxs, jsx } from 'react/jsx-runtime';
import classNames from 'classnames';
import { arrayMove, List } from 'react-movable';
import focusSensorHOC from '../global/focus-sensor-hoc.js';
import getUID from '../global/get-uid.js';
import Shortcuts from '../shortcuts/shortcuts.js';
import Loader from '../loader/loader.js';
import Header from './header.js';
import { s as style } from '../_helpers/table.js';
import selectionShortcutsHOC from './selection-shortcuts-hoc.js';
import disableHoverHOC from './disable-hover-hoc.js';
import RowWithFocusSensorCallbacks from './row-with-focus-sensor.js';
import '../global/composeRefs.js';
import 'memoize-one';
import 'core-js/modules/es.regexp.to-string.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/data-tests.js';
import '../loader/loader__core.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../control-help/control-help.js';
import './header-cell.js';
import '@jetbrains/icons/unsorted-12px';
import '@jetbrains/icons/chevron-12px-down';
import './row.js';
import '@jetbrains/icons/chevron-right';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/drag';
import '../button/button.js';
import '../link/clickableLink.js';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import '../tooltip/tooltip.js';
import '../popup/popup.js';
import 'react-dom';
import '../global/schedule-raf.js';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../_helpers/theme.js';
import './cell.js';

const _excluded = ["ref"];
/**
 * Interactive table with selection and keyboard navigation support.
 */
class Table extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      shortcutsScope: getUID('ring-table-'),
      userSelectNone: false
    });
    _defineProperty(this, "onMouseDown", e => {
      if (e.shiftKey) {
        this.setState({
          userSelectNone: true
        });
      }
    });
    _defineProperty(this, "onMouseUp", () => {
      if (this.state.userSelectNone) {
        this.setState({
          userSelectNone: false
        });
      }
    });
    _defineProperty(this, "onRowFocus", row => {
      const {
        selection,
        onSelect
      } = this.props;
      onSelect(selection.focus(row));
    });
    _defineProperty(this, "onRowSelect", (row, selected) => {
      const {
        selection,
        onSelect
      } = this.props;
      if (selected) {
        onSelect(selection.select(row));
      } else {
        onSelect(selection.deselect(row));
      }
    });
    _defineProperty(this, "onSortEnd", _ref => {
      let {
        oldIndex,
        newIndex
      } = _ref;
      const data = arrayMove(this.props.data, oldIndex, newIndex);
      this.props.onReorder({
        data,
        oldIndex,
        newIndex
      });
    });
    _defineProperty(this, "onCheckboxChange", e => {
      const {
        checked
      } = e.currentTarget;
      const {
        selection,
        onSelect
      } = this.props;
      if (checked) {
        onSelect(selection.selectAll());
      } else {
        onSelect(selection.reset());
      }
      this.restoreFocusWithoutScroll();
    });
    _defineProperty(this, "restoreFocusWithoutScroll", () => {
      const {
        scrollX,
        scrollY
      } = window;
      this.props.onFocusRestore();
      window.scrollTo(scrollX, scrollY);
    });
  }
  componentDidMount() {
    document.addEventListener('mouseup', this.onMouseUp);
  }
  componentDidUpdate(_ref2) {
    let {
      data,
      selection,
      onSelect,
      selectable,
      remoteSelection
    } = _ref2;
    if (data !== this.props.data && remoteSelection) {
      onSelect(selection.cloneWith({
        data: this.props.data
      }));
    }
    if (!this.props.selectable && this.props.selectable !== selectable) {
      onSelect(selection.resetSelection());
    }
  }
  componentWillUnmount() {
    document.removeEventListener('mouseup', this.onMouseUp);
  }
  render() {
    const {
      data,
      selection,
      columns,
      caption,
      getItemKey,
      selectable,
      focused,
      isItemSelectable,
      getItemLevel,
      getItemClassName,
      getMetaColumnClassName,
      getItemDataTest,
      draggable,
      alwaysShowDragHandle,
      dragHandleTitle,
      loading,
      onSort,
      sortKey,
      sortOrder,
      loaderClassName,
      stickyHeader,
      stickyHeaderOffset,
      isItemCollapsible,
      isParentCollapsible,
      isItemCollapsed,
      onItemCollapse,
      onItemExpand,
      isDisabledSelectionVisible,
      getCheckboxTooltip,
      onItemDoubleClick,
      onItemClick,
      renderEmpty,
      RowComponent,
      renderLoader
    } = this.props;
    // NOTE: Do not construct new object per render because it causes all rows rerendering
    const columnsArray = typeof columns === 'function' ? columns(null) : columns;
    const headerProps = {
      caption,
      selectable,
      draggable,
      columns: columnsArray,
      onSort,
      sortKey,
      sortOrder,
      sticky: stickyHeader,
      topStickOffset: stickyHeaderOffset
    };
    const selectedSize = selection.getSelected().size;
    const allSelectedSize = selection.selectAll().getSelected().size;
    headerProps.checked = selectedSize > 0 && selectedSize === allSelectedSize;
    headerProps.onCheckboxChange = this.onCheckboxChange;
    headerProps.checkboxDisabled = this.props.data.length === 0;
    const wrapperClasses = classNames({
      [style.tableWrapper]: true,
      [style.loading]: loading
    });
    const classes = classNames(this.props.className, {
      [style.table]: true,
      [style.wideFirstColumn]: this.props.wideFirstColumn,
      [style.multiSelection]: selection.getSelected().size > 0,
      [style.userSelectNone]: this.state.userSelectNone,
      [style.disabledHover]: this.props.disabledHover
    });
    const renderList = _ref3 => {
      let {
        children,
        props
      } = _ref3;
      const empty = jsx("tr", {
        children: jsx("td", {
          colSpan: columnsArray.length || 1,
          className: style.tableMessage,
          children: renderEmpty ? renderEmpty() : null
        })
      });
      const tbody = Array.isArray(children) && children.length > 0 ? children : empty;
      return jsxs("table", {
        className: classes,
        "data-test": "ring-table",
        children: [jsx(Header, _objectSpread2({}, headerProps)), jsx("tbody", _objectSpread2(_objectSpread2({}, props), {}, {
          "data-test": "ring-table-body",
          children: tbody
        }))]
      });
    };
    const renderItem = _ref4 => {
      var _restProps$key;
      let {
        value,
        props = {},
        isDragged
      } = _ref4;
      if (value == null) {
        return null;
      }
      const {
          ref
        } = props,
        restProps = _objectWithoutProperties(props, _excluded);
      const row = /*#__PURE__*/createElement(RowComponent, _objectSpread2(_objectSpread2({
        innerRef: ref,
        level: getItemLevel(value),
        item: value,
        showFocus: selection.isFocused(value),
        autofocus: selection.isFocused(value),
        focused: focused && selection.isFocused(value),
        selectable: selectable && isItemSelectable(value),
        selected: selectable && selection.isSelected(value),
        onFocus: this.onRowFocus,
        onSelect: this.onRowSelect,
        onDoubleClick: onItemDoubleClick,
        onClick: onItemClick,
        collapsible: isItemCollapsible(value),
        parentCollapsible: isParentCollapsible(value),
        collapsed: isItemCollapsed(value),
        onCollapse: onItemCollapse,
        onExpand: onItemExpand,
        showDisabledSelection: isDisabledSelectionVisible(value),
        checkboxTooltip: getCheckboxTooltip(value),
        className: classNames(getItemClassName(value), {
          [style.draggingRow]: isDragged
        }),
        metaColumnClassName: getMetaColumnClassName(value),
        draggable: draggable,
        alwaysShowDragHandle: alwaysShowDragHandle,
        dragHandleTitle: dragHandleTitle,
        columns: columns,
        "data-test": getItemDataTest(value)
      }, restProps), {}, {
        key: (_restProps$key = restProps.key) !== null && _restProps$key !== void 0 ? _restProps$key : getItemKey(value)
      }));
      return isDragged ? jsx("table", {
        style: _objectSpread2({}, props.style),
        className: style.draggingTable,
        children: jsx("tbody", {
          children: row
        })
      }) : row;
    };
    return jsxs("div", {
      className: wrapperClasses,
      "data-test": "ring-table-wrapper",
      ref: this.props.innerRef,
      children: [focused && jsx(Shortcuts, {
        map: this.props.shortcutsMap,
        scope: this.state.shortcutsScope
      }), jsx("div", {
        role: "presentation",
        onMouseDown: this.onMouseDown,
        children: draggable ? jsx(List, {
          values: data,
          renderList: renderList,
          renderItem: renderItem,
          onChange: this.onSortEnd
        }) : renderList({
          children: data.map((value, index) => renderItem({
            value}))
        })
      }), loading && jsx("div", {
        className: style.loadingOverlay,
        children: renderLoader ? renderLoader(loaderClassName) : jsx(Loader, {
          className: loaderClassName
        })
      })]
    });
  }
}
_defineProperty(Table, "defaultProps", {
  isItemSelectable: () => true,
  loading: false,
  onSort: () => {},
  onReorder: () => {},
  getItemKey: item => item.id,
  sortKey: 'id',
  sortOrder: true,
  draggable: false,
  alwaysShowDragHandle: false,
  stickyHeader: true,
  getItemLevel: () => 0,
  getItemClassName: () => null,
  getMetaColumnClassName: () => null,
  getItemDataTest: () => null,
  isItemCollapsible: () => false,
  isParentCollapsible: () => false,
  isItemCollapsed: () => false,
  onItemCollapse: () => {},
  onItemExpand: () => {},
  onItemDoubleClick: () => {},
  onItemClick: () => {},
  remoteSelection: false,
  isDisabledSelectionVisible: () => false,
  getCheckboxTooltip: () => undefined,
  RowComponent: RowWithFocusSensorCallbacks,
  wideFirstColumn: false
});
const getContainer = () => disableHoverHOC(selectionShortcutsHOC(focusSensorHOC(Table)));
// eslint-disable-next-line react/no-multi-comp
class TableContainer extends Component {
  constructor() {
    super(...arguments);
    // https://stackoverflow.com/a/53882322/6304152
    _defineProperty(this, "Table", getContainer());
  }
  render() {
    return jsx(this.Table, _objectSpread2({}, this.props));
  }
}

export { Table, TableContainer as default };
