import { a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsx } from 'react/jsx-runtime';
import { memo, useMemo } from 'react';
import classNames from 'classnames';
import { Directions } from '../popup/popup.consts.js';
import PopupMenu, { ListProps } from '../popup-menu/popup-menu.js';
import Dropdown from '../dropdown/dropdown.js';
import { A as Anchor } from '../_helpers/anchor.js';
import { s as styles } from '../_helpers/tab-link.js';
import getTabTitles from './collapsible-tab.js';
import { CustomItem } from './custom-item.js';
import 'core-js/modules/es.parse-float.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '../popup/popup.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'react-dom';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../global/data-tests.js';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.target.js';
import '../list/list.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.trim.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'util-deprecate';
import 'memoize-one';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../avatar/avatar.js';
import '../global/url.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../link/link.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../global/typescript-utils.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';

const _excluded = ["hasActiveChildren", "moreClassName", "moreActiveClassName"];
const AnchorLink = _ref => {
  let {
      hasActiveChildren,
      moreClassName,
      moreActiveClassName
    } = _ref,
    restProps = _objectWithoutProperties(_ref, _excluded);
  const classnames = classNames(styles.title, hasActiveChildren && styles.selected, hasActiveChildren && moreActiveClassName, moreClassName);
  return jsx(Anchor, _objectSpread2(_objectSpread2({
    title: 'More',
    className: classnames
  }, restProps), {}, {
    children: 'More'
  }));
};
const morePopupDirections = [Directions.BOTTOM_CENTER, Directions.BOTTOM_LEFT, Directions.BOTTOM_RIGHT];
const MoreButton = /*#__PURE__*/memo(_ref2 => {
  let {
    items,
    selected,
    onSelect,
    moreClassName,
    moreActiveClassName,
    morePopupClassName,
    morePopupItemClassName,
    morePopupBeforeEnd
  } = _ref2;
  const onSelectHandler = useMemo(() => onSelect != null ? listItem => {
    if (listItem.disabled === true || listItem.custom === true) {
      return;
    }
    const cb = onSelect(String(listItem.key));
    cb();
  } : undefined, [onSelect]);
  const hasActiveChild = useMemo(() => items.some(item => item.props.alwaysHidden && item.props.id === selected), [items, selected]);
  const data = useMemo(() => {
    const popupItems = getTabTitles({
      items,
      selected,
      collapsed: true
    }).map(tab => {
      const disabled = tab.props.disabled === true;
      const custom = tab.props.child.type === CustomItem;
      return {
        template: tab,
        key: tab.key,
        rgItemType: ListProps.Type.CUSTOM,
        className: morePopupItemClassName,
        disabled,
        custom
      };
    });
    if (morePopupBeforeEnd) {
      popupItems.push({
        template: morePopupBeforeEnd,
        key: 'before-end-content',
        className: styles.morePopupBeforeEnd,
        rgItemType: ListProps.Type.CUSTOM
      });
    }
    return popupItems;
  }, [items, morePopupBeforeEnd, morePopupItemClassName, selected]);
  const popupAnchor = useMemo(() => jsx(AnchorLink, {
    moreClassName: moreClassName,
    moreActiveClassName: moreActiveClassName,
    hasActiveChildren: hasActiveChild
  }), [hasActiveChild, moreActiveClassName, moreClassName]);
  const popup = useMemo(() => jsx(PopupMenu, {
    directions: morePopupDirections,
    className: morePopupClassName,
    onSelect: onSelectHandler,
    data: data
  }), [data, morePopupClassName, onSelectHandler]);
  if (items.length === 0) {
    return null;
  }
  return jsx("div", {
    className: classNames(styles.title, moreClassName, hasActiveChild && moreActiveClassName),
    children: jsx(Dropdown, {
      hoverMode: true,
      anchor: popupAnchor,
      children: popup
    })
  });
});
MoreButton.displayName = 'MoreButton';
const FakeMoreButton = /*#__PURE__*/memo(_ref3 => {
  let {
    moreClassName,
    moreActiveClassName,
    hasActiveChildren
  } = _ref3;
  return jsx("div", {
    className: classNames(styles.moreButton, styles.title),
    children: jsx(AnchorLink, {
      moreClassName: moreClassName,
      moreActiveClassName: moreActiveClassName,
      hasActiveChildren: hasActiveChildren,
      tabIndex: -1,
      disabled: true
    })
  });
});
FakeMoreButton.displayName = 'FakeMoreButton';

export { AnchorLink, FakeMoreButton, MoreButton };
