import { a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import { jsx } from 'react/jsx-runtime';
import * as React from 'react';
import classNames from 'classnames';
import { s as styles, T as TabLink } from '../_helpers/tab-link.js';
import { CustomItem } from './custom-item.js';
import '../link/link.js';
import '../global/data-tests.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';

const _excluded = ["items", "selected", "collapsed", "onSelect"];
const TabTitle = /*#__PURE__*/React.memo(function TabTitle(_ref) {
  let {
    selected,
    child,
    handleSelect,
    collapsed = false,
    tabIndex
  } = _ref;
  if (child == null || typeof child !== 'object' || child.type === CustomItem) {
    return child;
  }
  const {
    title,
    titleProps,
    disabled,
    href,
    className,
    activeClassName,
    collapsedClassName,
    collapsedActiveClassName
  } = child.props;
  const titleClasses = classNames(styles.title, className, {
    [styles.selected]: selected,
    [styles.collapsed]: collapsed,
    [activeClassName !== null && activeClassName !== void 0 ? activeClassName : '']: selected,
    [collapsedClassName !== null && collapsedClassName !== void 0 ? collapsedClassName : '']: collapsed,
    [collapsedActiveClassName !== null && collapsedActiveClassName !== void 0 ? collapsedActiveClassName : '']: collapsed && selected
  });
  return jsx(TabLink, _objectSpread2({
    title: title,
    isSelected: selected,
    active: true,
    href: href,
    className: titleClasses,
    disabled: disabled,
    onPlainLeftClick: handleSelect,
    tabIndex: tabIndex,
    collapsed: collapsed
  }, titleProps));
});
const getTabTitles = _ref2 => {
  let {
      items,
      selected = '0',
      collapsed,
      onSelect = () => undefined
    } = _ref2,
    props = _objectWithoutProperties(_ref2, _excluded);
  return items.map((tab, index) => {
    const key = tab.props.id || String(index);
    const isSelected = selected === key;
    return jsx(TabTitle, _objectSpread2({
      handleSelect: onSelect(key),
      selected: isSelected,
      child: tab,
      collapsed: collapsed
    }, props), key);
  });
};

export { getTabTitles as default };
