import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { memo, useState, useRef, useMemo, useCallback, useEffect } from 'react';
import classNames from 'classnames';
import fastdom from 'fastdom';
import { s as styles } from '../_helpers/tab-link.js';
import { MoreButton, FakeMoreButton } from './collapsible-more.js';
import getTabTitles from './collapsible-tab.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import '../link/link.js';
import '../global/data-tests.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '../popup/popup.consts.js';
import '../popup-menu/popup-menu.js';
import 'core-js/modules/es.parse-float.js';
import '../popup/popup.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'react-dom';
import '../global/get-uid.js';
import '../global/schedule-raf.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.target.js';
import '../list/list.js';
import 'core-js/modules/es.symbol.description.js';
import 'core-js/modules/es.string.trim.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'util-deprecate';
import 'memoize-one';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../avatar/avatar.js';
import '../global/url.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../global/get-event-key.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../dropdown/dropdown.js';
import '../global/typescript-utils.js';
import '../_helpers/anchor.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../global/controls-height.js';
import '../_helpers/button__classes.js';
import './custom-item.js';

const DEFAULT_DEBOUNCE_INTERVAL = 100;
const MEASURE_TOLERANCE = 0.5;
const CollapsibleTabs = _ref => {
  let {
    children,
    selected,
    onSelect,
    onLastVisibleIndexChange,
    moreClassName,
    moreActiveClassName,
    morePopupClassName,
    morePopupBeforeEnd,
    morePopupItemClassName,
    initialVisibleItems
  } = _ref;
  const [sizes, setSizes] = useState({
    tabs: [],
    more: undefined
  });
  const [lastVisibleIndex, setLastVisibleIndex] = useState(null);
  const elements = {
    sizes,
    lastVisibleIndex
  };
  const [preparedElements, setPreparedElements] = useState({
    visible: [],
    hidden: []
  });
  const measureRef = useRef(null);
  const selectedIndex = useMemo(() => {
    var _children$filter$find;
    return (_children$filter$find = children.filter(tab => tab.props.alwaysHidden !== true).findIndex(tab => tab.props.id === selected)) !== null && _children$filter$find !== void 0 ? _children$filter$find : null;
  }, [children, selected]);
  const visibleElements = useMemo(() => {
    let items;
    if (preparedElements.ready) {
      items = preparedElements.visible;
    } else {
      items = initialVisibleItems ? children.filter(item => item.props.alwaysHidden !== true).slice(0, initialVisibleItems) : [];
    }
    return getTabTitles({
      items,
      selected,
      onSelect
    });
  }, [initialVisibleItems, children, preparedElements.ready, preparedElements.visible, onSelect, selected]);
  const hiddenElements = useMemo(() => {
    if (preparedElements.ready) {
      return preparedElements.hidden;
    } else if (initialVisibleItems) {
      return children.filter(item => !visibleElements.some(visibleItem => visibleItem.props.child === item));
    } else {
      return [];
    }
  }, [children, preparedElements.hidden, preparedElements.ready, visibleElements, initialVisibleItems]);
  const adjustTabs = useCallback(entry => {
    const containerWidth = entry.contentRect.width;
    const {
      tabs: tabsSizes,
      more = 0
    } = elements.sizes;
    let renderMore = children.some(tab => tab.props.alwaysHidden);
    const tabsToRender = [];
    let filledWidth = renderMore ? more !== null && more !== void 0 ? more : 0 : 0;
    for (let i = 0; i < tabsSizes.length; i++) {
      if (filledWidth + tabsSizes[i] < containerWidth + MEASURE_TOLERANCE) {
        filledWidth += tabsSizes[i];
        tabsToRender.push(tabsSizes[i]);
      } else {
        break;
      }
    }
    if (tabsToRender.length < tabsSizes.length && !renderMore) {
      for (let i = tabsToRender.length - 1; i >= 0; i--) {
        if (filledWidth + more < containerWidth + MEASURE_TOLERANCE) {
          filledWidth += more;
          renderMore = true;
          break;
        } else {
          filledWidth -= tabsToRender[i];
          tabsToRender.pop();
        }
      }
    }
    if (selectedIndex > tabsToRender.length - 1) {
      const selectedWidth = tabsSizes[selectedIndex];
      for (let i = tabsToRender.length - 1; i >= 0; i--) {
        if (filledWidth + selectedWidth < containerWidth + MEASURE_TOLERANCE) {
          filledWidth += selectedWidth;
          break;
        } else {
          filledWidth -= tabsToRender[i];
          tabsToRender.pop();
        }
      }
    }
    const newLastVisibleIndex = tabsToRender.length - 1;
    if (elements.lastVisibleIndex !== newLastVisibleIndex) {
      setLastVisibleIndex(newLastVisibleIndex);
      onLastVisibleIndexChange === null || onLastVisibleIndexChange === void 0 || onLastVisibleIndexChange(newLastVisibleIndex);
    }
  }, [children, elements.lastVisibleIndex, elements.sizes, onLastVisibleIndexChange, selectedIndex]);
  // Prepare list of visible and hidden elements
  useEffect(() => {
    const timeout = setTimeout(() => {
      var _elements$lastVisible2;
      const res = children.reduce((accumulator, tab) => {
        var _elements$lastVisible;
        if (tab.props.alwaysHidden !== true && accumulator.visible.length - 1 < ((_elements$lastVisible = elements.lastVisibleIndex) !== null && _elements$lastVisible !== void 0 ? _elements$lastVisible : 0)) {
          accumulator.visible.push(tab);
        } else {
          accumulator.hidden.push(tab);
        }
        return accumulator;
      }, {
        visible: [],
        hidden: [],
        ready: elements.lastVisibleIndex !== null
      });
      if (selectedIndex > ((_elements$lastVisible2 = elements.lastVisibleIndex) !== null && _elements$lastVisible2 !== void 0 ? _elements$lastVisible2 : 0)) {
        const selectedItem = children.find(tab => !tab.props.alwaysHidden && tab.props.id === selected);
        if (selectedItem != null) {
          res.visible.push(selectedItem);
        }
      }
      const allVisibleTheSame = res.visible.length === preparedElements.visible.length && res.visible.every((item, index) => item === preparedElements.visible[index]);
      const allHiddenTheSame = res.hidden.length === preparedElements.hidden.length && res.hidden.every((item, index) => item === preparedElements.hidden[index]);
      if (!allVisibleTheSame || !allHiddenTheSame || preparedElements.ready !== res.ready) {
        fastdom.mutate(() => setPreparedElements(res));
      }
    }, DEFAULT_DEBOUNCE_INTERVAL);
    return () => {
      clearTimeout(timeout);
    };
  }, [children, elements.lastVisibleIndex, preparedElements, selected, selectedIndex]);
  // Get list of all possibly visible elements to render in a measure container
  const childrenToMeasure = useMemo(() => {
    const items = children.filter(tab => tab.props.alwaysHidden !== true);
    return getTabTitles({
      items,
      tabIndex: -1
    });
  }, [children]);
  // Initial measure for tabs and more button sizes
  useEffect(() => {
    if (measureRef.current == null) {
      return undefined;
    }
    const measureTask = fastdom.measure(() => {
      var _container$children, _moreButton$offsetWid;
      const container = measureRef.current;
      const descendants = [...((_container$children = container === null || container === void 0 ? void 0 : container.children) !== null && _container$children !== void 0 ? _container$children : [])];
      const moreButton = descendants.pop();
      let moreButtonWidth = (_moreButton$offsetWid = moreButton === null || moreButton === void 0 ? void 0 : moreButton.offsetWidth) !== null && _moreButton$offsetWid !== void 0 ? _moreButton$offsetWid : 0;
      const {
        marginLeft: moreButtonMarginLeft = '0',
        marginRight: moreButtonMarginRight = '0'
      } = moreButton ? getComputedStyle(moreButton) : {};
      moreButtonWidth += +moreButtonMarginLeft.replace('px', '') + +moreButtonMarginRight.replace('px', '');
      const tabsWidth = descendants.map(node => {
        const {
          marginLeft,
          marginRight
        } = getComputedStyle(node);
        const width = node.getBoundingClientRect().width;
        return width + +marginLeft.replace('px', '') + +marginRight.replace('px', '');
      });
      const newSummaryWidth = tabsWidth.reduce((acc, curr) => acc + curr, 0);
      const oldSummaryWidth = elements.sizes.tabs.reduce((acc, curr) => acc + curr, 0);
      if (elements.sizes.more !== moreButtonWidth || newSummaryWidth !== oldSummaryWidth) {
        fastdom.mutate(() => setSizes({
          more: moreButtonWidth,
          tabs: tabsWidth
        }));
      }
    });
    return () => {
      fastdom.clear(measureTask);
    };
  }, [children, elements.sizes.more, elements.sizes.tabs]);
  // Start observers to listen resizing and mutation
  useEffect(() => {
    if (measureRef.current === null) {
      return undefined;
    }
    let measureTask = () => {};
    const resizeObserver = new ResizeObserver(entries => {
      entries.forEach(entry => {
        fastdom.clear(measureTask);
        measureTask = fastdom.mutate(() => adjustTabs(entry));
      });
    });
    resizeObserver.observe(measureRef.current);
    return () => {
      fastdom.clear(measureTask);
      resizeObserver.disconnect();
    };
  }, [adjustTabs]);
  const isAdjusted = elements.lastVisibleIndex !== null && preparedElements.ready === true || initialVisibleItems;
  const className = classNames(styles.titles, styles.autoCollapse, isAdjusted && styles.adjusted);
  return jsxs("div", {
    className: styles.autoCollapseContainer,
    children: [jsxs("div", {
      className: classNames(className, styles.rendered),
      children: [visibleElements, jsx(MoreButton, {
        moreClassName: moreClassName,
        moreActiveClassName: moreActiveClassName,
        morePopupClassName: morePopupClassName,
        morePopupBeforeEnd: morePopupBeforeEnd,
        morePopupItemClassName: morePopupItemClassName,
        items: hiddenElements,
        selected: selected,
        onSelect: onSelect
      })]
    }), jsxs("div", {
      ref: measureRef,
      className: classNames(className, styles.measure),
      children: [childrenToMeasure, jsx(FakeMoreButton, {
        hasActiveChildren: hiddenElements.some(item => item.props.alwaysHidden && item.props.id === selected),
        moreClassName: moreClassName,
        moreActiveClassName: moreActiveClassName
      })]
    })]
  });
};
var CollapsibleTabs$1 = /*#__PURE__*/memo(CollapsibleTabs);

export { CollapsibleTabs, CollapsibleTabs$1 as default };
