import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import closeIcon from '@jetbrains/icons/close-12px';
import Icon from '../icon/icon.js';
import { Button } from '../button/button.js';
import { ControlsHeight } from '../global/controls-height.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../link/clickableLink.js';
import '../_helpers/button__classes.js';

var styles = {"tag":"tag_rui_2f49 resetButton_rui_8bff","withRemove":"withRemove_rui_2f49","container":"container_rui_2f49","focused":"focused_rui_2f49","content":"content_rui_2f49 ellipsis_rui_8bff","disabled":"disabled_rui_2f49","remove":"remove_rui_2f49","removeIcon":"removeIcon_rui_2f49","icon":"icon_rui_2f49","avatarContainer":"avatarContainer_rui_2f49","customIcon":"customIcon_rui_2f49","avatarIcon":"avatarIcon_rui_2f49"};

/**
 * @name Tag
 */
class Tag extends PureComponent {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      focused: false
    });
    _defineProperty(this, "onDocumentClick", event => {
      if (this.tagNode) {
        this.setState({
          focused: this.tagNode === event.target
        });
      }
    });
    _defineProperty(this, "tagNode", void 0);
    _defineProperty(this, "tagRef", el => {
      this.tagNode = el;
    });
  }
  componentDidUpdate(prevProps) {
    if (this.props.focused !== prevProps.focused) {
      this.setState({
        focused: this.props.focused
      });
    }
    if (this.state.focused) {
      var _this$tagNode;
      (_this$tagNode = this.tagNode) === null || _this$tagNode === void 0 || _this$tagNode.focus();
    }
    this.setDocumentClickListener(this.state.focused);
  }
  componentWillUnmount() {
    this.setDocumentClickListener(false);
    this.setState({
      focused: false
    });
  }
  setDocumentClickListener(setListener) {
    if (setListener) {
      document.addEventListener('click', this.onDocumentClick);
    } else {
      document.removeEventListener('click', this.onDocumentClick);
    }
  }
  renderCustomIcon() {
    if (this.props.rgTagIcon) {
      return jsx(Icon, {
        className: styles.icon,
        title: this.props.rgTagTitle,
        glyph: this.props.rgTagIcon
      });
    }
    return null;
  }
  _renderImageElement(avatarSrc) {
    const classes = classNames({
      [styles.customIcon]: this.props.icon,
      [styles.avatarIcon]: avatarSrc
    });
    return jsx("img", {
      alt: avatarSrc ? 'Avatar' : 'Icon',
      className: classes,
      src: avatarSrc || this.props.icon
    });
  }
  renderImage() {
    if (this.props.icon && !this.props.avatar) {
      return this._renderImageElement();
    }
    return null;
  }
  renderAvatar() {
    if (this.props.avatar) {
      return jsx("span", {
        className: styles.avatarContainer,
        children: this._renderImageElement(this.props.avatar)
      });
    }
    return null;
  }
  renderRemoveIcon() {
    if (!this.props.readOnly && this.props.onRemove) {
      return jsx(Button, {
        title: "Remove",
        icon: closeIcon,
        "data-test": "ring-tag-remove",
        className: styles.remove,
        iconClassName: styles.removeIcon,
        onClick: this.props.onRemove,
        style: {
          '--ring-secondary-color': this.props.textColor
        },
        height: ControlsHeight.M
      });
    }
    return null;
  }
  render() {
    const classes = classNames('ring-js-shortcuts', styles.tag, {
      [styles.focused]: this.state.focused,
      [styles.disabled]: this.props.disabled,
      [styles.withRemove]: !this.props.readOnly && this.props.onRemove
    }, this.props.className);
    const {
      backgroundColor,
      textColor,
      render
    } = this.props;
    return jsxs("span", {
      className: styles.container,
      children: [render({
        'data-test': 'ring-tag',
        className: classes,
        ref: this.tagRef,
        onClick: this.props.onClick,
        style: {
          backgroundColor,
          color: textColor
        },
        disabled: this.props.disabled,
        children: jsxs(Fragment, {
          children: [this.renderAvatar(), this.renderCustomIcon(), this.renderImage(), jsx("span", {
            className: styles.content,
            children: this.props.children
          })]
        })
      }), this.renderRemoveIcon()]
    });
  }
}
_defineProperty(Tag, "defaultProps", {
  onClick: () => {},
  readOnly: false,
  disabled: false,
  focused: false,
  render: props => jsx("button", _objectSpread2({
    type: "button"
  }, props))
});

export { Tag as default };
