import { _ as _defineProperty } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { PureComponent } from 'react';
import classNames from 'classnames';
import getEventKey from '../global/get-event-key.js';
import Select from '../select/select.js';
import TagsList from '../tags-list/tags-list.js';
import Caret from '../caret/caret.js';
import memoize from '../global/memoize.js';
import rerenderHOC from '../global/rerender-hoc.js';
import { S as Size, i as inputStyles } from '../_helpers/input.js';
import { ControlsHeightContext } from '../global/controls-height.js';
import getUID from '../global/get-uid.js';
import { ControlLabel } from '../control-label/control-label.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.trim.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/close-12px';
import 'dequal';
import '../dropdown/dropdown.js';
import '../global/data-tests.js';
import '../global/typescript-utils.js';
import '../_helpers/anchor.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-12px-down';
import 'util-deprecate';
import '../icon/icon.js';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.string.starts-with.js';
import 'core-js/modules/es.weak-map.js';
import '../link/clickableLink.js';
import '../_helpers/button__classes.js';
import '../avatar/avatar.js';
import 'core-js/modules/es.string.includes.js';
import '../global/url.js';
import 'core-js/modules/es.string.match.js';
import '../global/dom.js';
import 'core-js/modules/es.string.split.js';
import '../_helpers/avatar-info.js';
import '../avatar/avatar-size.js';
import '../avatar/fallback-avatar.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../popup/popup.js';
import 'react-dom';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../popup/popup.target.js';
import '../list/list.js';
import 'core-js/modules/es.symbol.description.js';
import 'react-virtualized/dist/es/List';
import 'react-virtualized/dist/es/AutoSizer';
import 'react-virtualized/dist/es/WindowScroller';
import 'react-virtualized/dist/es/CellMeasurer';
import 'memoize-one';
import '../global/create-stateful-context.js';
import '../list/list__item.js';
import '../checkbox/checkbox.js';
import '@jetbrains/icons/checkmark-12px';
import '@jetbrains/icons/remove-12px';
import '../global/composeRefs.js';
import '../control-help/control-help.js';
import '../link/link.js';
import '../_helpers/link.js';
import '../_helpers/list.js';
import '../list/consts.js';
import '../list/list__classes.js';
import '../list/list__custom.js';
import '../list/list__title.js';
import '../list/list__separator.js';
import '../list/list__hint.js';
import '../global/fuzzy-highlight.js';
import '../i18n/i18n-context.js';
import '../i18n/i18n.js';
import '../select/select__popup.js';
import '@jetbrains/icons/search';
import '../loader-inline/loader-inline.js';
import '../shortcuts/shortcuts-hoc.js';
import '../text/text.js';
import '../_helpers/select__filter.js';
import '../tag/tag.js';

var styles = {"outerContainer":"outerContainer_rui_e356","container":"container_rui_e356","input":"input_rui_e356","tagsInput":"tagsInput_rui_1a67","tagsInputDisabled":"tagsInputDisabled_rui_1a67","tagsInputFocused":"tagsInputFocused_rui_1a67","tagsList":"tagsList_rui_1a67","tagsSelect":"tagsSelect_rui_1a67"};

function noop() {}
/**
 * @name Tags Input
 */
const POPUP_VERTICAL_SHIFT = 2;
class TagsInput extends PureComponent {
  constructor(props) {
    var _this;
    super(props);
    _this = this;
    _defineProperty(this, "state", {
      tags: [],
      prevTags: null,
      suggestions: [],
      loading: true,
      focused: false,
      query: '',
      activeIndex: 0
    });
    _defineProperty(this, "ngModelStateField", void 0);
    _defineProperty(this, "id", this.props.id || getUID('ring-tags-list-'));
    _defineProperty(this, "node", void 0);
    _defineProperty(this, "nodeRef", node => {
      this.node = node;
    });
    _defineProperty(this, "input", void 0);
    _defineProperty(this, "caret", void 0);
    _defineProperty(this, "focusInput", () => {
      var _this$getInputNode;
      (_this$getInputNode = this.getInputNode()) === null || _this$getInputNode === void 0 || _this$getInputNode.focus();
    });
    _defineProperty(this, "focus", () => {
      this.focusInput();
    });
    _defineProperty(this, "addTag", tag => {
      var _this$select, _this$select2;
      if (tag == null) {
        return;
      }
      const isUniqueTag = this.state.tags.filter(item => tag.key === item.key).length === 0;
      (_this$select = this.select) === null || _this$select === void 0 || _this$select.clear();
      (_this$select2 = this.select) === null || _this$select2 === void 0 || _this$select2.filterValue('');
      if (isUniqueTag) {
        this.setState(prevState => ({
          tags: prevState.tags.concat([tag])
        }));
        this.props.onAddTag({
          tag
        });
        this.setActiveIndex();
      }
    });
    _defineProperty(this, "clickHandler", event => {
      var _this$select3;
      if (event.target !== this.node && event.target.parentElement !== this.node) {
        return;
      }
      (_this$select3 = this.select) === null || _this$select3 === void 0 || _this$select3._clickHandler();
    });
    _defineProperty(this, "filterExistingTags", suggestions => {
      const tagsMap = new Map(this.state.tags.map(tag => [tag.key, tag]));
      return suggestions.filter(suggestion => !tagsMap.has(suggestion.key));
    });
    _defineProperty(this, "loadSuggestions", function () {
      let query = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
      return _this.setState({
        loading: true,
        query
      }, async () => {
        try {
          const suggestionsResult = _this.props.dataSource({
            query
          });
          const allSuggestions = Array.isArray(suggestionsResult) ? suggestionsResult : await suggestionsResult;
          const suggestions = _this.filterExistingTags(allSuggestions);
          if (_this.node && query === _this.state.query) {
            _this.setState({
              suggestions,
              loading: false
            });
          }
        } catch (e) {
          _this.setState({
            loading: false
          });
        }
      });
    });
    _defineProperty(this, "_focusHandler", () => {
      this.setActiveIndex(null);
      this.setState({
        focused: true
      });
    });
    _defineProperty(this, "_blurHandler", () => {
      this.setState({
        focused: false
      });
    });
    _defineProperty(this, "selectTag", moveForward => {
      const activeIndex = typeof this.state.activeIndex === 'number' ? this.state.activeIndex : this.state.tags.length + 1;
      let newActiveIndex = activeIndex + (moveForward ? 1 : -1);
      if (newActiveIndex >= this.state.tags.length) {
        newActiveIndex = this.state.tags.length - 1;
      } else if (newActiveIndex < 0) {
        newActiveIndex = 0;
      }
      if (this.state.activeIndex !== newActiveIndex) {
        this.setActiveIndex(newActiveIndex);
      }
    });
    _defineProperty(this, "handleKeyDown", event => {
      var _this$select4;
      const key = getEventKey(event);
      const isInputFocused = () => {
        var _this$getInputNode$ta, _this$getInputNode2;
        return event.target instanceof Element && event.target.matches((_this$getInputNode$ta = (_this$getInputNode2 = this.getInputNode()) === null || _this$getInputNode2 === void 0 ? void 0 : _this$getInputNode2.tagName) !== null && _this$getInputNode$ta !== void 0 ? _this$getInputNode$ta : '');
      };
      if (key === ' ' && this.props.allowAddNewTags) {
        var _this$getInputNode3;
        event.stopPropagation();
        const value = (_this$getInputNode3 = this.getInputNode()) === null || _this$getInputNode3 === void 0 ? void 0 : _this$getInputNode3.value;
        if (value != null && value !== '') {
          this.handleTagCreation(value);
        }
        return true;
      }
      if ((_this$select4 = this.select) !== null && _this$select4 !== void 0 && (_this$select4 = _this$select4._popup) !== null && _this$select4 !== void 0 && _this$select4.isVisible()) {
        return true;
      }
      if (key === 'ArrowLeft') {
        if (this.getInputNode() && this.caret != null && Number(this.caret.getPosition()) > 0) {
          return true;
        }
        this.selectTag();
        return false;
      }
      if (key === 'ArrowRight' && !isInputFocused()) {
        if (this.state.activeIndex === this.state.tags.length - 1) {
          if (!this.props.disabled) {
            var _this$getInputNode4;
            (_this$getInputNode4 = this.getInputNode()) === null || _this$getInputNode4 === void 0 || _this$getInputNode4.focus();
            this.setActiveIndex();
          }
        } else {
          this.selectTag(true);
        }
        return false;
      }
      if (!this.props.disabled) {
        var _this$getInputNode5;
        if (key === 'Backspace' && !((_this$getInputNode5 = this.getInputNode()) !== null && _this$getInputNode5 !== void 0 && _this$getInputNode5.value)) {
          var _this$select5;
          event.preventDefault();
          const tagsLength = this.state.tags.length;
          (_this$select5 = this.select) === null || _this$select5 === void 0 || _this$select5._hidePopup(true); // otherwise confirmation may be overlapped by popup
          this.onRemoveTag(this.state.tags[tagsLength - 1]);
          return false;
        }
        if ((key === 'Delete' || key === 'Backspace') && this.state.activeIndex != null && this.state.tags[this.state.activeIndex]) {
          this.onRemoveTag(this.state.tags[this.state.activeIndex]).then(() => this.selectTag(true));
          return false;
        }
      }
      return true;
    });
    _defineProperty(this, "handleClick", memoize(tag => () => {
      this.setActiveIndex(this.state.tags.indexOf(tag));
    }));
    _defineProperty(this, "handleRemove", memoize(tag => () => this.onRemoveTag(tag)));
    _defineProperty(this, "handleTagCreation", label => {
      this.addTag({
        key: label,
        label
      });
    });
    _defineProperty(this, "select", void 0);
    _defineProperty(this, "selectRef", el => {
      this.select = el;
    });
    this.ngModelStateField = TagsInput.ngModelStateField;
  }
  static getDerivedStateFromProps(_ref, _ref2) {
    let {
      tags
    } = _ref;
    let {
      prevTags
    } = _ref2;
    const nextState = {
      prevTags: tags
    };
    if (tags != null && tags !== prevTags) {
      Object.assign(nextState, {
        tags,
        activeIndex: tags.length
      });
    }
    return nextState;
  }
  componentDidMount() {
    if (this.props.autoOpen && !this.props.disabled) {
      var _this$select6;
      this.focusInput();
      this.loadSuggestions();
      (_this$select6 = this.select) === null || _this$select6 === void 0 || _this$select6._showPopup();
    }
  }
  getInputNode() {
    if (!this.input) {
      var _this$select7;
      this.input = (_this$select7 = this.select) === null || _this$select7 === void 0 ? void 0 : _this$select7.filter;
      if (this.input) {
        this.caret = new Caret(this.input);
      }
    }
    return this.input;
  }
  setActiveIndex(activeIndex) {
    this.setState({
      activeIndex
    });
  }
  onRemoveTag(tagToRemove) {
    return Promise.resolve(this.props.onRemoveTag({
      tag: tagToRemove
    })).then(() => {
      const tags = this.state.tags.filter(tag => tag !== tagToRemove);
      if (this.node) {
        this.setState({
          tags
        });
        this.focusInput();
      }
      return tags;
    }, this.focusInput);
  }
  render() {
    const {
      focused,
      tags,
      activeIndex
    } = this.state;
    const {
      disabled,
      canNotBeEmpty,
      allowAddNewTags,
      filter,
      size,
      labelType,
      height = this.context,
      label
    } = this.props;
    const classes = classNames(styles.tagsInput, [inputStyles["size".concat(size)]], [inputStyles["height".concat(height)]], {
      [styles.tagsInputDisabled]: disabled,
      [styles.tagsInputFocused]: focused
    }, this.props.className);
    return jsxs("div", {
      // it transfers focus to input
      role: "presentation",
      className: classes,
      onKeyDown: this.handleKeyDown,
      onClick: this.clickHandler,
      ref: this.nodeRef,
      children: [label && jsx(ControlLabel, {
        htmlFor: this.id,
        disabled: disabled,
        type: labelType,
        children: label
      }), jsx(TagsList, {
        tags: tags,
        activeIndex: activeIndex,
        disabled: disabled,
        canNotBeEmpty: canNotBeEmpty,
        handleRemove: this.handleRemove,
        className: styles.tagsList,
        tagClassName: styles.tag,
        handleClick: this.handleClick,
        customTagComponent: this.props.customTagComponent,
        children: jsx(Select, {
          id: this.id,
          ref: this.selectRef,
          size: Select.Size.AUTO,
          type: Select.Type.INPUT_WITHOUT_CONTROLS,
          inputPlaceholder: this.props.placeholder,
          data: this.state.suggestions,
          className: classNames(styles.tagsSelect),
          onSelect: this.addTag,
          onFocus: this._focusHandler,
          onBlur: this._blurHandler,
          renderOptimization: this.props.renderOptimization,
          add: allowAddNewTags ? {
            prefix: 'Add new tag'
          } : undefined,
          onAdd: allowAddNewTags ? this.handleTagCreation : undefined,
          filter: filter,
          maxHeight: this.props.maxPopupHeight,
          minWidth: this.props.minPopupWidth,
          top: POPUP_VERTICAL_SHIFT,
          loading: this.state.loading,
          onFilter: this.loadSuggestions,
          onBeforeOpen: this.loadSuggestions,
          onKeyDown: this.handleKeyDown,
          disabled: this.props.disabled,
          loadingMessage: this.props.loadingMessage,
          notFoundMessage: this.props.notFoundMessage,
          hint: this.props.hint
        })
      })]
    });
  }
}
_defineProperty(TagsInput, "defaultProps", {
  dataSource: noop,
  onAddTag: noop,
  onRemoveTag: noop,
  customTagComponent: null,
  maxPopupHeight: 500,
  minPopupWidth: 360,
  canNotBeEmpty: false,
  disabled: false,
  autoOpen: false,
  renderOptimization: true,
  allowAddNewTags: false,
  filter: {
    fn: () => true
  },
  placeholder: 'Select an option',
  size: Size.M
});
_defineProperty(TagsInput, "ngModelStateField", 'tags');
_defineProperty(TagsInput, "contextType", ControlsHeightContext);
const RerenderableTagsInput = rerenderHOC(TagsInput);

export { RerenderableTagsInput, TagsInput as default };
