import { _ as _defineProperty, a as _objectWithoutProperties, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { createContext, Component } from 'react';
import classNames from 'classnames';
import Popup from '../popup/popup.js';
import { Listeners } from '../global/dom.js';
import joinDataTestAttributes from '../global/data-tests.js';
import scheduleRAF from '../global/schedule-raf.js';
import { T as Theme, a as ThemeProvider } from '../_helpers/theme.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.replace.js';
import 'react-dom';
import '../global/get-uid.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import 'core-js/modules/es.string.split.js';
import '../popup/popup.target.js';

var styles = {"tooltip":"tooltip_rui_64ba","inheritedTheme":"inheritedTheme_rui_64ba","long":"long_rui_64ba"};

const _excluded = ["children", "data-test", "title", "delay", "theme", "selfOverflowOnly", "popupProps", "long"];
const scheduleScroll = scheduleRAF();
const TooltipContext = /*#__PURE__*/createContext(undefined);
/**
 * @name Tooltip
 */
class Tooltip extends Component {
  constructor() {
    super(...arguments);
    _defineProperty(this, "state", {
      showPopup: false,
      showNestedPopup: false
    });
    _defineProperty(this, "timeout", void 0);
    _defineProperty(this, "listeners", new Listeners());
    _defineProperty(this, "containerNode", void 0);
    _defineProperty(this, "containerRef", el => {
      this.containerNode = el;
    });
    _defineProperty(this, "tryToShowPopup", () => {
      const {
        delay,
        title
      } = this.props;
      if (!title) {
        return;
      }
      if (delay) {
        this.timeout = window.setTimeout(this.showPopup, delay);
      } else {
        this.showPopup();
      }
    });
    _defineProperty(this, "showPopup", () => {
      var _this$context;
      if (this.props.selfOverflowOnly) {
        const {
          containerNode
        } = this;
        // rare cases when containerNode is null are possible;
        // probably the collision is due to the asynchronous nature of the code,
        // i.e. this code runs after the component is unmounted,
        // although at first glance it looks unlikely.
        if (!containerNode) {
          return;
        }
        // inline element?
        if (containerNode.clientWidth === 0 && containerNode.clientHeight === 0) {
          return;
        }
        if (containerNode.scrollWidth <= containerNode.clientWidth && containerNode.scrollHeight <= containerNode.clientHeight) {
          return;
        }
      }
      (_this$context = this.context) === null || _this$context === void 0 || _this$context.onNestedTooltipShow();
      this.setState({
        showPopup: true
      });
    });
    _defineProperty(this, "hidePopup", () => {
      var _this$context2;
      clearTimeout(this.timeout);
      (_this$context2 = this.context) === null || _this$context2 === void 0 || _this$context2.onNestedTooltipHide();
      this.setState({
        showPopup: false
      });
    });
    _defineProperty(this, "hideIfMovedOutsidePopup", ev => {
      var _this$popup;
      if (!('relatedTarget' in ev) || (_this$popup = this.popup) !== null && _this$popup !== void 0 && (_this$popup = _this$popup.container) !== null && _this$popup !== void 0 && _this$popup.contains(ev.relatedTarget)) {
        return;
      }
      this.hidePopup();
    });
    _defineProperty(this, "popup", void 0);
    _defineProperty(this, "popupRef", el => {
      this.popup = el;
    });
    _defineProperty(this, "onNestedTooltipShow", () => {
      this.setState({
        showNestedPopup: true
      });
    });
    _defineProperty(this, "onNestedTooltipHide", () => {
      this.setState({
        showNestedPopup: false
      });
    });
  }
  componentDidMount() {
    if (this.props.title) {
      this.addListeners();
    }
  }
  componentDidUpdate(prevProps) {
    if (!prevProps.title && this.props.title) {
      this.addListeners();
    } else if (prevProps.title && !this.props.title) {
      this.listeners.removeAll();
    }
  }
  componentWillUnmount() {
    clearTimeout(this.timeout);
    this.listeners.removeAll();
  }
  addListeners() {
    if (this.containerNode != null) {
      this.listeners.add(this.containerNode, 'mouseenter', this.tryToShowPopup);
      this.listeners.add(this.containerNode, 'mouseleave', ev => {
        var _this$popup2;
        if (ev.relatedTarget && (_this$popup2 = this.popup) !== null && _this$popup2 !== void 0 && (_this$popup2 = _this$popup2.container) !== null && _this$popup2 !== void 0 && _this$popup2.contains(ev.relatedTarget)) {
          return;
        }
        this.hidePopup();
      });
    }
    this.listeners.add(document, 'scroll', () => scheduleScroll(this.hidePopup), {
      passive: true
    });
  }
  render() {
    const _this$props = this.props,
      {
        children,
        'data-test': dataTest,
        title,
        delay,
        theme,
        selfOverflowOnly,
        popupProps,
        long
      } = _this$props,
      restProps = _objectWithoutProperties(_this$props, _excluded);
    const ariaProps = typeof title === 'string' && !!title ? {
      'aria-label': title,
      role: 'tooltip'
    } : {};
    const {
      onNestedTooltipShow,
      onNestedTooltipHide
    } = this;
    const popup = jsx(Popup, _objectSpread2(_objectSpread2({
      trapFocus: false,
      anchorElement: this.containerNode,
      hidden: !this.state.showPopup || this.state.showNestedPopup,
      onCloseAttempt: this.hidePopup,
      maxHeight: 400,
      attached: false,
      onMouseOut: this.hideIfMovedOutsidePopup,
      top: 4,
      dontCloseOnAnchorClick: true,
      ref: this.popupRef
    }, popupProps), {}, {
      className: classNames(styles.tooltip, {
        [styles.long]: long,
        [styles.inheritedTheme]: theme === 'inherit'
      }, popupProps === null || popupProps === void 0 ? void 0 : popupProps.className),
      children: title
    }));
    return jsx(TooltipContext.Provider, {
      value: {
        onNestedTooltipShow,
        onNestedTooltipHide
      },
      children: jsxs("span", _objectSpread2(_objectSpread2(_objectSpread2({}, ariaProps), restProps), {}, {
        ref: this.containerRef,
        "data-test": joinDataTestAttributes('ring-tooltip', dataTest),
        "data-test-title": typeof title === 'string' ? title : undefined,
        children: [children, theme === 'inherit' ? popup : jsx(ThemeProvider, {
          theme: theme,
          passToPopups: true,
          WrapperComponent: props => jsx("span", _objectSpread2({}, props)),
          children: popup
        })]
      }))
    });
  }
}
_defineProperty(Tooltip, "defaultProps", {
  title: '',
  selfOverflowOnly: false,
  theme: Theme.DARK,
  popupProps: {}
});
_defineProperty(Tooltip, "PopupProps", Popup.PopupProps);
_defineProperty(Tooltip, "contextType", TooltipContext);

export { Tooltip as default };
