import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { forwardRef, useRef, useState, useImperativeHandle, useCallback } from 'react';
import classNames from 'classnames';
import attachmentIcon from '@jetbrains/icons/attachment';
import Icon from '../icon/icon.js';
import '../_helpers/_rollupPluginBabelHelpers.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';

var styles = {"upload":"upload_rui_5064","success":"success_rui_5064","error":"error_rui_5064","dragOver":"dragOver_rui_5064","disabled":"disabled_rui_5064","invisibleFileInput":"invisibleFileInput_rui_5064","attachmentIcon":"attachmentIcon_rui_5064"};

const Upload = /*#__PURE__*/forwardRef(function Upload(_ref, ref) {
  let {
    children,
    className,
    onFilesSelected,
    onFilesRejected,
    validate = () => true,
    variant = 'empty',
    multiple,
    renderIcon = () => jsx(Icon, {
      className: styles.attachmentIcon,
      glyph: attachmentIcon
    }),
    accept,
    disabled
  } = _ref;
  const fileInputRef = useRef(null);
  const [dragOver, setDragOver] = useState(false);
  useImperativeHandle(ref, () => ({
    openFilePicker: () => {
      var _fileInputRef$current;
      return (_fileInputRef$current = fileInputRef.current) === null || _fileInputRef$current === void 0 ? void 0 : _fileInputRef$current.click();
    }
  }), []);
  const handleSelectedFiles = useCallback(files => {
    if (!files.length) {
      return;
    }
    const rejected = files.filter(file => !validate(file));
    if (rejected.length > 0) {
      onFilesRejected === null || onFilesRejected === void 0 || onFilesRejected(files);
      return;
    }
    onFilesSelected(files);
  }, [onFilesRejected, onFilesSelected, validate]);
  const onDragEnter = useCallback(() => setDragOver(true), []);
  const onDragOver = useCallback(e => e.preventDefault(), []);
  const onDragLeave = useCallback(() => setDragOver(false), []);
  const onInputChange = useCallback(() => {
    var _fileInputRef$current2;
    setDragOver(false);
    if ((_fileInputRef$current2 = fileInputRef.current) !== null && _fileInputRef$current2 !== void 0 && _fileInputRef$current2.files) {
      handleSelectedFiles(Array.from(fileInputRef.current.files));
    }
  }, [handleSelectedFiles]);
  return jsxs("div", {
    className: classNames(className, styles.upload, {
      [styles.disabled]: disabled,
      [styles.dragOver]: dragOver,
      [styles.success]: variant === 'success',
      [styles.error]: variant === 'error'
    }),
    "data-test": "ring-upload",
    children: [jsx("input", {
      onDragEnter: onDragEnter,
      onDragOver: onDragOver,
      onDragLeave: onDragLeave,
      disabled: disabled,
      ref: fileInputRef,
      "data-test": "ring-file-input",
      multiple: multiple,
      accept: accept,
      onChange: onInputChange,
      type: "file",
      autoComplete: "off",
      "aria-label": "file-picker",
      className: styles.invisibleFileInput
    }), renderIcon(), children]
  });
});

export { Upload, Upload as default };
