import { _ as _defineProperty, b as _objectSpread2 } from '../_helpers/_rollupPluginBabelHelpers.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { createRoot } from 'react-dom/client';
import LocalStorage from '../storage/storage__local.js';
import alertService from '../alert-service/alert-service.js';
import Link from '../link/link.js';
import Alert from '../alert/alert.js';
import Group from '../group/group.js';
import { ControlsHeightContext, getGlobalControlsHeight } from '../global/controls-height.js';
import UserAgreement from './user-agreement.js';
import '../global/get-uid.js';
import 'core-js/modules/es.regexp.to-string.js';
import '../alert/container.js';
import 'react';
import 'react-dom';
import 'classnames';
import '../global/data-tests.js';
import '../link/clickableLink.js';
import '../_helpers/link.js';
import '@jetbrains/icons/exception';
import '@jetbrains/icons/checkmark';
import '@jetbrains/icons/warning';
import '@jetbrains/icons/close';
import '../icon/icon.js';
import 'util-deprecate';
import '../icon/icon__constants.js';
import '../_helpers/icon__svg.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.starts-with.js';
import '../global/memoize.js';
import 'core-js/modules/es.weak-map.js';
import '../loader-inline/loader-inline.js';
import '../global/dom.js';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.string.split.js';
import '../button/button.js';
import '@jetbrains/icons/chevron-down';
import '@jetbrains/icons/chevron-12px-down';
import '../_helpers/button__classes.js';
import '../_helpers/theme.js';
import '../popup/popup.target.js';
import '../popup/popup.js';
import '../global/schedule-raf.js';
import '../shortcuts/shortcuts.js';
import '../shortcuts/core.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.string.match.js';
import 'combokeys';
import '../global/sniffer.js';
import 'sniffr';
import '../tab-trap/tab-trap.js';
import '../popup/position.js';
import 'core-js/modules/es.array.sort.js';
import '../popup/popup.consts.js';
import '../dialog/dialog.js';
import '../island/island.js';
import '../island/adaptive-island-hoc.js';
import '../global/linear-function.js';
import '../_helpers/island.js';
import '../island/header.js';
import '../island/content.js';
import 'element-resize-detector';
import '../_helpers/dialog__body-scroll-preventer.js';
import 'scrollbar-width';
import '../panel/panel.js';
import '../i18n/i18n-context.js';
import '../i18n/i18n.js';

const GUEST_SESSION_KEY = 'end-user-agreement-consent';
const ONE_HOUR = 60 * 60 * 1000; // eslint-disable-line @typescript-eslint/no-magic-numbers
const storageKey = 'userAgreementKey';
const showMessage = 'userAgreementShow';
const hideMessage = 'userAgreementHide';
const DEFAULT_CONSENT = {
  accepted: false,
  majorVersion: 0,
  minorVersion: 0
};
const DEFAULT_AGREEMENT = {
  enabled: false,
  majorVersion: 0,
  minorVersion: 0,
  requiredForREST: false,
  content: ''
};
function ignorePostponedCatchReason(reason) {
  if (reason === 'Postponed') {
    return;
  }
  throw reason;
}
class UserAgreementService {
  constructor(config) {
    var _this = this;
    _defineProperty(this, "config", void 0);
    _defineProperty(this, "_dialogPromise", null);
    _defineProperty(this, "_alertPromise", null);
    _defineProperty(this, "tabId", Math.random());
    _defineProperty(this, "interval", ONE_HOUR);
    _defineProperty(this, "container", document.createElement('div'));
    _defineProperty(this, "reactRoot", createRoot(this.container));
    _defineProperty(this, "storage", new LocalStorage());
    _defineProperty(this, "checkingPromise", null);
    _defineProperty(this, "guest", false);
    _defineProperty(this, "userAgreement", DEFAULT_AGREEMENT);
    _defineProperty(this, "userConsent", DEFAULT_CONSENT);
    _defineProperty(this, "intervalId", void 0);
    _defineProperty(this, "startChecking", () => {
      this.intervalId = window.setInterval(() => this.checkConsentAndShowDialog().catch(ignorePostponedCatchReason), this.interval);
      window.addEventListener('storage', this.onStorageEvent);
      this.checkConsentAndShowDialog().catch(ignorePostponedCatchReason);
    });
    _defineProperty(this, "stopChecking", () => {
      clearInterval(this.intervalId);
      window.removeEventListener('storage', this.onStorageEvent);
      this.hideDialog();
    });
    _defineProperty(this, "onStorageEvent", event => {
      if (event.key === storageKey && event.newValue != null) {
        const {
          tabId,
          command
        } = JSON.parse(event.newValue);
        if (tabId !== this.tabId) {
          if (command === showMessage) {
            this.checkConsentAndShowDialog(true);
          } else if (command === hideMessage) {
            this.hideDialogAndAlert(true);
          }
        }
      }
    });
    _defineProperty(this, "_notifyAboutShowing", () => {
      localStorage.setItem(storageKey, JSON.stringify({
        command: showMessage,
        tabId: this.tabId
      }));
    });
    _defineProperty(this, "_notifyAboutHiding", () => {
      localStorage.setItem(storageKey, JSON.stringify({
        command: hideMessage,
        tabId: this.tabId
      }));
    });
    _defineProperty(this, "getUserAgreement", async () => {
      this.userAgreement = (await this.config.getUserAgreement()) || DEFAULT_AGREEMENT;
      return this.userAgreement;
    });
    _defineProperty(this, "getUserConsent", async () => {
      const {
        guest,
        endUserAgreementConsent
      } = await this.config.getUserConsent();
      this.guest = guest;
      if (guest) {
        this.userConsent = (await this.storage.get(GUEST_SESSION_KEY)) || this.userConsent;
      } else {
        this.userConsent = endUserAgreementConsent || this.userConsent;
      }
      return this.userConsent;
    });
    _defineProperty(this, "checkConsentAndShowDialog", async withoutNotifications => {
      if (await this.checkConsent()) {
        return this.hideDialogAndAlert(withoutNotifications);
      } else {
        return this.showDialogOrAlert(withoutNotifications);
      }
    });
    _defineProperty(this, "checkConsent", async () => {
      if (!this.checkingPromise) {
        this.checkingPromise = Promise.all([this.getUserAgreement(), this.getUserConsent()]);
      }
      const [userAgreement, userConsent] = await this.checkingPromise;
      this.checkingPromise = null;
      const {
        enabled,
        majorVersion: actualVersion
      } = userAgreement;
      const {
        accepted,
        majorVersion: acceptedVersion
      } = userConsent;
      return !enabled || accepted && actualVersion === acceptedVersion;
    });
    _defineProperty(this, "alertKey", void 0);
    _defineProperty(this, "showAlert", withoutNotifications => {
      if (this._alertPromise) {
        return this._alertPromise;
      }
      this._alertPromise = new Promise((resolve, reject) => {
        const {
          userAgreement,
          reviewNow,
          remindLater
        } = this.config.translations || {};
        const onRemind = () => {
          this.hideDialogAndAlert(withoutNotifications);
          reject('Postponed');
        };
        const onReview = async () => {
          await this.showDialog(true, false, {
            onRemindLater: onRemind
          });
          this.hideAlert(withoutNotifications);
          resolve();
        };
        const message = jsxs(Group, {
          children: [jsx("span", {
            children: userAgreement || 'User Agreement'
          }), jsx(Link, {
            onClick: onReview,
            "data-test": "review",
            children: reviewNow || 'Review now'
          }), jsx(Link, {
            onClick: onRemind,
            "data-test": "later",
            children: remindLater || 'Remind me later'
          })]
        });
        this.alertKey = alertService.addAlert(message, Alert.Type.WARNING, 0, {
          closeable: false
        });
      });
      if (!withoutNotifications) {
        this._notifyAboutShowing();
      }
      return this._alertPromise;
    });
    _defineProperty(this, "hideAlert", withoutNotifications => {
      const {
        onRemindLater
      } = this.config;
      alertService.remove(this.alertKey);
      this.alertKey = null;
      this._alertPromise = null;
      if (onRemindLater) {
        onRemindLater();
      }
      if (!withoutNotifications) {
        this._notifyAboutHiding();
      }
    });
    _defineProperty(this, "showDialog", function (withoutNotifications) {
      let preview = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      let restOptions = arguments.length > 2 ? arguments[2] : undefined;
      const {
        translations,
        onDialogShow
      } = _this.config;
      const {
        content
      } = _this.userAgreement;
      const show = true;
      if (!_this._dialogPromise) {
        _this._dialogPromise = new Promise((resolve, reject) => {
          const onAccept = async () => {
            await _this.onAccept();
            resolve();
          };
          const onDecline = async () => {
            await _this.onDecline();
            reject();
          };
          const onClose = _this.hideDialogAndAlert;
          const props = _objectSpread2({
            children: content,
            show,
            onAccept,
            onDecline,
            onClose,
            translations,
            preview
          }, restOptions);
          _this.reactRoot.render(jsx(ControlsHeightContext.Provider, {
            value: getGlobalControlsHeight(),
            children: jsx(UserAgreement, _objectSpread2({}, props))
          }));
          if (onDialogShow) {
            onDialogShow();
          }
        });
        if (!withoutNotifications) {
          _this._notifyAboutShowing();
        }
      }
      return _this._dialogPromise;
    });
    _defineProperty(this, "hideDialog", withoutNotifications => {
      const {
        onDialogHide
      } = this.config;
      this.reactRoot.render(null);
      if (onDialogHide) {
        onDialogHide();
        this._dialogPromise = null;
        if (!withoutNotifications) {
          this._notifyAboutHiding();
        }
      }
    });
    _defineProperty(this, "showDialogOrAlert", (withoutNotifications, preview, restOptions) => {
      if (this.guest && !this.userAgreement.requiredForREST) {
        return this.showAlert(withoutNotifications);
      }
      return this.showDialog(withoutNotifications, preview, restOptions);
    });
    _defineProperty(this, "hideDialogAndAlert", withoutNotifications => {
      this.hideAlert(withoutNotifications);
      this.hideDialog(withoutNotifications);
    });
    _defineProperty(this, "onAccept", async () => {
      const {
        setUserConsent,
        onAccept
      } = this.config;
      if (this.guest) {
        const {
          majorVersion,
          minorVersion
        } = this.userAgreement;
        this.userConsent = {
          majorVersion,
          minorVersion,
          accepted: true
        };
        await this.storage.set(GUEST_SESSION_KEY, this.userConsent);
      } else {
        this.userConsent = await setUserConsent();
      }
      this.hideDialog();
      if (onAccept) {
        onAccept();
      }
    });
    _defineProperty(this, "onDecline", () => {
      const {
        onDecline
      } = this.config;
      this.hideDialog();
      if (onDecline) {
        onDecline();
      }
    });
    if (!config) {
      throw new Error('Please pass a config to UserAgreementService');
    }
    if (!config.getUserAgreement) {
      throw new Error('Please pass a "getUserAgreement" option to UserAgreementService');
    }
    if (!config.getUserConsent) {
      throw new Error('Please pass a "getUserConsent" option to UserAgreementService');
    }
    if (!config.setUserConsent) {
      throw new Error('Please pass a "setUserConsent" option to UserAgreementService');
    }
    this.config = config;
    this.interval = config.interval || this.interval;
  }
}

export { UserAgreementService as default, hideMessage, showMessage };
